package org.eso.ohs.phase2.orang;

import java.util.Date;

import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.WeatherConstraint;
import org.eso.ohs.dfs.WindInfo;
import org.eso.ohs.phase2.visibility.VisicalcOptions;

public interface OrangModel {
	
	public final static int OB_SELECTION_TELESCOPE = 0;	
	public final static int OB_SELECTION_INSTRUMENT = 1;	
	public final static int OB_SELECTION_QUEUES = 2;	
	public static final int OB_SELECTION_WRONG = -1;
	
	public static final int DEFAULT_MOON_ELV = 0;
	public static final int DEFAULT_SUN_ELEV = -18;
    public static final int DEFAULT_WIND = 0;
    public static final String DEFAULT_SEEING = "0.2";
	public static final int DEFAULT_INTERVAL = 10; 	
			
	public int getObSelectionType();	
	public void setObSelectionType(int type);
	
	public String getSelectedTelescope();		
	public void setSelectedTelescope(String telescope);
	
	public String getSelectedInstrument();
	public void setSelectedInstrument(String instrument);
	
	
	public String [] getSelectedQueues();
	public void setSelectedQueues(String [] queues);
	
	/**
	 * @return String[] the current today queues  assigned the current telescope
	 */
	public String[] getQueuesTelescopes(); 
	
	
	/**
	 * @return String[]  the current today queues  assigned the current instrument
	 */
	public String[] getQueuesInstruments();
	
	
	
	public Date getStartDate();	
	public void setStartDate(Date date);
	public Date getEndDate();
	public void setEndDate(Date date);
	public String getEndISODate();	
	public String getStartISODate();
	public int getStepInterval();
	public void setStepInterval(int step);
	
	
      public void setSeeing(String seeing);
      public void setMaxSeeing(String seeing);
      public String getSeeing();
      public String getMaxSeeing();
      
    public void setWind(int wind);
    public int getWind();
	
	public void setSkyTransp(String skyTransp);
	public String getSkyTransp();

       public void setSeeingEnabled(boolean enabled);
        public boolean getSeeingEnabled();
  
    public void setWindEnabled(boolean enabled);
    public boolean getWindEnabled();
    
	public void setSkyTranspEnabled(boolean enabled);
	public boolean getSkyTranspEnabled();

      public Seeing getSeeingValue(); 
      public Seeing getMaxSeeingValue(); 
         public WindInfo getWindValue(); 
	public SkyTransparency getSkyTranspValue();	
	
	
	
	/**
	 * @return WeatherConstraint the current weather constraint 
	 */
	public WeatherConstraint getWeatherConstraint();  
		
	
	public int getAirmassValuePercentage();		
	public void setAirmassValuePercentage(int value);
		
	public boolean isFliConstrain();
	public void setFliConstrain(boolean b);
	
	public int getFliRelaxPercentage();
	public void setFliRelaxPercentage(int value);
	
	public boolean isMoonAngleConstrain();
	public void setMoonAngleConstrain(boolean b);
	
	public boolean isTimeIntervalConstrain();
	public void setTimeIntervalConstrain(boolean b);
	
	public boolean isSidTimeIntervalConstrain();
	public void setSidTimeIntervalConstrain(boolean b);
	
	public int getMoonAngleDegree();	
	public void setMoonAngleDegree(int value);
	
	public boolean isMoonDistanceConstrain();
	public void setMoonDistanceConstrain(boolean b);
	
	public int getMoonRelaxPercentage();
	public void setMoonRelaxPercentage(int value);
	
	public boolean isSunAngleConstrain();
	public void setSunAngleConstrain(boolean b);
	
	public int getSunAngleDegree();
	public void setSunAngleDegree(int value);
	
	public double getLatitude();	
	public double getLongitude();
	
	
	public boolean isTelescopeZenithConstraint();
	public void setTelescopeZenithConstraint(boolean b);
	
   
    public void setExecutionCheck(boolean verifyExecution) ;
    public boolean isExecutionConstraint() ;

	public VisicalcOptions getVisiCalcOptions();	
	
	public void setRankingAlgorithm(String name);
	public String getRankingAlgorithm();	
	
	public String getTimelineAlgorithm();
	public void setTimelineAlgorithm(String name);
	
	public String getUser();	
	public boolean isSuperUser();
	
		
	
	/**
	 * @return boolean 
	 */
	public boolean obSelectionChanged();
	
	/**
	 * @return boolean
	 */
	public boolean weatherChanged();
	
	/**
	 * @return boolean
	 */
	public boolean visibilityConstraintChanged();
	
	/**
	 * @return boolean
	 */
	public boolean rankAlgorithmChanged();
	
	/**
	 * @return boolean
	 */
	public boolean timelineAlgorithmChanged();
	

	
	public OrangWidgetsData getWidgetsData();

	public boolean optionsChanged();
	
	
	/*
	 * Calculates the Local sidereal time, for the starting date/time
	 * of the OB visibility computation
	 */
	public double getLST();
	
	
	/*
	 * REturns the FLI, for the starting date/time
	 * of the OB visibility computation
	 */
	public double getFLI();
	
	
	public void setSidTimeConstrain(boolean b);
	
	
	public void setDuration(String string);
	
}