#include <jni.h>
#include "JNIWrapper.h"
#include <stdio.h>
#include "slalib.h"

/**
 * SLALIB s l a A o p - Apparent to observed place, for optical sources
 * distant from the solar system.
 * @version 15 August 2001
 * @author C. Mavros for ESO
 *
 * @param rap     double  geocentric apparent right ascension
 * @param dap     double  geocentric apparent declination
 * @param date    double  UTC date/time (Modified Julian Date, JD-2400000.5)
 * @param dut     double  delta UT:  UT1-UTC (UTC seconds)
 * @param elongm  double  mean longitude of the observer (radians, east +ve)
 * @param phim    double  mean geodetic latitude of the observer (radians)
 * @param hm      double  observer's height above sea level (metres)
 * @param xp      double  polar motion x-coordinate (radians)
 * @param yp      double  polar motion y-coordinate (radians)
 * @param tdk     double  local ambient temperature (DegK; std=273.155)
 * @param pmb     double  local atmospheric pressure (mB; std=1013.25)
 * @param rh      double  local relative humidity (in the range 0.0-1.0)
 * @param wl      double  effective wavelength (micron, e.g. 0.55)
 * @param tlr     double  tropospheric lapse rate (DegK/metre, e.g. 0.0065)
 *
 * @return Array of doubles [5]
 * @return aob observed azimuth (radians: N=0,E=90)
 * @return zob observed zenith distance (radians)
 * @return hob observed Hour Angle (radians)
 * @return dob observed Declination (radians)
 * @return rob observed Right Ascension (radians)
*/

JNIEXPORT jdoubleArray JNICALL Java_org_eso_ohs_phase2_visiplot_JNIWrapper_jnislaAop
  (JNIEnv * env, jobject jobj, jdouble jrap, jdouble jdap, jdouble jdate, 
  jdouble jdut, jdouble jelongm, jdouble jphim, jdouble jhm, 
  jdouble jxp, jdouble jyp, jdouble jtdk, jdouble jpmb, 
  jdouble jrh, jdouble jwl, jdouble jtlr){

    double jp[5];
    int i = 0;
    jdoubleArray jpArray = (*env)->NewDoubleArray(env,5);
    double aob, zob, hob, dob, rob;
    double *aob_ptr, *zob_ptr, *hob_ptr, *dob_ptr, *rob_ptr;

    aob_ptr = &aob;
    zob_ptr = &zob;
    hob_ptr = &hob;
    dob_ptr = &dob;
    rob_ptr = &rob;

    /** Call slaAop */
    slaAop (jrap, jdap, jdate, jdut, jelongm, jphim, jhm, jxp, jyp, 
      jtdk, jpmb, jrh, jwl, jtlr, aob_ptr, zob_ptr, hob_ptr,
      dob_ptr, rob_ptr);

    /* Pass pointers to array */
    jp[0]=aob;
    jp[1]=zob;
    jp[2]=hob;
    jp[3]=dob;
    jp[4]=rob;

    (*env)->SetDoubleArrayRegion(env, jpArray, 0, 5, jp);

    return(jpArray);

}
