#include <jni.h>
#include "JNIWrapper.h"
#include <stdio.h>
#include "slalib.h"


/**
 *  SLALIB s l a M a p -  Transform star RA,Dec from mean place to geocentric apparent.
 *
 *  The reference frames and timescales used are post IAU 1976.
 *
 *  References:
 *     1984 Astronomical Almanac, pp B39-B41.
 *     (also Lederle & Schwan, Astron. Astrophys. 134, 1-6, 1984)
 *
 *  @version 25 July 2001
 *  @author C. Mavros for ESO
 *
 *  @param rm,dm    double     mean RA,Dec (rad)
 *  @param pr,pd    double     proper motions:  RA,Dec changes per Julian year
 *  @param px       double     parallax (arcsec)
 *  @param rv       double     radial velocity (km/sec, +ve if receding)
 *  @param eq       double     epoch and equinox of star data (Julian)
 *  @param date     double     TDB for apparent place (JD-2400000.5)
 *
 *  @ return an Array of doubles [2] RA, Dec apparent (rad)
 *
 */

JNIEXPORT jdoubleArray JNICALL Java_org_eso_ohs_phase2_visiplot_JNIWrapper_jnislaMap
  (JNIEnv * env , jobject obj, jdouble jRM, jdouble jDM, jdouble jPR, jdouble jPX,
    jdouble jRV, jdouble jEQ, jdouble JDate, jdouble jPD){

    double jp[2];
    int i = 0;
    jdoubleArray jpArray = (*env)->NewDoubleArray(env,2);
    double ra, dec;
    double *ra_ptr, *dec_ptr;

    ra_ptr = &ra;
    dec_ptr = &dec;

    /** Call slaMap */
    slaMap (  jRM, jDM, jPR, jPD, jPX, jRV, jEQ, JDate, ra_ptr, dec_ptr );

    /** Pass pointers to array */
    jp[0]=ra;
    jp[1]=dec;

    (*env)->SetDoubleArrayRegion(env, jpArray, 0, 3, jp);

    return(jpArray);

}
