#include <jni.h>
#include "JNIWrapper.h"
#include <stdio.h>
#include "slalib.h"

/**
 *   SLALIB s l a R d p l a n - Approximate topocentric apparent RA,Dec of a
 *  planet, and its angular diameter.
 *
 *  @version 25 July 2001
 *  @author C. Mavros for ESO
 *
 *  @param date  double  Modified Julian Date of observation (JD - 2400000.5)
 *  @param np    int     planet: 3 = Moon 0 = Sun
 *  @param elong double  observer's east longitude (radians)
 *  @param phi   double  observer's geodetic latitude (radians)
 *  @return Array of doubles  [3] RA (topocentric apparent, radians)
 *  @return Dec (topocentric apparent, radians)
 *  @return Diam angular diameter (equatorial, radians)
 */

JNIEXPORT jdoubleArray JNICALL Java_org_eso_ohs_phase2_visiplot_JNIWrapper_jnislaRdplan
  (JNIEnv * env, jobject jobj, jdouble jDATE, jint jNP, jdouble jLONG, jdouble jPHI){

    double jp[3];
    int i = 0;
    jdoubleArray jpArray = (*env)->NewDoubleArray(env,3);
    double ra, dec, diam;
    double *ra_ptr, *dec_ptr, *diam_ptr;

    ra_ptr = &ra;
    dec_ptr = &dec;
    diam_ptr = &diam;

    /** Call slaRdplan */
    slaRdplan(jDATE, jNP, jLONG, jPHI, ra_ptr, dec_ptr, diam_ptr);

    /* Pass pointers to array */
    jp[0]=ra;
    jp[1]=dec;
    jp[2]=diam;

    (*env)->SetDoubleArrayRegion(env, jpArray, 0, 3, jp);

    return(jpArray);

}
