package org.eso.ohs.phase2.visiplot;

import java.awt.Color;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.Paramfile;

/**
 * This class handles loading and saving of Visiplot Options in and from a file
 * 
 * @author pnunes 16.01.2006
 *
 */
public class VisiplotOptionsHandler {

	// Basic input data
	// The following two keys are needed but are not
	// saved in settings files, or read in from settings files.
	// If you add to these keys, remember to add to the setDefault()
	// method to set the default setting for this key.
	public static String startTimeKey         = "VISIPLOT.START.TIME";
	public static String endTimeKey           = "VISIPLOT.END.TIME";
	public static int defaultPrecision    = 10;
	public static String dataQueryIntKey      = "VISIPLOT.QUERY.INTERVAL";		
	
	
	// group identical plots
	public static String groupPlotKey         = "VISIPLOT.PLOT.GROUP.OBS";
				
	public static final String header_ =
		"OHS software Visibility Plot configuration preferences";
		
	
	// Printer name
	public static String printerName          = "VISIPLOT.PRINTER";
	
	
	//	 Plot constraints	
	public static String useOBValue = "read from OB";		
	public static String fliConKey            = "VISIPLOT.FLI";
	public static String fliConEnabledKey     = "VISIPLOT.FLI.ENABLED";	
	public static String airMassConKey        = "VISIPLOT.AIR.MASS";
	public static String airMassConEnabledKey = "VISIPLOT.AIR.MASS.ENABLED";
	public static String moonDistConKey       = "VISIPLOT.MOON.DIST";
	public static String moonDistConEnabledKey = "VISIPLOT.MOON.DIST.ENABLED";
	public static String obexecConKey         = "VISIPLOT.OBEXEC.TIME.ENABLED";
	public static String obtimeIntervalConKey = "VISIPLOT.OBTIME.INTERVAL.ENABLED";
	public static String obSiderealTimeIntervalConKey = "VISIPLOT.OB.ST.TIME.INTERVAL.ENABLED";
	
	public static String sunConKey            = "VISIPLOT.SUN";
	public static String sunConEnabledKey     = "VISIPLOT.SUN.ENABLED";
	public static double defaultSunCon     = -18;	
	public static String zenithConKey 		 = "VISIPLOT.ZENITH.ENABLED";
	

	// plot header info - all either "T" or "F"
    public static String reportSeeingKey      = "VISIPLOT.REPORT.SEEING";
    public static String reportWindKey      = "VISIPLOT.REPORT.WIND";
	public static String reportFLIKey         = "VISIPLOT.REPORT.FLI";
	public static String reportMoonDistKey    = "VISIPLOT.REPORT.MOONDIST";
	public static String reportSkyTransKey    = "VISIPLOT.REPORT.SKY";
	public static String reportAirMassKey     = "VISIPLOT.REPORT.AIRMASS";
	public static String reportRankKey        = "VISIPLOT.REPORT.RANK";
	public static String reportProgIdKey      = "VISIPLOT.REPORT.PROGID";
	public static String reportOBIdKey        = "VISIPLOT.REPORT.OBID";
	public static String reportRAKey          = "VISIPLOT.REPORT.RA";
	public static String reportDecKey         = "VISIPLOT.REPORT.DEC";
	public static String reportExecTimeKey    = "VISIPLOT.REPORT.EXECTIME";
	public static String reportStartTimeKey   = "VISIPLOT.REPORT.STARTTIME";

	
	
	//	 Sort plots by
	public static String sortPlotKey          = "VISIPLOT.PLOT.SORTBY";
	public static String defaultSortOrder  = VisiplotOptionsHandler.sortByProgid;
	
	// Sort options:
	// string values of constraints which can be used to
	// sort the visibility plots.
	public static String sortByProgid      = "program id";
	public static String sortByObid        = "OB-Id";
	public static String sortByRank        = "rank class";
	public static String sortBySeeing      = "seeing";
	public static String sortByFli         = "frac lunar illum.";
	public static String sortByMoondist    = "moon distance";
	public static String sortBySkytrans    = "sky transp.";
	
	
	
	//	Chart colors
	public static String airMassLineColor = "VISIPLOT.AIR.MASS.LINE.COLOR";
	public static String airMassAreaColor = "VISIPLOT.AIR.MASS.AREA.COLOR";
	public static String airMassLabelsColor = "VISIPLOT.AIR.MASS.LABELS.COLOR";
	public static String fliLineColor = "VISIPLOT.FLI.LINE.COLOR";
	public static String fliAreaColor = "VISIPLOT.FLI.AREA.COLOR";
	public static String fliLabelsColor = "VISIPLOT.FLI.LABELS.COLOR";
	public static String moonDistLineColor = "VISIPLOT.MOON.DIST.LINE.COLOR";
	public static String moonDistAreaColor = "VISIPLOT.MOON.DIST.AREA.COLOR";
	public static String moonDistLabelsColor = "VISIPLOT.MOON.DIST.LABELS.COLOR";
	public static String obexecLineColor = "VISIPLOT.OBEXEC.LINE.COLOR";
	public static String obexecAreaColor = "VISIPLOT.OBEXEC.AREA.COLOR";
	public static String obexecLabelsColor = "VISIPLOT.OBEXEC.LABELS.COLOR";
	
	public static String obSiderealTimeIntervalLineColor = "VISIPLOT.OB.ST.TIME.INTERVAl.LINE.COLOR";
	public static String obSiderealTimeIntervalAreaColor = "VISIPLOT.OB.ST.TIME.INTERVAl.AREA.COLOR";
	public static String obSiderealTimeIntervalLabelsColor = "VISIPLOT.OB.ST.TIME.INTERVAl.LABELS.COLOR";	
	public static String sunLineColor = "VISIPLOT.SUN.LINE.COLOR";
	public static String sunAreaColor = "VISIPLOT.SUN.AREA.COLOR";
	public static String sunLabelsColor = "VISIPLOT.SUN.LABELS.COLOR";
	public static String timeLineColor = "VISIPLOT.TIME.LINE.COLOR";
	public static String timeAreaColor = "VISIPLOT.TIME.AREA.COLOR";
	public static String timeLabelsColor = "VISIPLOT.TIME.LABELS.COLOR";
	
	
	public static final String ST_TIME_INTERVALS_CHART_LABEL = "st time intervals";
	public static final String EXECUTION_TIME_CHART_LABEL = "execution time";
	public static final String TIME_INTERVALS_CHART_LABEL = "time intervals";
	public static final String SUN_ANGLE_CHART_LABEL = "sun angle";
	public static final String FRACTIONAL_LUNAR_ILL_CHART_LABEL = "fractional lunar ill.";
	public static final String MOON_DISTANCE_CHART_LABEL = "moon distance";
	public static final String AIR_MASS_CHART_LABEL = "air mass";
	

	
	/**
     * Update the store  with key/value pairs read from the
     * reader object parameter. All PAF parameters read are stored,
     * regardless of whether they are relevant to the options object or
     * not. Note that if start time or end time kezs are read, they
     * are ignored.
     *
     * @param the reader object containing the PAF format text parameters
     * @throws IOException if the reader format is not PAF format
     */

    public static VisiPlotOptions readFrom(Reader in) throws IOException {
        Paramfile paf = null;
        try {
            // read the file, up to 10000 lines.
            paf = new Paramfile(in, 10000);
        } catch (NotASCIIException e) {
            throw new IOException("Input format error: "
                    +e.getMessage());
        }
        
        
        // load the properties one by one
        VisiPlotOptions options = new VisiPlotOptions();
        
        
        // computation parameters
        String interval = paf.getValue(dataQueryIntKey);
        options.getVisicalcOptions().setInterval( getIntValue(interval)  );
        
        String group = paf.getValue(groupPlotKey);
        options.setGroupOBs( getBooleanValue(group) );
        
        String fli = paf.getValue(fliConKey);
        options.setFliValue(fli);
        
        String fliEnabled = paf.getValue(fliConEnabledKey);
        options.getVisicalcOptions().setCalcFli(getBooleanValue(fliEnabled));
                        
        String airmass = paf.getValue(airMassConKey);
        options.setAirmassValue(airmass);
        
        String airmassEnabled = paf.getValue(airMassConEnabledKey);
        options.getVisicalcOptions().setCalcAirmass(getBooleanValue(airmassEnabled));
                
        String moonDist = paf.getValue(moonDistConKey);
        options.setMoonDistanceValue(moonDist);
        
        String moonDistEnabled = paf.getValue(moonDistConEnabledKey);
        options.getVisicalcOptions().setCalcMoonDist(getBooleanValue(moonDistEnabled));
	
        String exectimeEnabled = paf.getValue(obexecConKey);
        options.getVisicalcOptions().setCalcExecTime(getBooleanValue(exectimeEnabled));
        
        String timeIntEnabled = paf.getValue(obtimeIntervalConKey);
        options.getVisicalcOptions().setCalcTimeInterval(getBooleanValue(timeIntEnabled));
        
        String sidTimeIntEnabled = paf.getValue(obSiderealTimeIntervalConKey);
        options.getVisicalcOptions().setCalcSiderealTimeInterval(getBooleanValue(sidTimeIntEnabled));
        
        String sun = paf.getValue(sunConKey);
        options.getVisicalcOptions().setSunConstraint(getDoubleValue(sun));
        
        String sunEnabled = paf.getValue(sunConEnabledKey);
        options.getVisicalcOptions().setCalcSun(getBooleanValue(sunEnabled));
        
        String zenithEnabled = paf.getValue(zenithConKey);
        options.getVisicalcOptions().setCalcZenithInvisibility(getBooleanValue(zenithEnabled));

        
        // report header
        String reportSeeing = paf.getValue(reportSeeingKey);
        options.setReportShowSeeing(getBooleanValue(reportSeeing));
        // report header
        String reportWind = paf.getValue(reportWindKey);
        options.setReportShowSeeing(getBooleanValue(reportWind));
        
        String reportFLI = paf.getValue(reportFLIKey);
        options.setReportShowSeeing(getBooleanValue(reportFLI));
        
        String reportMoonDist = paf.getValue(reportMoonDistKey);
        options.setReportShowSeeing(getBooleanValue(reportMoonDist));
        
        String reportSkyTrans = paf.getValue(reportSkyTransKey);
        options.setReportShowSeeing(getBooleanValue(reportSkyTrans));
        
        String reportAirMass = paf.getValue(reportAirMassKey);
        options.setReportShowSeeing(getBooleanValue(reportAirMass));
        
        String reportRank = paf.getValue(reportRankKey);
        options.setReportShowSeeing(getBooleanValue(reportRank));
        
        String reportProgId = paf.getValue(reportProgIdKey);
        options.setReportShowSeeing(getBooleanValue(reportProgId));
        
        String reportRA = paf.getValue(reportRAKey);
        options.setReportShowSeeing(getBooleanValue(reportRA));
        
        String reportDec = paf.getValue(reportDecKey);
        options.setReportShowSeeing(getBooleanValue(reportDec));
        
        String reportExecTime = paf.getValue(reportExecTimeKey);
        options.setReportShowSeeing(getBooleanValue(reportExecTime));
        
        String reportStartTime = paf.getValue(reportStartTimeKey);
        options.setReportShowSeeing(getBooleanValue(reportStartTime));
        
       
        
        // plot sorting 	
        String sortPlot = paf.getValue(sortPlotKey);
        options.setSortPlot(sortPlot);
                        
        // chart colors
        String airmassLine = paf.getValue(airMassLineColor);
        String airmassArea = paf.getValue(airMassAreaColor);
        String airmassLabels = paf.getValue(airMassLabelsColor);                           
        ChartColorItem item  = ChartColorItem.setChartColorItem(new ChartColorItem(AIR_MASS_CHART_LABEL, true, true),true,true,airmassLine,airmassArea,airmassLabels);
        options.setAirMassAreaColor(item.getAreaColor());
        options.setAirMassLineColor(item.getPerimeterColor());
        options.setAirMassLabelColor(item.getLabelsColor());
        
        
        String fliLine = paf.getValue(fliLineColor);
        String fliArea = paf.getValue(fliAreaColor);
        String fliLabels = paf.getValue(fliLabelsColor);                
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(FRACTIONAL_LUNAR_ILL_CHART_LABEL, true, true),true,true,fliLine,fliArea,fliLabels);
        options.setFliAreaColor(item.getAreaColor());
        options.setFliLineColor(item.getPerimeterColor());
        options.setFliLabelColor(item.getLabelsColor());
        
        
        String moonDistLine = paf.getValue(moonDistLineColor);
        String moonDistArea= paf.getValue(moonDistAreaColor);
        String moonDistLabels = paf.getValue(moonDistLabelsColor);                
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(MOON_DISTANCE_CHART_LABEL, true, true),true,true,moonDistLine,moonDistArea,moonDistLabels);
        options.setMoonDistAreaColor(item.getAreaColor());
        options.setMoonDistLineColor(item.getPerimeterColor());
        options.setMoonDistLabelColor(item.getLabelsColor());
        
        
        String obexecLine = paf.getValue(obexecLineColor);
        String obexecArea= paf.getValue(obexecAreaColor);
        String obexecLabels = paf.getValue(obexecLabelsColor);        
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(EXECUTION_TIME_CHART_LABEL, true, true),true,true,obexecLine,obexecArea,obexecLabels);
        options.setObexecAreaColor(item.getAreaColor());
        options.setObexecLineColor(item.getPerimeterColor());
        options.setObexecLabelColor(item.getLabelsColor());
       
        
        
        String sunLine = paf.getValue(sunLineColor);
        String sunArea = paf.getValue(sunAreaColor);
        String sunLabels = paf.getValue(sunLabelsColor);        
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(SUN_ANGLE_CHART_LABEL, true, true),true,true,sunLine,sunArea,sunLabels);
        options.setSunAreaColor(item.getAreaColor());
        options.setSunLineColor(item.getPerimeterColor());
        options.setSunLabelColor(item.getLabelsColor());
        
        
        String timeLine= paf.getValue(timeLineColor);
        String timeArea = paf.getValue(timeAreaColor);
        String timeLabels = paf.getValue(timeLabelsColor);        
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(TIME_INTERVALS_CHART_LABEL, true, true),true,true,timeLine,timeArea,timeLabels);
        options.setObTimeIntervalAreaColor(item.getAreaColor());
        options.setObTimeIntervalLineColor(item.getPerimeterColor());
        options.setObTimeIntervalLabelColor(item.getLabelsColor());                
        
        
        String obSiderealTimeIntervalLine = paf.getValue(obSiderealTimeIntervalLineColor);
        String obSiderealTimeIntervalArea = paf.getValue(obSiderealTimeIntervalAreaColor);
        String obSiderealTimeIntervalLabels = paf.getValue(obSiderealTimeIntervalLabelsColor);                
        item  = ChartColorItem.setChartColorItem(new ChartColorItem(ST_TIME_INTERVALS_CHART_LABEL, true, true),true,true,obSiderealTimeIntervalLine,obSiderealTimeIntervalArea,obSiderealTimeIntervalLabels);        
        options.setObSiderealTimeIntervalAreaColor(item.getAreaColor());
        options.setObSiderealTimeIntervalLineColor(item.getPerimeterColor());
        options.setObSiderealTimeIntervalLabelColor(item.getLabelsColor()); 

        return(options);
    }

    /**
     * Write the store out, in PAF format, to the Write object passed
     * in the call. Note that start time and end time keys are not saved.
     *
     * @param the output writer to be used to write the PAF output store
     * @throws IOException if the write failed.
     */

    public static void writeTo(Writer out, VisiPlotOptions options) throws IOException {
        Paramfile paramfile = new Paramfile();
        
        // computation parameters
        paramfile.put(dataQueryIntKey,options.getVisicalcOptions().getInterval()+"");
        paramfile.put(groupPlotKey,boolToStr(options.isGroupOBs()));
        paramfile.put(fliConKey,options.getFliValue());
        paramfile.put(fliConEnabledKey,boolToStr(options.getVisicalcOptions().calcFli()));
        paramfile.put(airMassConKey,options.getAirmassValue());
        paramfile.put(airMassConEnabledKey,boolToStr(options.getVisicalcOptions().calcAirmass()));
        paramfile.put(moonDistConKey,options.getMoonDistanceValue());
        paramfile.put(moonDistConEnabledKey,boolToStr(options.getVisicalcOptions().calcMoonDist()));
        paramfile.put(obexecConKey,boolToStr(options.getVisicalcOptions().calcExecTime()));
        paramfile.put(obtimeIntervalConKey,boolToStr(options.getVisicalcOptions().calcTimeInterval()));
        paramfile.put(obSiderealTimeIntervalConKey,boolToStr(options.getVisicalcOptions().calcSiderealTimeInterval()));
        paramfile.put(sunConKey,options.getSunConstraint());
        paramfile.put(sunConEnabledKey,boolToStr(options.getVisicalcOptions().calcSun()));
        paramfile.put(zenithConKey,boolToStr(options.getVisicalcOptions().calcZenithInvisibility()));

        
        // report header
        
        paramfile.put(reportSeeingKey,boolToStr(options.reportShowSeeing()));
        paramfile.put(reportWindKey,boolToStr(options.reportShowWind()));
        paramfile.put(reportFLIKey,boolToStr(options.reportShowFLI()));
        paramfile.put(reportMoonDistKey,boolToStr(options.reportShowMoonDist()));
        paramfile.put(reportSkyTransKey,boolToStr(options.reportShowSkyTrans()));
        paramfile.put(reportAirMassKey,boolToStr(options.reportShowAirMass()));
        paramfile.put(reportRankKey,boolToStr(options.reportShowRank()));
        paramfile.put(reportProgIdKey,boolToStr(options.reportShowProgId()));
        paramfile.put(reportRAKey,boolToStr(options.reportShowRA()));
        paramfile.put(reportDecKey,boolToStr(options.reportShowDec()));
        paramfile.put(reportExecTimeKey,boolToStr(options.reportShowExecTime()));
        paramfile.put(reportStartTimeKey,boolToStr(options.reportShowStartTime()));

        
        // plot sorting 	
        paramfile.put(sortPlotKey,options.getSortPlot());
        
        
        
        // chart colors        
        paramfile.put(airMassLineColor,getComma(options.getAirMassLineColor()));
        paramfile.put(airMassAreaColor,getComma(options.getAirMassAreaColor()));
        paramfile.put(airMassLabelsColor,getComma(options.getAirMassLabelColor()));
        
 
        paramfile.put(fliLineColor,getComma(options.getFliLineColor()));
        paramfile.put(fliAreaColor,getComma(options.getFliAreaColor()));
        paramfile.put(fliLabelsColor,getComma(options.getFliLabelColor()));
        
        paramfile.put(moonDistLineColor,getComma(options.getMoonDistLineColor()));
        paramfile.put(moonDistAreaColor,getComma(options.getMoonDistAreaColor()));
        paramfile.put(moonDistLabelsColor,getComma(options.getMoonDistLabelColor()));
        

        paramfile.put(obexecLineColor,getComma(options.getObexecLineColor()));
        paramfile.put(obexecAreaColor,getComma(options.getObexecAreaColor()));
        paramfile.put(obexecLabelsColor,getComma(options.getObexecLabelColor()));

       

        paramfile.put(sunLineColor,getComma(options.getSunLineColor()));
        paramfile.put(sunAreaColor,getComma(options.getSunAreaColor()));
        paramfile.put(sunLabelsColor,getComma(options.getSunLabelColor()));
        


        paramfile.put(timeLineColor,getComma(options.getObTimeIntervalLineColor()));
        paramfile.put(timeAreaColor,getComma(options.getObTimeIntervalAreaColor()));
        paramfile.put(timeLabelsColor,getComma(options.getObTimeIntervalLabelColor()));
        
        paramfile.put(obSiderealTimeIntervalLineColor,getComma(options.getObSiderealTimeIntervalLineColor()));
        paramfile.put(obSiderealTimeIntervalAreaColor,getComma(options.getObSiderealTimeIntervalAreaColor()));
        paramfile.put(obSiderealTimeIntervalLabelsColor,getComma(options.getObSiderealTimeIntervalLabelColor()));
        

        paramfile.writeTo(out, VisiplotOptionsHandler.header_);
    }
    
    
    private static String getComma(Color color) {
    	String result;
		int red = color.getRed();
		int green = color.getGreen();
		int blue = color.getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}

	/**
     * @return the double value of a value string, or throw a number
     * format exception if it can not be parsed.
     * @throws NumberFormatException if the value is null or not a number.
     */

    private static double getDoubleValue(String value) throws NumberFormatException {
        
        if (value == null) {
            throw new NumberFormatException("null value");
        }
        return Double.valueOf(value).doubleValue();
    }
	
    /**
     * @return the integer value of a value string, or throw a number
     * format exception if it can not be parsed.
     * @throws NumberFormatException if the value is null or not a number.
     */

    private static int getIntValue(String value) throws NumberFormatException {
        
        if (value == null) {
            throw new NumberFormatException("null value");
        }
        return Integer.parseInt(value);
    }


    
    private static boolean getBooleanValue(String value) throws  IllegalArgumentException {
    	
    	boolean res = false;
    	if (value != null) {
    		if ( value.equals("T") )
    			res = true;
    		else
    			if (value.equals("F"))
    				res = false;
    			else
    				throw new IllegalArgumentException("Invalid argument");
    	}
    	return(res);
    }
    
    
    
    
	private static String boolToStr(boolean val) {
		if (val)
			return "T";
		else return "F";
	}
	
	
	

}
