//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 1996 European Southern Observatory
// LICENSE
//
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jeremy Maxfield
// SUBSYSTEM: Dataflow core
// CI NUMBER: DFS-DFC-XXX
//
// $Revision: 1.2 $
// $Log: DfcDataDefinition.h,v $
// Revision 1.2  2004/10/26 15:23:54  nferguso
// Temporary check-in for repository move.
//
// Revision 1.1  2004/09/06 14:35:21  cguirao
// *** empty log message ***
//
// Revision 1.1  2004/09/02 14:35:20  cguirao
// *** empty log message ***
//
// Revision 1.1  2004/06/24 13:12:15  cguirao
// First check in
//
// Revision 1.2  1998/06/30 08:57:38  dataflow
// Removing $NAME
//
// Revision 1.1  1996/12/20 11:40:49  dfadmin
// Initial revision
//
//-----------------------------------------------------------------------------

#ifndef _DFC_DATA_DEFINITION_H_
#define _DFC_DATA_DEFINITION_H_

#include "DfsTValSlist.h"
#include <DfcParsable.h>
#include <DfcParseString.h>
#include <DfcComparison.h>
#include <DfcVariable.h>
#include <DfcVariableDefinition.h>

class DfcDataDefinition: public DfcParsable
{
   public:

     DfcDataDefinition();

     DfcDataDefinition(const DfcDataDefinition& other);
     DfcDataDefinition& operator=(const DfcDataDefinition& other);
     int  operator==(const DfcDataDefinition& other);

     DfsCString getType()const;
     DfsCString getName()const;

     DfsTValSlist<DfcVariable>   getVariables()const;
     DfsTValSlist<DfcComparison> getCriteria()const;
     DfsCString                  getCriteriaAsString()const;
     DfsTValSlist<DfcVariableDefinition>getPrimaryKeys()const;
     DfsTValSlist<DfcVariableDefinition> getColumns()const;

     DfsCString getDescription()const;

     bool parse(DfcParseString& inString);

     void testPrint(std::ostream& outStream)const;

   private:
     bool parseAsGeneral(DfcParseString& inString);
     bool parseAsFrame(DfcParseString& inString);
     bool parseAsTable(DfcParseString& inString);
     bool parseCriteria(DfcParseString& inString);
     bool parseVariables(DfcParseString& inString);
     bool parsePrimaryKeys(DfcParseString& inString);
     bool parseColumns(DfcParseString& inString);
     bool parseDescription(DfcParseString& inString);

     DfsCString VARIABLE_STRING;
     DfsCString CRITERIA_STRING;
     DfsCString DESCRIPTION_STRING;
     DfsCString PRIMARY_KEY_STRING;
     DfsCString COLUMNS_STRING;
     DfsCString GENERAL_STRING;
     DfsCString FRAME_STRING;
     DfsCString TABLE_STRING;

     DfsCString type;
     DfsCString name;
     DfsCString description;
      
     DfsTValSlist<DfcVariable>              variableList;
     DfsTValSlist<DfcComparison>            criteriaList;
     DfsTValSlist<DfcVariableDefinition>    primaryKeyList;
     DfsTValSlist<DfcVariableDefinition>    columnList;
};
#endif
