//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcDb.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.4  1999/06/23 14:52:46  adisaro
//validity management
//PAF content extraction
//
//Revision 1.3  1998/06/30 08:57:39  dataflow
//Removing $NAME
//
//Revision 1.2  1997/12/03 15:09:26  dfadmin
//Checkin for NTT Jan 98
//
//Revision 1.4  1997/07/22 08:47:03  mwiedmer
//add ref counting
//
//Revision 1.3  1997/04/11 13:39:49  mwiedmer
//use RWCString
//
//Revision 1.1  1997/04/10 09:14:50  dfadmin
//Initial revision
//
//Revision 1.2  1997/04/04 14:41:39  mwiedmer
//use mSQL as new database engine
//
//Revision 1.1  1997/03/18 10:56:55  mwiedmer
//make DfcDB become DfcDb,
//DfcDBManager to DfcDbManager, DfcIDF to DfcIdf
//and use environment variable to determine
//the DFC_DB_ROOT
//
//Revision 1.1  1996/12/13 14:22:58  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_DB_H
#define DFC_DB_H

// includes from this library
#include <DfcDbData.h>
// includes from other ESO libraries
#include <CorBase.h>
#include <CorRef.h>
#include <CorBool.h>
// includes from other libraries
// system includes
#include "DfsCString.h"
#include "TimeStamp.h"

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
class DfcDbSelection;

/// A database class.
class DfcDb : public CorBase {
  /**
  {\footnotesize $Id: DfcDb.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcDb.h>
  
    DfcDb myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class provides easy access to the ESO databases. It is a database that
  can be queried upon.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{DfcDbSelection}\\
  \Ref{DfcDbData}\\
  \Ref{DfcDbManager}\\
  \Ref{CorBase}\\
  */

public:

  /// 
  DfcDb(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    const DfsCString& aName = "DB",
    /** name of the database */
    ///
    int aSocket = -1
    /** socket number of the database */
  );


  ///
  virtual ~DfcDb();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  DfcDbData applySelection(
    /** {\large Purpose} \\
    Apply a selection criteria on the database.

    {\large Preconditions} \\
    \begin{itemize}
    \item aSelection must not be 0.
    \end{itemize}

    {\large Returns}\\
    This function returns the data that matches the selection.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    */
    ///
    const DfcDbSelection* aSelection
    /** selection to apply */
  );


  ///
  DfsCString getName() const;
  /** {\large Purpose} \\
  Query the name of the database.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the of this database.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  DfcDb(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDb& aRef
    /** the reference instance */
  );
    

  /// 
  DfcDb& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDb& aRef
    /** the reference instance */
  );
    

  ///
  void become(
    /** {\large Purpose} \\
    This function makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDb& aRef
    /** the reference instance */
  );
    



  ///
  void addTable(
    /** {\large Purpose} \\
    Create a table from a given dataset.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aName,
    /** name of the new table */
    ///
    DfcDbData aDataset
    /** a dataset */
  );


protected:

private:
  friend void utest_DfcDb(CorUnitTest& aTest);  // the unit test function
  DfsCString getDbVirtualTime();  		// gets the current database virtual time
  DfsCString addValidityCondition(DfsCString Tables, DfsCString VirtualTime);  		// compute the validity condition from the involved tables

  DfsCString  mName;        // name of the database
  DfsCString  mServer;
  int        mSocket;      // socket of this database
  CorBool    mInitialized; // valid instance?
  CorRef*    mRefCount;    // reference count
};

#endif // DFC_DB_H
