//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcDbManager.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.4  1998/06/30 08:57:39  dataflow
//Removing $NAME
//
//Revision 1.3  1998/06/03 15:50:43  pgrosbol
//replace explicit declaration of 'gethostname' by
//include of sys/unistd.h
//
//Revision 1.2  1997/12/03 15:09:26  dfadmin
//Checkin for NTT Jan 98
//
//Revision 1.6  1997/07/22 08:49:44  mwiedmer
//add env variable name to set default server
//
//Revision 1.5  1997/07/11 08:36:39  mwiedmer
//add isValid()
//
//Revision 1.4  1997/04/11 13:39:51  mwiedmer
//use RWCString
//
//Revision 1.1  1997/04/10 09:14:50  dfadmin
//Initial revision
//
//Revision 1.3  1997/04/07 08:59:04  mwiedmer
//use RW lists
//
//Revision 1.2  1997/04/04 14:41:40  mwiedmer
//use mSQL as new database engine
//
//Revision 1.1  1997/03/18 10:56:56  mwiedmer
//make DfcDB become DfcDb,
//DfcDBManager to DfcDbManager, DfcIDF to DfcIdf
//and use environment variable to determine
//the DFC_DB_ROOT
//
//Revision 1.2  1996/12/18 09:48:11  mwiedmer
//add getDbName() methods
//
//Revision 1.1  1996/12/13 14:23:17  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_DBMANAGER_H
#define DFC_DBMANAGER_H

// includes from this library
#include <DfcDb.h>
// includes from other ESO libraries
#include <CorBase.h>
#include <CorStream.h>
#include "DfsCString.h"
#include "DfsTValSlist.h"
// includes from other libraries
// system includes
#include <sys/unistd.h>

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
// extern "C" int gethostname(char *name, int namelen);

// The following environment variable defines the location of the the
// LOCAL database.
extern const char* dfcDbRootEnv;
extern const char* dfcDbServerEnv;

/// the main database manager
class DfcDbManager : public CorBase {
  /**
  {\footnotesize $Id: DfcDbManager.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcDbManager.h>
  
    DfcDbManager myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class is the main DB manager. It knows about all the databases the
  user could ever want.
  
  {\large CAUTIONS} \\
  There can only be one.

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  \Ref{CorBase}\\
  \Ref{DfcDb}\\
  */

public:

  /// 
  DfcDbManager(
    /** {\large Purpose} \\
    The c'tor. Connects to a given server and database.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aDatabase = "",
    /** database to connect to */
    ///
    const DfsCString& aServer = ""
    /** server to connect to */
  );
    

  ///
  virtual ~DfcDbManager();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  CorBool connect(
    /** {\large Purpose} \\
    Connect to a given server.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aServer = ""
    /** server to connect to */
  );
    

  ///
  DfcDb open(
    /** {\large Purpose} \\
    Open a specified database.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the specified database (which may be a new one).

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aName
    /** name of the database */
  );


  ///
  CorStream open(
    /** {\large Purpose} \\
    Open a specified file. In read mode the file will be searched for in
    the local cache first, then in the OLAS.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a stream associated to a file.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aName,
    /** name of the file */
    ///
    CorStream::StreamFlow aState
    /** state of the file */
  );


  ///
  CorBool isValid() const;
  /** {\large Purpose} \\
  Query if the database is valid (connection OK, etc).

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if the database is valid.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  static DfsCString getHostname();
  /** {\large Purpose} \\
  Query the hostname currently working on.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of the hostname.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual DfsCString getDbRoot();
  /** {\large Purpose} \\
  Query the database root directory.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation a directory.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:

  /// 
  DfcDbManager(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbManager& aRef
    /** the reference instance */
  );
    

  /// 
  DfcDbManager& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbManager& aRef
    /** the reference instance */
  );
    

private:
  friend void utest_DfcDbManager(CorUnitTest& aTest); // the unit test function

  ///
  static DfsCString setDbRoot();
  /** {\large Purpose} \\
  Query the root directory of all local databases. This is cross checked
  against the value of an environment variable. If that is not set, then
  it defaults to /dataflow.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of the root directory.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  static DfsCString setHostname();
  /** {\large Purpose} \\
  Set the hostname currently working on.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void close();
  /** {\large Purpose} \\
  Close the connection to a database again.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  static DfsCString  mDfcDbRoot; // where is the LOCAL database located?
  static DfsCString  mHostname;  // hostname currently working at
  static int         mDbSocket;  // socket
  static DfsTValSlist<DfcDb> mDb; // list of databases
};

#endif // DFC_DBMANAGER_H
