//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcDbSelection.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.6  1998/06/30 08:57:40  dataflow
//Removing $NAME
//
//Revision 1.5  1997/12/03 15:09:26  dfadmin
//Checkin for NTT Jan 98
//
//Revision 1.5  1997/05/07 07:45:26  mwiedmer
//some static functions have gone virtual,
//const member is not const anymore,
//juggled member function access rights
//
//Revision 1.4  1997/04/22 13:58:56  mwiedmer
//add mergeStrings(), cleanup docs, made addAndSelection() and
//addOrSelection() virtual
//
//Revision 1.3  1997/04/15 09:31:19  mwiedmer
//use LIKE operator as default operator
//
//Revision 1.2  1997/04/11 13:39:53  mwiedmer
//use RWCString
//
//Revision 1.1  1997/04/10 09:14:50  dfadmin
//Initial revision
//
//Revision 1.1  1997/04/04 14:42:15  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_DBSELECTION_H
#define DFC_DBSELECTION_H

// includes from this library
// includes from other ESO libraries
#include <CorBase.h>
#include "DfsCString.h"
#include "DfsTValVector.h"
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

/// A database selection class.
class DfcDbSelection : public CorBase {
  /**
  {\footnotesize $Id: DfcDbSelection.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcDbSelection.h>
  
    DfcDbSelection myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  Using this class you can create selections on keyword-vlue pairs.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:

  static const int        KW_ANY;
  static const DfsCString VAL_ANY;
  static const int        OP_EQUAL;
  static const int        OP_LESS_THAN;
  static const int        OP_GREATER_THAN;
  static const int        OP_LIKE;


  /// 
  DfcDbSelection(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    int aKeyword = KW_ANY,
    /** keyword */
    ///
    const DfsCString& aValue = VAL_ANY,
    /** value */
    ///
    int anOperator = OP_LIKE
    /** operator */
  );
    

  /// 
  DfcDbSelection(
    /** {\large Purpose} \\
    The c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    const DfsCString& aSelection
    /** full selection string */
  );
    

  /// 
  DfcDbSelection(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbSelection& aRef
    /** the reference instance */
  );
    

  /// 
  DfcDbSelection& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbSelection& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcDbSelection();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual void select(
    /** {\large Purpose} \\
    Create a selection on a keyword-value pair.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns something?

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int aKeyword = KW_ANY,
    /** keyword */
    ///
    const DfsCString& aValue = VAL_ANY,
    /** value */
    ///
    int anOperator = OP_LIKE
    /** operator */
  );


  ///
  void select(
    /** {\large Purpose} \\
    Take on a complete selection.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aSelection
    /** new selection */
  );


  ///
  virtual void addAndSelection(
    /** {\large Purpose} \\
    Add a selection, ANDing it to this.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbSelection* aSelection
    /** Another value */
  );


  ///
  virtual void addOrSelection(
    /** {\large Purpose} \\
    Add a selection, ORing it to this.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbSelection* aSelection
    /** Another value */
  );


  /// 
  DfsCString getSelection() const;
  /** {\large Purpose} \\
  Query the select part of the current selection.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of the select part of
  this selection.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  CorBool isSimpleSelection() const;
  /** {\large Purpose} \\
  Check whether this selection is only made up of a selection string.
  (ie, no tables etc hvae been specified)

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corFALSE if this is a complex selection.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  DfsCString getTables() const;
  /** {\large Purpose} \\
  Query the table part of the current selection.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of all the tables used by
  this selection.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  DfsCString getQualifiers() const;
  /** {\large Purpose} \\
  Query the qualifier part of the current selection.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of the qualifiers of
  this selection.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual int getNextOperator();
  /** {\large Purpose} \\
  Query the next available operator.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the next available operator.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  virtual int getNextKey();
  /** {\large Purpose} \\
  Query the next available keycode.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the next available keycode.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  virtual const DfsCString& getKey(
    /** {\large Purpose} \\
    Convert the keyword code into a string.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the string representation of the operator code.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    int aKeyword
    /** the keyword code */
  );


  /// 
  virtual const DfsCString& getOperator(
    /** {\large Purpose} \\
    Convert the operator code into a string.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the string representation of the operator code.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    int anOperator
    /** the operator code */
  );


protected:

  /// 
  void setSelection(
  /** {\large Purpose} \\
  Set the selection for this instance (ie. the SELECT part).

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of all the tables used by
  this selection.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

    {\large Arguments}\\
  */
    ///
    const DfsCString& aValue
    /** the new selection*/
  );

  /// 
  void setTables(
    /** {\large Purpose} \\
    Set the tables for this instance (ie. the FROM part).

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aValue
    /** the new tables */
  );

  /// 
  void setQualifiers(
    /** {\large Purpose} \\
    Set the qualifiers for this instance (ie. the WHERE part).

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    const DfsCString& aValue
    /** the new qualifiers */
  );


  /// 
  void addTablesAndSelection(
    /** {\large Purpose} \\
    Add more tables and selections to this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    /// 
    const DfcDbSelection* aSelection
    /** selection defining other tables etc */
  );


  DfsCString mergeStrings(
    const DfsCString& stringOne,
    const DfsCString& stringTwo,
    const DfsCString aDelimiter = " '"
  ) const;
// one: "c1, c2, c3"
// two: "c2, c4"
// del: " ,"
// res: "c1, c2, c3, c4"

private:
  friend void utest_DfcDbSelection(CorUnitTest& aTest); // test function

  ///
  static DfsTValVector<DfsCString> setOperators();
  /** {\large Purpose} \\
  Set the operators defined by this class.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a vector with all the operators matching the
  operator codes.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  static DfsTValVector<DfsCString> setKeys();
  /** {\large Purpose} \\
  Set the keywords defined by this class.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a vector with all the keywords matching the
  keyword codes.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  CorBool   mSelectOnly; // is only mSelect filled?
  DfsCString mSelect;    // the current selection (SELECT part)
  DfsCString mFrom;      // the current selection (FROM part)
  DfsCString mWhere;     // the current selection (WHERE part)

  static DfsTValVector<DfsCString> mKeys;
  static DfsTValVector<DfsCString> mOperators;

  static const int mNextOperator;   // next free operator number
  static const int mNextKey;        // next free integer number key
};

#endif // DFC_DBSELECTION_H
