//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1997 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT Data Flow System
//AUTHOR:    Preben Grosbol - ESO/DMD/DPG
//SUBSYSTEM: Data FlowCore
//CI NUMBER: DFC-TMP-001
//
//$Revision: 1.2 $
//$Log: DfcFitsKeyword.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:04  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:44  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/15 13:59:39  dfadmin
//Initial revision
//
//
//-----------------------------------------------------------------------------

#ifndef DFC_FITSKEYWORD_H
#define DFC_FITSKEYWORD_H

#include <DfcFits.h>
#include <stdlib.h>
#include "DfsCString.h"
#include "DfsCTokenizer.h"
#include "DfsConstants.h"

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class DfcFitsUnitTest; // the unit test class

/// A simple class 
class DfcFitsKeyword{
  /**
   {\footnotesize $Id: DfcFitsKeyword.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

   {\large SYNOPSIS} \\
   \begin{verbatim}
   #include <DfcFitsKeyword.h> 
   \end{verbatim}

   {\large DESCRIPTION} \\
   This text serves as an example for the documentation of a class definition

   {\large CAUTIONS} \\
   <optional> \\

   {\large EXAMPLES} \\
   <optional> \\

   {\large SEE ALSO} \\
   <optional> \\
   \Ref{DfcClass}\\
  */

public:
  /// 
  DfcFitsKeyword(char* card);

  /// 
  DfcFitsKeyword(
		 char*  name,
		 char   value,
		 char*  comment
		 );

  /// 
  DfcFitsKeyword(
		 char*  name,
		 char*  value,
		 char*  comment
		 );

  /// 
  DfcFitsKeyword(
		 char*  name,
		 long   value,
		 char*  comment
		 );

  /// 
  DfcFitsKeyword(
		 char*  name,
		 double value,
		 char*  comment
		 );

  ///
  ~DfcFitsKeyword();

  ///
  int operator == (const DfcFitsKeyword& op1) const
  {
    return (!(getName()).compareTo((op1.getName()).data()));
  }

  ///
  DfsCString getName() const;

  ///
  DfsCString getComment() const;

  ///
  char getType() const;

  ///
  int getStatus();

  ///
  int getBoolean();

  ///
  DfsCString getString() const;

  ///
  long getInteger();

  ///
  double getReal();

  ///
  void setComment(const char* comment);

  ///
  void setBoolean(char value);

  ///
  void setBoolean(int value);

  ///
  void setString(const char* value);

  ///
  void setInteger(long value);

  ///
  void setReal(double value);

private:
  DfsCString  kwName;          // Name of FITS keyword
  DfsCString  kwComment;       // Comment field of keyword
  int        kwStatus;        // Status of keyword
  char       kwType;          // Data type of keyword value
  char       bValue;          // Boolean value of FITS keyword
  long       iValue;          // Long int value of FITS keyword
  double     rValue;          // Double value of FITS keyword
  DfsCString  sValue;          // String value of FITS keyword
};

#endif // DFC_FITSKEYWORD_H
