//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.2 $
//$Log: DfcKeywordData.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.4  2002/07/22 13:27:36  dataflow
//Wrong initialization. CG
//Compliant with G++ .3.1
//
//Revision 1.3  2002/07/03 10:05:03  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:47  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:52  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_DATA_H
#define DFC_KEYWORD_DATA_H

// includes from this library
#include "DfcKeywordHeader.h"

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
template<class KeywordType>
class DfcKeywordData: public DfcKeywordHeader {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeywordData.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large DESCRIPTION} \\
  This class is not intended to be used directly by end users.\\
  Provides commonality for any of the derived data classes. I suspect this class may be superfluous and can be removed later but for the moment it's a nice diagrammitcal concept. \\


*/

public:
  ///
  DfcKeywordData(
  ///
  const DfcKeywordData<KeywordType> &other
  /**
  The Keyword that is to be copied. This Keyword is in no way referenced by the new Keyword afterwards.
  */
  );
  // throw(DfcMemoryExc);
  /** {\large Purpose} \\
  copy constructor, perfroms a shallow copy of the original only.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordData(
  ///
  const DfcKeywordInterface &other
  );
  // throw(DfcMemoryExc);
  /** {\large Purpose} \\
  copy constructor, perfroms a shallow copy of the original only.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordData(
  ///
  const DFC_STRING &newName, 
  /**
  A new name for the Keyword.
   */
  const DFC_STRING &alias,
  /**
  An alias for the Keyword.
  */
  ///
  const DFC_STRING &newDescription,
  /**
  Arbitrary text string to store with the Keyword.
  */
  ///
  int numElements = 1
  );
  // throw(DfcMemoryExc);
  /** {\large Purpose} \\
  Constructor which creates a Keyword out of an Array that is passed as an arguent.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory to create the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  void addCopyOfKeyword(
  /// 
  DfcKeywordInterface &newKeyword
  /**
  A reference to the Keyword to be added into the current Keyword. This Keyword will be copied into the KeywordStructure so the responsibility for the deletion of this argument still lies with the method invoker. Likewise, further changes of this argument will not result in the newly added Keyword being changed. \\
  In this part of the hierarchy the invocation of this method results in a DfcTypeExc being thrown. You can not add Keywords to a data Keyword.
  */
  );
  // throw(DfcAlreadyExistsExc, DfcTypeExc);
  /** {\large Purpose} \\
  Add a new Keyword (Structure or Data) to the current Keyword.
 
  {\large Preconditions}\\
  The current Keword should really be a KeywordStructure if this method is to succeed.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - The name already exists in the current Keyword. Names should be unique in any given KeywordStructure.
  \item DfcTypeExc - the Keyword that we're attempting to add the Keyuword into does not contain Keywords !?!
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void deleteKeyword(
  ///
  const DFC_STRING &target
  /**
  The keyword to find and remove from the current Keyword.
   */
  );
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find and remove the specified Keyword from the current Keyword. \\
  In this part of the class hierarchy this simply results in a DfcTypeExc being raised. You cannot remoce a Keyword from a Keyword that only contains data.
 
  {\large Preconditions}\\
  the current Keyword should be a KeywordStructure object.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExist - the requested Keyword for removal was not found in the current Keyword (it may be that this Keyword does not contain other Keywords).
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  int getNumKeywords(void) const;
  // throw();
  /** {\large Purpose} \\
  Returns the number of Keywords that this Keyword contains. \\
  In this case return 0 as DataKeywords have no subkeywords.

  {\large Returns}\\
  the number of keywords.

  */

  ///
  int getNumElements(void) const;
  // throw();
  /**
  {\large Purpose} \\
  Returns the number of data elements in the Keyword. 

  {\large Returns}\\
  the number of data elements.
  */


  ///
  DfcKeywordInterface *operator()(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  For this part of the class hierarchy the invocation of this method simply results in a DfcDoesntExistExc being thrown as this Keyword does not contain any other Keywords to find.

  {\large Returns}\\
  A pointer to the requested Keyword if it exists, null otherwise.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  /// 
  DfcVector<int> getDimensions(void) const;
  /**
  {\large Purpose}\\
  Returns the dimensions of the thing we are dealing with. Implements a method from DfcNumericalInterface.

  {\large Returns}\\
  The dimensions of the structure. Rather than raise an exception if something non-numeric is being accessed (ie a KeywordStructure, or a Column of Frames, the Dimensions returned will be zero.
   */

  ///
  DfcKeywordInterface *operator()(
  ///
  int position
  /**
  The element's index. 
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct(1)(1)(3).report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  /// 
  virtual DfcKeywordData<KeywordType> &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  /// 
  virtual DfcKeywordData<KeywordType> &operator=(const DfcKeywordData<KeywordType> &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  virtual ~DfcKeywordData(void);  
protected:
  ///
  KeywordType *getElement(
  ///
  int whichPos = 0
  /**
  The number of the element to obtain from this Keyword.
   */
  ) const;
  // throw(DfcBoundsExc);
  /** 
  {\large Purpose} \\
  Obtain a piece of data from the Keyword in its storage format. It may then be converted if desired by one of the derived classes to another type.  

  {\large Returns}\\
  The requested piece of data

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcBoundsExc - the supplied parameter was greater than the number of elements contained in the Keyword, or was less than 0.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setElement(
  ///
  KeywordType *newValue,
  ///
  int whichPos = 0
  /**
  The number of the element to obtain from this Keyword.
   */
  );
  // throw(DfcBoundsExc);
  /** 
  {\large Purpose} \\
  Set a piece of data from the Keyword in its storage format. It should have been converted if desired by one of the derived classes to another type.  

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcBoundsExc - the supplied parameter was greater than the number of elements contained in the Keyword, or was less than 0.
  \end{itemize}

  {\large Arguments}\\
  */

  // this implementation is not based around MIDAS
  //  DfcMidasSession midas;
  /**
  This will not be here in the full implementation. For the moment though
  everything is implemented using MIDAS. The structure of the classes is
  kept the same as far as possible as in the real release which is why things
  may appear a little strange. Has been made protected instead of private
  (as it should be) to minimise changes in the header files. Making it 
  private would have required adding a protected method as well to access
  it.
   */

private:
  //  DfsTValSlist<KeywordType> data;
  /**
  Actual storage for the data items.
   */

  int numElements;
  /**
  Depending upon the storage type we use this may or may not be required.
   */

  KeywordType **data;
  /**
  The actual Keyword data is stored in an underlying Array class instance.
   */
};


// START OF DEFINITIONS

//
//
//
template <class KeywordType>
DfcKeywordData<KeywordType>::DfcKeywordData(
  const DfcKeywordData<KeywordType> &other
): DfcKeywordHeader(other),
   data(NULL)
{
  numElements = other.getNumElements();
  data = new (KeywordType*)[numElements];  
  if (data == NULL) {
    DfcAbort(DfcMemoryExc("DfcKeywordData::DfcKeywordData"));
  }
  for (int loop = 0; loop < numElements; loop++) 
  {
    data[loop] = new KeywordType(*other.data[loop]);
  }
}

//
//
//
template <class KeywordType>
DfcKeywordData<KeywordType>::DfcKeywordData(
  const DfcKeywordInterface &other
): DfcKeywordHeader(other),
   data(NULL)
{
  numElements = other.getNumElements();
  data = new (KeywordType*)[numElements];  
  for (int loop = 0; loop < numElements; loop++) data[loop] = NULL;
  if (data == NULL) {
    DfcAbort(DfcMemoryExc("DfcKeywordData::DfcKeywordData"));
  }
}

//
//
//
template <class KeywordType>
DfcKeywordData<KeywordType>::DfcKeywordData(
  const DFC_STRING &newName, 
  const DFC_STRING &alias,
  const DFC_STRING &newDescription,
  int numEls
): 
  DfcKeywordHeader(newName, alias, newDescription),
  numElements(numEls)
{
  data = new (KeywordType*)[numElements];  
  for (int loop = 0; loop < numElements; loop++) data[loop] = NULL;
  if (data == NULL) {
    DfcAbort(DfcMemoryExc("DfcKeywordData::DfcKeywordData"));
//    constructionFailed("not enough memory");
  }
}

//
//
//
template <class KeywordType>
void DfcKeywordData<KeywordType>::addCopyOfKeyword(
  DfcKeywordInterface &newKeyword
)
{
  DfcAbort(DfcTypeExc("DfcKeywordData::addKeyword", getName(), getType(), newKeyword.getType()));
}

//
//
//
template <class KeywordType>
void DfcKeywordData<KeywordType>::deleteKeyword(
  const DFC_STRING &target
)
{
  DfcAbort(DfcTypeExc("DfcKeywordData::removeKeyword", getName(), getType(), "KEYWORD"));
}

//
//
//
template <class KeywordType>
int DfcKeywordData<KeywordType>::getNumKeywords(void) const
{
  return (int)0;
}

//
//
//
template <class KeywordType>
int DfcKeywordData<KeywordType>::getNumElements(void) const
{
  return numElements;
}

//
//
//
template <class KeywordType>
DfcKeywordInterface *DfcKeywordData<KeywordType>::operator()(
  const DFC_STRING &target
) const
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
DfcVector<int> DfcKeywordData<KeywordType>::getDimensions(void) const
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
DfcKeywordInterface *DfcKeywordData<KeywordType>::operator()(
  int position
) const
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
// Note that CC doesn't like covariant return types which is why I always
// return the base class.
//
template <class KeywordType>
DfcKeywordData<KeywordType> &DfcKeywordData<KeywordType>::operator=(const DfcKeywordInterface &other)
{
  if (this == &other) return *this;
  this->DfcKeywordHeader::operator=(other);
  numElements = other.getNumElements();
  // first tear down any existing data
  for (int loop = 0; loop < numElements; loop++)
  {
    if (data[loop] != NULL) delete data[loop];
  }
  delete [] data;
  // now build the new space
  data = new (KeywordType*)[numElements];  
  for (int loop = 0; loop < numElements; loop++) data[loop] = NULL;
  if (data == NULL) {
    DfcAbort(DfcMemoryExc("DfcKeywordData::DfcKeywordData"));
  }
  
  return *this;
}


//
// Note that CC doesn't like covariant return types which is why I always
// return the base class.
//
template <class KeywordType>
DfcKeywordData<KeywordType> &DfcKeywordData<KeywordType>::operator=(const DfcKeywordData<KeywordType> &other)
{
  if (this == &other) return *this;
  this->DfcKeywordHeader::operator=(other);
  numElements = other.getNumElements();
  // first tear down any existing data
  for (int loop = 0; loop < numElements; loop++)
  {
    if (data[loop] != NULL) delete data[loop];
  }
  delete [] data;
  // now build the new space
  data = new (KeywordType*)[numElements];  
  for (int loop = 0; loop < numElements; loop++) 
  {
    data[loop] = new KeywordType(*other.data[loop]);
  }
  if (data == NULL) {
    DfcAbort(DfcMemoryExc("DfcKeywordData::DfcKeywordData"));
  }
  
  return *this;
}

//
//
//
template <class KeywordType>
DfcKeywordData<KeywordType>::~DfcKeywordData(void)
{
  // first delete the contained data
  for (int nextToDie = 0; nextToDie < numElements; nextToDie++)
  {
    delete data[nextToDie];
  }
  delete [] data;
  data = NULL;
}

//
//
//
template <class KeywordType>
KeywordType *DfcKeywordData<KeywordType>::getElement(
  int whichPos
) const
{
  if (whichPos < 0 || whichPos >= numElements)
  {
    DfcAbort(DfcBoundsExc("DfcKeywordData::getElement", whichPos, numElements));
  }

  return data[whichPos];
}

//
//
//
template <class KeywordType>
void DfcKeywordData<KeywordType>::setElement(
  KeywordType *newValue,
  int whichPos
)
{
  if (whichPos < 0 || whichPos >= numElements)
  {
    DfcAbort(DfcBoundsExc("DfcKeywordData::setElement", whichPos, numElements));
  }
  if (data[whichPos] != NULL) delete data[whichPos];
  data[whichPos] = newValue;
}

// END OF DEFINITIONS

#endif // DFC_KEYWORD_DATA_H







