//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcKeywordHeader.h,v $
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:03  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:47  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:52  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_HEADER_H
#define DFC_KEYWORD_HEADER_H

// includes from this library
#include "DfcKeywordInterface.h"
extern "C"
{
  #include <stdio.h>    // for sprintf in report()
}

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
class DfcKeywordHeader: public DfcKeywordInterface{
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeywordHeader.h,v 1.1 2004/09/06 14:35:22 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class implements any commonality between derived classes, adhering to the DfcKeywordInterface interface. \\
  It is not intended for use by end users.\\
  Largely methods are defined here to throw exceptions - such as getAsReal(), and getNumKeywords() and are overridden in a more meaningful class.
*/

public:
  ///
  DfcKeywordHeader(
  ///
  const DFC_STRING &name, 
  /**
  the name for the Keyword
  */
  ///
  const DFC_STRING &alias,
  /**
  An alias for the Keyword.
   */
  ///
  const DFC_STRING &description
  /**
  An arbitrary textual description to accompany the Keyword.
   */
  );
  // throw();
  /** {\large Purpose} \\
  Default Constructor.
  */

  ///
  DfcKeywordHeader(
  ///
  const DfcKeywordInterface &other
  );
  // throw();
  /**
  {\large Purpose}\\
  Copy constructor. Only performs a shallow copy though.
  */


  ///
  DfcKeywordHeader(
  ///
  const DfcKeywordHeader &other
  /**
  Reference to the other Keyword
  */
  );
  // throw();
  /**
  {\large Purpose}\\
  Copy constructor. Only performs a shallow copy though.
  */

  
  ///
  void setAlias(const DFC_STRING &nerwAlias);

  ///
  DFC_STRING getAlias(void) const;

  ///
  DFC_STRING getName(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the name of this Keyword.
   */

  ///
  DFC_STRING getDescription(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the description field of this Keyword which is free form textual thing describing something about the Keyword.
   */

  ///
  void setDescription(
  ///
  const DFC_STRING &newDesc
  /**
  A new descriptive text string for the Keyword.
   */
  );
  // throw();
  /**
  {\large Purpose}\\
  Set a descriptive text field in the Keyword. This forms part of the report produced by the report() method.
  */


  ///
  DFC_STRING report(void) const;
  // throw();
  /** {\large Purpose} \\
  Output some information about this Keyword, probably it's name, type, description and stuff like that.

  {\large Returns}\\
  DFC_STRING containing the report. 
   */

  ///
  void setName(
  ///
  const DFC_STRING &newName
  /**
  The new name for the Keyword.
   */
  );
  // throw(DfcAlreadyExistsExc);
  /** {\large Purpose} \\
  Set the name of the Keyword. Each Keyword in a KeywordStructure must have a unique name, this is enforced.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - the new name for the Keyword already exists within the containing Keyword which is not allowed. The attempt to set the name has failed and the name remains as it was.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInterface *getParent(void) const ;
  // throw()
  /**
  {/large Purpose} \\
  Discover the parent Kewyword of this Keyword. This method is provided with the intent of finding out where in the hierarchy of Keywords one is, not so that the hierarchy can be navigated backwards.

  {/large Returns} \\
  const pointer to the parent object if this Keyword has a parent, otherwise to Null.
  */

  /// 
  DfcKeywordHeader *clone(void) const;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  virtual DFC_BOOL operator==(const DfcKeywordInterface &other) const;
  /** {\large Purpose} \\
  Equality operator (associated operators must be defined later. Only performs comparisons on the basis of the name field. This is how the objects are to be uniquely identified.
  */

  /// 
  DfcKeywordHeader &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  /// 
  DfcKeywordHeader &operator=(const DfcKeywordHeader &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  ///
  virtual DFC_REAL_32 getAsReal32(
  ///
  int whichElement = 0
  /**
  Each Keyword can contain several items of data (of the same type). This field specifies which one is being requested.
   */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc);
  /** {\large Purpose} \\
  Obtain data from the Keyword as a 32 bit real value. \\
  \\
  At this level the behaviour is given that all such calls 

  {\large Preconditions}
  0 <= whichElement < this::getNumElements()

  {\large Returns}\\
  A 32 bit real.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - the data cannot be obtained as a 32 bit real. For example, this Keyword contains Time objects or other Keywords.
  \item DfcBoundsExc - the requested element is not a valid one.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  virtual DFC_REAL_64 getAsReal64(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_INT_UNSIGN_8 getAsIntUnsign8(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_INT_UNSIGN_16 getAsIntUnsign16(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_INT_SIGN_16 getAsIntSign16(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_INT_2C_32 getAsInt2C_32(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_INT_2C_64 getAsInt2C_64(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual char getAsChar(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DFC_STRING getAsString(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcCoordSys getAsCoordSys(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcTime getAsTime(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcFrame getAsFrameRef(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcTable getAsTableRef(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcEnumerated getAsEnumerated(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual DfcTarget getAsTarget(int whichElement = 0) const;  
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsReal32(DFC_REAL_32 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsReal64(DFC_REAL_64 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsIntUnsign8(DFC_INT_UNSIGN_8 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsIntUnsign16(DFC_INT_UNSIGN_16 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsIntSign16(DFC_INT_SIGN_16 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsInt2C_32(DFC_INT_2C_32 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsInt2C_64(DFC_INT_2C_64 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsChar(char newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsString(const DFC_STRING &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsCoordSys(const DfcCoordSysInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsTime(const DfcTimeInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsFrameRef(DfcFrameInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsTableRef(DfcTableInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsEnumerated(const DfcEnumeratedInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  virtual void setAsTarget(const DfcTargetInterface &newValue, int whichElement = 0);  
  // throw(DfcTypeExc, DfcBoundsExc);

  // provide implementation from DfcNumericalInterface

  /// 
  virtual DfcVector<int> getDimensions(void) const;
  /**
  {\large Purpose}\\
  Returns the dimensions of the thing we are dealing with. Implements a method from DfcNumericalInterface.

  {\large Returns}\\
  The dimensions of the structure. Rather than raise an exception if something non-numeric is being accessed (ie a KeywordStructure, or a Column of Frames, the Dimensions returned will be zero.
   */


  ///
  virtual DFC_REAL_64 getAsReal64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as double. Implements the interface inherited from DfcNumericalInterface. At this stage in the hierarchy it simply throws an exception which derived classes can override.
  
  {\large Returns}\\
  A double value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  virtual DFC_INT_2C_64 getAsInt2C_64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as int. Implements method from DfcNumericalInterface. At this stage in the hierarchy it simply throws an exception which derived classes can override.
  
  {\large Returns}\\
  An int value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  virtual void setAsReal64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set.
  */
  ///
  DFC_REAL_64 newVal
  );
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as double. Implements method from DfcNumericalInterface. At this point in the hierarchy this class simply egnerates an exception. It should be overridden to provide better behaviour.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  virtual void setAsInt2C_64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set. Implements a method inherited from DfcNumericalInterface. At this level in the hierarchy, simply throws an exception. Better behaviour is defined in DfcScalarColumn.
  */
  ///
  DFC_INT_2C_64 newVal
  );
  // throw(DfcBoundsExc, DfcTypeExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as an int.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  virtual ~DfcKeywordHeader(void);  

protected:
  ///
  void setParent(DfcKeywordInterface *newParent);

  ///
  DfcKeyword *getSafeInterface(void);
  // throw(void)
  /**
  {\large Purpose}\\
  Obtain a proxy object for this object. 
 
  {\large Returns}\\
  A pointer to a DfcKeyword object
  */

  ///
  DfcKeywordInterface *stripProxies(void);
  // throw();
  /** 
  {\large Purpose} \\
  obtain a pointer to the actual data that a proxy is using.
  
  */

  ///
  void proxyIsDead(void);
  // throw();
  /**
  {\large Purpose}\\
  Informs this object that its proxy object has been deleted. This allows it to make sure there are no dangling pointers.
   */

private:
  DFC_STRING name;
  /**
  A name for this Keyword. It will be used for navigating through the Keyword hierarchy and so should there should only be one of each string in the Keywords contained in any one KeywrodStructure.
  */

  DFC_STRING alias;
  /**
  An alias for this Keyword.
  */

  DFC_STRING description;
  /**
  Simple textual description of what the Keyword represents.
  */  

  DfcKeyword *proxy;
  /**
  Pointer to the associated proxy class for this class.
   */

  DfcKeywordInterface *parent;
  /**
  Pointer to the parent class, null if the Keyword does not have a parent.
   */
};

#endif // DFC_KEYWORD_HEADER_H

//
// eof
//





