//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcKeywordInt2C64.h,v $
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:02  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:48  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:52  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_INT2C64_H
#define DFC_KEYWORD_INT2C64_H

// includes from this library
#include "DfcScalarKeyword.h"

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
class DfcKeywordInt2C64: public DfcScalarKeyword<DFC_INT_2C_64> {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeywordInt2C64.h,v 1.1 2004/09/06 14:35:22 cguirao Exp $ } \\

  {\large DESCRIPTION} \\
  Only required in order to get the type of the class. It may seem wasteful in terms of space, however the ScalarKeyword class generates templates for every possible instantiation anyway so we are only making this process explicit. Unfortunately we have to supply all the standard constructors and operators as well which is tedious. The alternative is to supply a collection of template matching functions that determine the type. However this may not work if we port to a platform where not all the types are supported.
*/

public:
  ///
  DfcKeywordInt2C64(
  ///
  const DfcKeywordInterface &other
  );
  // throw(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInt2C64(
  ///
  const DfcKeywordInt2C64 &other
  /**
  The Keyword that is to be copied. This Keyword is in no way referenced by the new Keyword afterwards.
  */
  );
  // throw(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInt2C64(
  ///
  const DFC_STRING &newName,
  /**
  A name for the Keyword - should be unique in the current KeywordStructure.
  */
  ///
  const DFC_STRING &alias,
  /**
  An alias for the Keyword.
   */
  ///
  const DFC_STRING &description,
  /**
  arbitrary textual description of the contents of the Keyword.
   */
  ///
  int numElements = 1
  /**
  The number of elements to create space for.
  */
  );
  // throw(DfcMemoryExc); 
  /**
  {\large Purpose} \\
  Create a new Keyword capable of storing the specified amount of data.
   */

  ///
  DfcKeywordInt2C64(
  ///
  const DFC_STRING &newName,
  ///
  int numElements = 1
  );

  
  ///
  DfcKeywordInt2C64 *clone(void) const;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  
  /// 
  DfcKeywordInt2C64 &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  
  /// 
  DfcKeywordInt2C64 &operator=(const DfcKeywordInt2C64 &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  virtual DFC_STRING getAsString(
  ///
  int whichElement = 0
  ) const;

  ///
  DfcKeywordInterface::TYPE getType(void) const;
  // throw(); 
  /** {/large Purpose} \\
  An indication of the type of the Keyword. This is used while the standard C++ RTTI mechanisms are still in their infancy and may, or may not, become redundant later. The primary purpose of the type field is to allow persistency, not so that users can query it and perform switch statements on the type. Because of this it may be hidden from the public interface later.
  */

  ///
  virtual ~DfcKeywordInt2C64(void);  
protected:

private:
};

#endif // DFC_KEYWORD_INT2C64_H

//
// eof 
//





