//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.2 $
//$Log: DfcKeywordStructure.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:09  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:49  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:54  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:22  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_STRUCTURE_H
#define DFC_KEYWORD_STRUCTURE_H

// includes from this library
#include "DfsTPtrSlist.h"
#include "DfcKeywordHeader.h"
extern "C"
{
  #include <stdio.h>  // for sprintf in report()
}


// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
class DfcKeywordStructure: public DfcKeywordHeader {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeywordStructure.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
  \end{verbatim}

  {\large DESCRIPTION} \\
  DfcKeywordStructure contains a number of KeywordData objects. As such it means that Keywords can be nested in a tree-like structure. Every class that contains Keywords contains a top level KeywordStructure object - even if it is empty.

  {\large EXAMPLES} \\
  \\
  // \\
  // We have an image object called myImage \\
  // We want to obtain the data as an int \\
  // \\
  DFC_INT_UNSIGN_16 val = myImage.getKeywords()("firstStruct")("secondStruct").getAsIntUnsign_16(); \\
  \\
  // now add a new Keyword to the heirarchy \\
  int fiveElements = 5; \\
  DfcIntUnsign16_Keyword newKeyword("uniqueName", "no description", fiveElements); \\
  myImage.getKeywords().addKeyword(newKeyword); \\
  \\

  {\large SEE ALSO} \\
  DfcKeywordIterator for example code on how to navigate a Keyword object hierarchy.

*/

public:
  ///
  DfcKeywordStructure(
  ///
  const DFC_STRING &newName,
  /**
  A name for the Keyword structure.
  */
  ///
  const DFC_STRING &alias = "",
  ///
  const DFC_STRING &description = ""
  /**
  An arbitrary textual description to accompany the Keyword. Optional really.
  */
  );
  // throw();
  /**
  {\large Purpose}\\
  Constructor.
  */

  ///
  DfcKeywordStructure(
  ///
  const DfcKeywordStructure &other
  /**
  The other structure to copy into this one.
  */
  );
  // throw(DfcMemoryExc);
  /** {\large Purpose} \\
  Copy Constructor. Performs a shallow copy only.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc -machine doesn't have enough memory to make a copy.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  DfcKeywordStructure(
  ///
  const DfcKeywordInterface &other
  /**
  The other structure to copy into this one.
  */
  );
  // throw(DfcMemoryExc);
  /** {\large Purpose} \\
  Copy Constructor. Performs a shallow copy only.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc -machine doesn't have enough memory to make a copy.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  int getNumKeywords(void) const;
  // throw();
  /** {\large Purpose} \\
  Returns the number of Keywords that this Keyword contains. 

  {\large Returns}\\
  the number of keywords.
  */

  ///
  int getNumElements(void) const;
  // throw();
  /**
  {\large Purpose} \\
  Returns the number of data elements in the Keyword. 

  {\large Returns}\\
  the number of data elements.

   */

  ///
  DfcKeywordInterface *operator()(
  ///
  int position
  /**
  The element's index. 
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct(1)(1)(3).report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInterface *operator()(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void addCopyOfKeyword(
  /// 
  DfcKeywordInterface &newKeyword
  /**
  A reference to the Keyword to be added into the current Keyword. This Keyword will be copied into the KeywordStructure so the responsibility for the deletion of this argument still lies with the method invoker. Likewise, further changes of this argument will not result in the newly added Keyword being changed. 
  */
  );
  // throw(DfcAlreadyExistsExc, DfcTypeExc);
  /** {\large Purpose} \\
  Add a new Keyword (Structure or Data) to the current Keyword.
 
  {\large Preconditions}\\
  The current Keword should really be a KeywordStructure if this method is to succeed.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - The name already exists in the current Keyword. Names should be unique in any given KeywordStructure.
  \item DfcTypeExc - the Keyword that we're attempting to add the Keyuword into does not contain Keywords !?!
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void deleteKeyword(
  ///
  const DFC_STRING &target
  /**
  The keyword to find and remove from the current Keyword.
   */
  );
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find and remove the specified Keyword from the current Keyword.
 
  {\large Preconditions}\\
  the current Keyword should be a KeywordStructure object.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExist - the requested Keyword for removal was not found in the current Keyword (it may be that this Keyword does not contain other Keywords).
  \end{itemize}

  {\large Arguments}\\
  */

  /// 
  DfcKeywordStructure *clone(void) const;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  ///
  DfcKeywordInterface::TYPE getType(void) const;
  // throw(); 
  /** {/large Purpose} \\
  An indication of the type of the Keyword. This is used while the standard C++ RTTI mechanisms are still in their infancy and may, or may not, become redundant later. The primary purpose of the type field is to allow persistency, not so that users can query it and perform switch statements on the type. Because of this it may be hidden from the public interface later.
  */


  /// 
  DfcKeywordStructure &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  virtual DFC_STRING report(void) const;

  /// 
  DfcKeywordStructure &operator=(const DfcKeywordStructure &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  ///
  virtual ~DfcKeywordStructure(void);  
protected:

private:
  mutable DfsTPtrSlist<DfcKeywordInterface> keywords;
  /**
  mutable because its used in operator(const DFC_STRING &) const
  This class is not derived from DfcKeywordData because:
  1 - I store pointers rather than values
  2 - I'll probably change the representation to a map later
   */
};

#endif // DFC_KEYWORD_STRUCTURE_H







