//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcParameter.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:57:51  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 09:57:50  dfadmin
//Initial revision
//
//Revision 1.5  1997/02/24 15:18:15  mwiedmer
//add automatic int <-> double casting
//
//Revision 1.4  1996/12/13 14:25:07  mwiedmer
//make persistent
//
//Revision 1.3  1996/10/11 08:50:18  mwiedmer
//add friend void utest_DfcParameter
//
//Revision 1.2  1996/09/03 14:41:29  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 12:18:22  dfadmin
//Update project structure
//
//Revision 1.1  1996/09/02 09:43:38  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_PARAMETER_H
#define DFC_PARAMETER_H

// includes from this library
#include <DfcValue.h>
// includes from other ESO libraries
#include <CorPersistent.h>
#include <CorBool.h>
#include "DfsCString.h"
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class DfcConstraint;
class CorStream;

/// a typed and named value
class DfcParameter : public CorPersistent {
  DECLARE_PERSISTENT(DfcParameter);
  /**
  {\footnotesize $Id: DfcParameter.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcParameter.h>
  
    DfcParameter myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class models a value that has a type and a name.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  \Ref{CorBase}\\
  \Ref{DfcSignature}\\
  \Ref{DfcValue}\\
  */

public:

  // exception classes
  // class TypeMismatch {};


  /// 
  DfcParameter(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName = ""
    /** name of this parameter */
  );


  /// 
  DfcParameter(
    /** {\large Purpose} \\
    C'tor with name and value.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name of this parameter */
    ///
    const DfcValue& aValue
    /* value of this parameter */
  );


  /// 
  DfcParameter(
    /** {\large Purpose} \\
    C'tor with name and value.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name of this parameter */
    ///
    const DfcValue& aValue,
    /** value of this parameter */
    ///
    const DfcConstraint& aConstraint
    /** constraints on this parameter */
  );


  /// 
  DfcParameter(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcParameter& aRef
    /** the reference instance */
  );
    

  /// 
  DfcParameter& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcParameter& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcParameter();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  DfsCString getName() const;
  /** {\large Purpose} \\
  Query the name of this parameter.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a string representing the name of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the name and value of this parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name */
    ///
    const DfcValue aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the value of this parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcValue& aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the name and value of this integer parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name */
    ///
    const int aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the value of this integer parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const int aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the name and value of this double parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name */
    ///
    const double aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the value of this double parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const double aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the name and value of this string parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName,
    /** name */
    ///
    const DfsCString aValue
    /** value */
  );


  ///
  CorBool setValue(
    /** {\large Purpose} \\
    Set the value of this string parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns whether the value could be set. Failure could mean
    that the constraints have not been met.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aValue
    /** value */
  );


  ///
  DfcValue getValue() const;
  /** {\large Purpose} \\
  Query the value of this parameter.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the value of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  int getInteger() const;
  /** {\large Purpose} \\
  Query the value of this parameter as an integer.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns an integer representation of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \item The exception TypeMismatch will be thrown if the value held is
  not an integer nor a double type.
  \end{itemize}
  */


  ///
  DfsCString getString() const;
  /** {\large Purpose} \\
  Query the value of this parameter as a string.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a string representation of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \item The exception TypeMismatch will be thrown if the value held is
  not a string type.
  \end{itemize}
  */


  ///
  double getDouble() const;
  /** {\large Purpose} \\
  Query the value of this parameter as a double.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a double representation of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \itemThe exception TypeMismatch will be thrown if the value held is
  not a double nor an integer  type.
  \end{itemize}
  */


  ///
  const CorPersistent* getPersistent() const;
  /** {\large Purpose} \\
  Query the value of this parameter as a persistent type.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a persistent representation of this instance.

  {\large Exceptions}\\
  \begin{itemize}
  \itemThe exception TypeMismatch will be thrown if the value held is
  not a persistent type.
  \end{itemize}
  */


  ///
  DfsCString getType() const;
  /** {\large Purpose} \\
  Query the type of this parameter.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a string representation of the type of this
  parameter.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void setConstraint(
    /** {\large Purpose} \\
    Set the constraint of this parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns something?

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcConstraint& aConstraint
    /** constraint to use */
  );


  ///
  const DfcConstraint& getConstraint() const;
  /** {\large Purpose} \\
  Query the constraint on this parameter.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the constraints of this parameter.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool check(
    /** {\large Purpose} \\
    Check the constraints against the value.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns true if the value is within the given constraints.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}
    */
    ///
    const DfcValue& aValue
    /** value to check */
  ) const;


  ///
  virtual void stream(
    /** {\large Purpose} \\
    Stream the contents of this signature through a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile
    /** file to stream the signature through */
  );

#if 0
  ///
  virtual void writeIDF(
    /** {\large Purpose} \\
    Save the contents of this instance to a stream.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    CorStream& aStream
    /** stream to save the parameter to */
  ) const;


  ///
  virtual CorBool readIDF(
    /** {\large Purpose} \\
    Load the contents of this instance from a stream.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    CorStream& aStream
    /** stream to load the parameter from */
  );
#endif

  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:

  ///
  void become(
    /** {\large Purpose} \\
    This instance will take on the attributes of a reference instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}
    */
    ///
    const DfcParameter& aRef
    /** a reference */
  );

private:
  friend void utest_DfcParameter(CorUnitTest&); // the unit test function

  CorBool        mInitialized;   // is this instance valid?
  DfsCString     mName;          // name of this parameter
  DfcValue*      mValue;         // its value
  DfcConstraint* mConstraint;    // and constraint
};

#endif // DFC_PARAMETER_H
