//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: 
//
//$Revision: 1.1 $
//$Log: DfcRange.h,v $
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:57:53  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 10:00:28  dfadmin
//Initial revision
//
//Revision 1.2  1996/10/11 08:51:23  mwiedmer
//add friend void utst_DfcRange
//
//Revision 1.1  1996/09/10 11:55:09  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_RANGE_H
#define DFC_RANGE_H

// includes from this library
#include <DfcConstraint.h>
// includes from other ESO libraries
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations

/// A range constraint
class DfcRange : public DfcConstraint {
  /**
  {\footnotesize $Id: DfcRange.h,v 1.1 2004/09/06 14:35:23 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcRange.h>
  
    DfcRange myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class holds the upper and lower limits to identify a range.
  Ranges can also contain one or the other to set lower or upper bounds
  only. Ranges are inclusive boundaries.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:

  /// 
  DfcRange(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfcValue* aLowerLimit = 0,
    /** lower limit or Null */
    ///
    DfcValue* anUpperLimit = 0
    /** upper limit or Null */
  );


  /// 
  DfcRange(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcRange& aRef
    /** the reference instance */
  );
    

  /// 
  DfcRange& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcRange& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcRange();  // do you need keyword virtual here?
  /** {\large Purpose} \\
  The d'tor. If it's not in a hierarchy it probably doesn't want to be
  virtual.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void setRange(
    /** {\large Purpose} \\
    Set the upper and lower limits of this range.\\
    A Null pointer to a limit means to ignore that limit.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfcValue* aLowerLimit = 0,
    /** lower limit or Null */
    ///
    DfcValue* anUpperLimit = 0
    /** upper limit or Null */
  );


  ///
  void setUpper(
    /** {\large Purpose} \\
    Set the upper limit. The limit will be ignored if the limit pointer is
    Null.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfcValue* aLimit = 0
    /** limit or Null */
  );


  ///
  void setLower(
    /** {\large Purpose} \\
    Set the lower limit. The limit will be ignored if the limit pointer is
    Null.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfcValue* aLimit = 0
    /** limit or Null */
  );


  ///
  const DfcValue* getUpper() const;
  /** {\large Purpose} \\
  Query the upper limit. Null means that there is no limit.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the upper limit or Null.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  const DfcValue* getLower() const;
  /** {\large Purpose} \\
  Query the upper limit. Null means that there is no limit.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the upper limit or Null.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual CorBool evaluate(
    /** {\large Purpose} \\
    Check whether the supplied value fits the constraints.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the value fits the constraints.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcValue& aVal
    /** a vlaue */
  ) const;


protected:

  ///
  CorBool checkType(
    /** {\large Purpose} \\
    Check whether the supplied value has the same type as the exisiting
    boundaries.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the value has the same type.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfcValue* aVal
    /** a value to check */
  ) const;

private:
  friend void utest_DfcRange(CorUnitTest&); // the unit test function

  DfcValue* mLowerLimit;  // lower and upper limits
  DfcValue* mUpperLimit;  // 0 means that the limit can be ignored
};

#endif // DFC_RANGE_H
