//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcScalarKeyword.h,v $
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.4  2002/07/22 13:27:36  dataflow
//Wrong initialization. CG
//Compliant with G++ .3.1
//
//Revision 1.3  2002/07/03 10:05:08  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:54  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:56  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_SCALAR_KEYWORD_H
#define DFC_SCALAR_KEYWORD_H

// includes from this library
#include "DfcKeywordData.h"
extern "C"
{
  #include <stdio.h>  // for sprintf for converting numbers to strings
}

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
template <class KeywordType>
class DfcScalarKeyword: public DfcKeywordData<KeywordType> {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcScalarKeyword.h,v 1.1 2004/09/06 14:35:23 cguirao Exp $ } \\

  {\large DESCRIPTION} \\
  This class holds any values that can be accessed as scalar values. \\
  It will be used by the end user only to create new Keywords. \\
*/

public:
  ///
  DfcScalarKeyword(
  ///
  const DfcScalarKeyword<KeywordType> &other
  /**
  The Keyword that is to be copied. This Keyword is in no way referenced by the new Keyword afterwards.
  */
  );
  // DfcAbort(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcScalarKeyword(
  ///
  const DfcKeywordInterface &other
  );
  // DfcAbort(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcScalarKeyword(
  ///
  const DFC_STRING &newName,
  /**
  A name for the Keyword - should be unique in the current KeywordStructure.
  */
  ///
  const DFC_STRING &alias,
  /**
  An alias for the Keyword.
   */
  ///
  const DFC_STRING &description,
  /**
  arbitrary textual description of the contents of the Keyword.
   */
  ///
  int numElements = 1
  /**
  The number of elements to create space for.
  */
  );
  // DfcAbort(DfcMemoryExc); 
  /**
  {\large Purpose} \\
  Create a new Keyword capable of storing the specified amount of data.
   */

  ///
  DFC_REAL_32 getAsReal32(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_REAL_64 getAsReal64(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_INT_UNSIGN_8 getAsIntUnsign8(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_INT_UNSIGN_16 getAsIntUnsign16(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_INT_SIGN_16 getAsIntSign16(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_INT_2C_32 getAsInt2C_32(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  DFC_INT_2C_64 getAsInt2C_64(int whichElement = 0) const;
  // DfcAbort(DfcTypeException, DfcBoundsException);
 
 ///
  void setAsReal32(DFC_REAL_32 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsReal64(DFC_REAL_64 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsIntUnsign8(DFC_INT_UNSIGN_8 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsIntUnsign16(DFC_INT_UNSIGN_16 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsIntSign16(DFC_INT_SIGN_16 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsInt2C_32(DFC_INT_2C_32 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);

  ///
  void setAsInt2C_64(DFC_INT_2C_64 newValue, int whichElement = 0);
  // DfcAbort(DfcTypeException, DfcBoundsException);




  /// 
  DfcScalarKeyword<KeywordType> &operator=(const DfcKeywordInterface &other);
  // DfcAbort(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  /// 
  DfcScalarKeyword<KeywordType> &operator=(const DfcScalarKeyword<KeywordType> &other);
  // DfcAbort(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  DFC_REAL_64 getAsReal64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // DfcAbort(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as double. Implements the interface inherited from DfcNumericalInterface.
  
  {\large Returns}\\
  A double value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DFC_INT_2C_64 getAsInt2C_64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // DfcAbort(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as int. Implements method from DfcNumericalInterface.
  
  {\large Returns}\\
  An int value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setAsReal64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set.
  */
  ///
  DFC_REAL_64 newVal
  );
  // DfcAbort(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as double. Implements method from DfcNumericalInterface.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setAsInt2C_64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set. Implements a method inherited from DfcNumericalInterface. 
  */
  ///
  DFC_INT_2C_64 newVal
  );
  // DfcAbort(DfcBoundsExc, DfcTypeExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as an int.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  virtual ~DfcScalarKeyword(void);  
protected:

private:
};

//
// unfortunately the definitions in g++ templates must be included in the
// source, they cannot be linked so I've put them all in the header file.
//
// START OF DEFINITIONS
//

//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType>::DfcScalarKeyword(
  const DfcScalarKeyword<KeywordType> &other
):
  DfcKeywordData<KeywordType>(other)
{
  // no body required yet
}

//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType>::DfcScalarKeyword(
  const DfcKeywordInterface &other
):
  DfcKeywordData<KeywordType>(other)
{
  // no body required yet
}

//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType>::DfcScalarKeyword(
  const DFC_STRING &newName,
  const DFC_STRING &alias,
  const DFC_STRING &description,
  int numElements
):
  DfcKeywordData<KeywordType>(newName, alias, description, numElements)
{
  for (int loop = 0; loop < getNumElements(); loop++)
  {
    setElement(new KeywordType(0), loop);
  }
}


//
//
//
template <class KeywordType>
DFC_REAL_32 DfcScalarKeyword<KeywordType>::getAsReal32(int whichElement) const
{
  return (DFC_REAL_32)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_REAL_64 DfcScalarKeyword<KeywordType>::getAsReal64(int whichElement) const
{
  return (DFC_REAL_64)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_INT_UNSIGN_8 DfcScalarKeyword<KeywordType>::getAsIntUnsign8(int whichElement) const
{
  return (DFC_INT_UNSIGN_8)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_INT_UNSIGN_16 DfcScalarKeyword<KeywordType>::getAsIntUnsign16(int whichElement) const
{
  return (DFC_INT_UNSIGN_16)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_INT_SIGN_16 DfcScalarKeyword<KeywordType>::getAsIntSign16(int whichElement) const
{
  return (DFC_INT_SIGN_16)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_INT_2C_32 DfcScalarKeyword<KeywordType>::getAsInt2C_32(int whichElement) const
{
  return (DFC_INT_2C_32)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
DFC_INT_2C_64 DfcScalarKeyword<KeywordType>::getAsInt2C_64(int whichElement) const
{
  return (DFC_INT_2C_64)*getElement(whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsReal32(DFC_REAL_32 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsReal64(DFC_REAL_64 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsIntUnsign8(DFC_INT_UNSIGN_8 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsIntUnsign16(DFC_INT_UNSIGN_16 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsIntSign16(DFC_INT_SIGN_16 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsInt2C_32(DFC_INT_2C_32 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsInt2C_64(DFC_INT_2C_64 newValue, int whichElement)
{
  KeywordType *storedCopy = new KeywordType((KeywordType)newValue);
  setElement(storedCopy, whichElement);
}

//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType> &DfcScalarKeyword<KeywordType>::operator=(const DfcKeywordInterface &other)
{
  if (this == &other) return *this;
  this->DfcKeywordData<KeywordType>::operator=(other);
  return *this;
}


//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType> &DfcScalarKeyword<KeywordType>::operator=(const DfcScalarKeyword<KeywordType> &other)
{
  if (this == &other) return *this;
  this->DfcKeywordData<KeywordType>::operator=(other);
  return *this;
}

//
//
//
template <class KeywordType>
DFC_REAL_64 DfcScalarKeyword<KeywordType>::getAsReal64(
  const DfcVector<int> &position
) const
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
DFC_INT_2C_64 DfcScalarKeyword<KeywordType>::getAsInt2C_64(
  const DfcVector<int> &position
) const
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsReal64(
  const DfcVector<int> &position, 
  DFC_REAL_64 newVal
)
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
void DfcScalarKeyword<KeywordType>::setAsInt2C_64(
  const DfcVector<int> &position, 
  DFC_INT_2C_64 newVal
)
{
  DfcAbort(DfcNotImplementedYetExc());
}

//
//
//
template <class KeywordType>
DfcScalarKeyword<KeywordType>::~DfcScalarKeyword(void)
{
  // no body required yet
}


// END OF DEFINITIONS

#endif // DFC_SCALAR_KEYWORD_H

//
// eof 
//





