//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcSignature.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:57:55  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 09:21:12  dfadmin
//Initial revision
//
//Revision 1.5  1997/04/07 08:59:05  mwiedmer
//use RW lists
//
//Revision 1.4  1996/12/13 14:26:29  mwiedmer
//make persistent
//
//Revision 1.3  1996/10/11 08:53:36  mwiedmer
//add load() merge() and save(), declare getParameter()
//
//Revision 1.2  1996/09/03 14:41:30  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 12:18:22  dfadmin
//Update project structure
//
//Revision 1.1  1996/09/02 09:43:38  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_SIGNATURE_H
#define DFC_SIGNATURE_H

// includes from this library
#include <DfcParameter.h>
// includes from other ESO libraries
#include <CorPersistent.h>
#include "DfsCString.h"
#include "DfsTPtrDlist.h"
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorStream;

/// a template signature
class DfcSignature : public CorPersistent {
  DECLARE_PERSISTENT(DfcSignature);
  /**
  {\footnotesize $Id: DfcSignature.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcSignature.h>

    DfcSignature myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class provides access to a collection of parameters. These are typed
  and named values. The parameters are private and not shared with the world.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  \Ref{CorBase}\\
  \Ref{DfcParameter}\\
  \Ref{DfcValue}\\
  */

public:

  /// 
  DfcSignature();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  DfcSignature(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcSignature& aRef
    /** the reference instance */
  );
    

  /// 
  DfcSignature& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcSignature& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcSignature();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  const DfcParameter* getParameter(
    /** {\large Purpose} \\
    Query a parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a pointer to the requested parameter or NULL.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aName
    /** name of the parameter */
  ) const;


  ///
  void setParameter(
    /** {\large Purpose} \\
    Edit or add a parameter.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcParameter& aParam
    /** parameter */
  );


  ///
  virtual void stream(
    /** {\large Purpose} \\
    Stream the contents of this signature through a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile
    /** file to stream the signature through */
  );

#if 0
  ///
  CorBool mergeIDF(
    /** {\large Purpose} \\
    Merge the contents of this signature with those from a database.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aStream
    /** stream to merge the signature from */
  );


  virtual CorBool readIDF(
    /** {\large Purpose} \\
    Load the contents of this signature with those from a database.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aStream
    /** stream to load the signature from */
  );


  virtual void writeIDF(
    /** {\large Purpose} \\
    Save the contents of this signature to an Instrument Description File.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aStream
    /** stream to save the signature to */
  ) const;
#endif

  void print() const;

  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:

  ///
  void clear();
  /** {\large Purpose} \\
  Clear the contents of this signature.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


private:
  friend void utest_DfcSignature(CorUnitTest&); // the unit test function

  DfsTPtrDlist<DfcParameter> mParameters;  // list of parameter pointers
};

#endif // DFC_SIGNATURE_H
