//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcStream.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:57:55  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 09:21:44  dfadmin
//Initial revision
//
//-----------------------------------------------------------------------------

#ifndef DFC_STREAM_H
#define DFC_STREAM_H

// includes from this library
// includes from other ESO libraries
#include <CorBase.h>
#include <CorBool.h>
#include <CorRef.h>
#include "DfsCString.h"
// includes from other libraries
// system includes
#include <fstream>

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

/// A simple stream class 
class DfcStream : public CorBase {
  /**
  {\footnotesize $Id: DfcStream.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcStream.h>
  
    DfcStream myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class serves as a protecting layer on top of the standard stream
  classes. It knows about the flow of information and can also stream
  strings.

  {\large CAUTIONS} \\
  Numerical data is streamed in byte format. This means that data files
  machine dependend. Also, sizes of integers and doubles may cause troubles.

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:

  enum StreamFlow {
    READ,
    WRITE,
    READ_WRITE
  };


  /// 
  DfcStream();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  DfcStream(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aFileName,
    /** filename to use with this stream */
    ///
    StreamFlow aMode
    /** mode to open in */
  );


  /// 
  DfcStream(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcStream& aRef
    /** the reference instance */
  );
    

  /// 
  DfcStream& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcStream& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcStream();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    char& aChar
    /** data to stream */
  );


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int& anInt
    /** data to stream */
  );


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    double& aDouble
    /** data to stream */
  );


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfsCString& aString
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char aChar
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const int anInt
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const double aDouble
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aString
    /** data to stream */
  );


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned char& aChar
    /** data to stream */
  );


  ///
  DfcStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int& anInt
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const unsigned char aChar
    /** data to stream */
  );


  ///
  DfcStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const unsigned int anInt
    /** data to stream */
  );


  ///
  void close();
  /** {\large Purpose} \\
  Clear this instance.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:

  ///
  void become(
    /** {\large Purpose} \\
    This function makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcStream& aRef
    /** the reference instance */
  );
    

  ///
  CorBool clear();
  /** {\large Purpose} \\
  Clear this instance.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


private:
  friend void utest_DfcStream(CorUnitTest&);  // the unit test function

  StreamFlow     mFlow;        // direction of dataflow
  std::fstream*  mStream;      // the stream
  CorBool        mInitialized; // state of this instance
  CorRef*        mRefCounter;  // number of times this stream is used
};

#endif // DFC_STREAM_H
