//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Dataflow Core
//
//$Revision: 1.2 $
//$Log: DfcValue.h,v $
//Revision 1.2  2004/10/26 15:45:27  nferguso
//temp
//
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:57:56  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 09:58:39  dfadmin
//Initial revision
//
//Revision 1.7  1997/03/18 10:56:58  mwiedmer
//make DfcDB become DfcDb,
//DfcDBManager to DfcDbManager, DfcIDF to DfcIdf
//and use environment variable to determine
//the DFC_DB_ROOT
//
//Revision 1.6  1997/02/24 15:18:15  mwiedmer
//add automatic int <-> double casting
//
//Revision 1.5  1996/12/13 14:28:15  mwiedmer
//make persistent
//
//Revision 1.4  1996/10/11 08:54:46  mwiedmer
//add clear() and friend void utest_DfcValue()
//
//Revision 1.3  1996/09/10 11:55:10  mwiedmer
//first release
//
//Revision 1.2  1996/09/03 14:41:30  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 12:18:23  dfadmin
//Update project structure
//
//Revision 1.1  1996/09/02 09:43:38  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef DFC_VALUE_H
#define DFC_VALUE_H

// includes from this library
// includes from other ESO libraries
#include <CorPersistent.h>
#include <CorBool.h>
#include "DfsCString.h"
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorStream;

/// a typed value
class DfcValue : public CorPersistent {
  DECLARE_PERSISTENT(DfcValue);
  /**
  {\footnotesize $Id: DfcValue.h,v 1.2 2004/10/26 15:45:27 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <DfcValue.h>
  
    DfcValue myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class models a typed value.\\
  Possible types are:\\
  \begin{verbatim}
   DfcSTRING  = "string"
   DfcDOUBLE  = "double"
   DfcINT     = "integer"
  \end{verbatim}

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  \Ref{DfcSignature}\\
  \Ref{DfcParameter}\\
  */

public:

  // types
  static const char* DfcPERSISTENT;
  static const char* DfcSTRING;
  static const char* DfcDOUBLE;
  static const char* DfcINT;

  // exception classes
  // class TypeMismatch {};

  /// 
  DfcValue();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  DfcValue(
    /** {\large Purpose} \\
    Construct a value from an integer.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int anInt
    /** value */
  );
    

  /// 
  DfcValue(
    /** {\large Purpose} \\
    Construct a value from a double.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    double aDouble
    /** value */
  );
    

  /// 
  DfcValue(
    /** {\large Purpose} \\
    Construct a value from a string.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aString
    /** value */
  );
    

  /// 
  DfcValue(
    /** {\large Purpose} \\
    Construct a value from a persistent instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPersistent* aPers
    /** value */
  );
    

  /// 
  DfcValue(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcValue& aRef
    /** the reference instance */
  );
    

  /// 
  DfcValue& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcValue& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~DfcValue();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void setValue(
    /** {\large Purpose} \\
    Set the value of this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int aValue
    /** new value */
  );


  ///
  void setValue(
    /** {\large Purpose} \\
    Set the value of this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    double aValue
    /** new value */
  );


  ///
  void setValue(
    /** {\large Purpose} \\
    Set the value of this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aValue
    /** new value */
  );


  ///
  void setValue(
    /** {\large Purpose} \\
    Set the value of this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPersistent* aValue
    /** new value */
  );


  ///
  int getInt() const;
  /** {\large Purpose} \\
  Query the value as an integer.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the integer representation of this value.

  {\large Exceptions}\\
  \begin{itemize}
  \item If the type is not an integer nor a double a TypeMismatch exception
  is thrown.
  \end{itemize}
  */


  ///
  double getDouble() const;
  /** {\large Purpose} \\
  Query the value as a double.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the double representation of this value.

  {\large Exceptions}\\
  \begin{itemize}
  \item If the type is not a double nor an integer a TypeMismatch exception
  is thrown.
  \end{itemize}
  */


  ///
  DfsCString getString() const;
  /** {\large Purpose} \\
  Query the value as a string.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of this value.

  {\large Exceptions}\\
  \begin{itemize}
  \item If the type is not a string a TypeMismatch exception is thrown.
  \end{itemize}
  */


  ///
  const CorPersistent* getPersistent() const;
  /** {\large Purpose} \\
  Query the value as a persistent instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the persistent representation of this value.

  {\large Exceptions}\\
  \begin{itemize}
  \item If the type is not a string a TypeMismatch exception is thrown.
  \end{itemize}
  */


  ///
  CorBool isType(
    /** {\large Purpose} \\
    Query whether this instance is of a given type.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns true if this instance is of the given type,
    otherwise it returns false.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString aType
    /** type */
  ) const;


  ///
  DfsCString getType() const;
  /** {\large Purpose} \\
  Query the type of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a string representation of the type.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool operator<(
  /** {\large Purpose} \\
  Query whether this instance is of lower value than that of a reference.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is of lower value than
  that of a given reference.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Arguments}\\
  */
  ///
  const DfcValue& aRef
  /** referenc instance */
  ) const;


  ///
  CorBool operator>(
  /** {\large Purpose} \\
  Query whether this instance is of greater value than that of a reference.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is of greater value than
  that of a given reference.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Arguments}\\
  */
  ///
  const DfcValue& aRef
  /** referenc instance */
  ) const;


  ///
  CorBool operator==(
  /** {\large Purpose} \\
  Query whether this instance is of equal value than that of a reference.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is of equal value than
  that of a given reference.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Arguments}\\
  */
  ///
  const DfcValue& aRef
  /** referenc instance */
  ) const;


  ///
  CorBool operator!=(
  /** {\large Purpose} \\
  Query whether this instance is of different value than that of a reference.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is of different value than
  that of a given reference.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Arguments}\\
  */
  ///
  const DfcValue& aRef
  /** referenc instance */
  ) const;


  ///
  virtual void stream(
    /** {\large Purpose} \\
    Stream the contents of this instance through a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    CorStream& aStream
    /** stream to stream the value through */
  );

#if 0
  ///
  virtual void writeIdf(
    /** {\large Purpose} \\
    Save the contents of this instance to an Instrument Description File.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    CorStream& aStream
    /** stream to save the value to */
  ) const;


  ///
  virtual CorBool readIdf(
    /** {\large Purpose} \\
    Load the contents of this instance from an Instrument Description File.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE on success.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    CorStream& aStream
    /** stream to load the value from */
  );
#endif

  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:

  ///
  void clear();
  /** {\large Purpose} \\
  Clear out the values and set this instance to uninitialized.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


private:
  friend void utest_DfcValue(CorUnitTest&); // the unit test function

  CorBool   mInitialized;
  DfsCString mType;

  union {
    int i;
    double d;
    DfsCString* s;
    CorPersistent* p;
  }         mVal;

};

#endif // DFC_VALUE_H
