//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Quality Control
//
//$Revision: 1.2 $
//$Log: QucDbManager.h,v $
//Revision 1.2  2004/10/26 15:58:06  nferguso
//temp
//
//Revision 1.1  2004/09/27 14:28:01  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 15:01:27  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/08/25 14:11:48  cguirao
//First check-in
//
//Revision 1.5  2002/07/04 14:06:24  dataflow
//Upgrade to GNU G++ 3.0.4. CG
//
//Revision 1.4  1998/06/30 09:19:38  dataflow
//Removing $Name
//
//Revision 1.3  1997/12/03 15:10:25  dfadmin
//Updates for NTT Jan 98
//
//Revision 1.4  1997/10/01 14:23:59  amanning
//*** empty log message ***
//
//Revision 1.3  1997/04/24 08:48:51  mwiedmer
//provide getShortPath() through keywords
//
//Revision 1.2  1997/04/11 14:10:47  mwiedmer
//use RWCString
//
//Revision 1.1  1997/04/10 09:37:45  dfadmin
//Initial revision
//
//Revision 1.1  1997/04/07 09:01:11  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef QUC_DBMANAGER_H
#define QUC_DBMANAGER_H

// includes from this library
// includes from other ESO libraries
#include <DfcDbManager.h>
// includes from other libraries
// system includes


// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

/// A simple database manager for the calibDB 
class QucDbManager : public DfcDbManager {
  /**
  {\footnotesize $Id: QucDbManager.h,v 1.2 2004/10/26 15:58:06 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <QucDbManager.h>
  
    QucDbManager myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class allows access to the calibration database files.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:


  /// 
  QucDbManager(
    /** {\large Purpose} \\
    The c'tor. Connects to a given server and database.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aDatabase = "",
    /** database to connect to */
    ///
    const DfsCString& aServer = ""
    /** server to connect to */
  );
    

  ///
  virtual ~QucDbManager();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual DfsCString getDbRoot();
  /** {\large Purpose} \\
  Query the database root directory.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the string representation of a directory.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  DfsCString getShortPath(
    /** {\large Purpose} \\
    Query the shortest possible path to a data file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the string representation of the path.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    const DfsCString& aPath
    /** path to shorten */
  );


protected:

  /// 
  QucDbManager(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const QucDbManager& aRef
    /** the reference instance */
  );
    

  /// 
  QucDbManager& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const QucDbManager& aRef
    /** the reference instance */
  );
    

private:
  friend void utest_QucDbManager(CorUnitTest&);  // the unit test function

};

#endif // QUC_DBMANAGER_H
