//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: Quality Control
//
//$Revision: 1.2 $
//$Log: QucDbSelection.h,v $
//Revision 1.2  2004/10/26 15:58:06  nferguso
//temp
//
//Revision 1.1  2004/09/27 14:28:01  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 15:01:27  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/08/25 14:11:48  cguirao
//First check-in
//
//Revision 1.9  2002/07/04 14:06:26  dataflow
//Upgrade to GNU G++ 3.0.4. CG
//
//Revision 1.8  1998/06/30 09:19:38  dataflow
//Removing $Name
//
//Revision 1.7  1997/12/03 15:10:25  dfadmin
//Updates for NTT Jan 98
//
//Revision 1.11  1997/10/01 14:23:59  amanning
//*** empty log message ***
//
//Revision 1.10  1997/09/15 09:29:19  mwiedmer
//fix static instantiation sequence
//
//Revision 1.9  1997/05/07 07:49:03  mwiedmer
//some member functions have become virtual,
//const member variable has lost its const'ness
//
//Revision 1.8  1997/04/24 08:48:52  mwiedmer
//provide getShortPath() through keywords
//
//Revision 1.7  1997/04/22 14:00:55  mwiedmer
//add addAndSelection()
//
//Revision 1.6  1997/04/15 15:51:17  mwiedmer
//use operator LIKE as default and new algorithm for KW_DPR_CALIB
//
//Revision 1.5  1997/04/15 09:33:39  mwiedmer
//add keywords
//
//Revision 1.4  1997/04/11 14:10:48  mwiedmer
//use RWCString
//
//Revision 1.1  1997/04/10 09:37:45  dfadmin
//Initial revision
//
//Revision 1.3  1997/04/07 09:23:44  mwiedmer
//add new keywords
//
//Revision 1.2  1997/04/07 09:03:47  mwiedmer
//use proper keys, add key KW_DATA
//
//Revision 1.1  1997/04/04 14:44:27  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef QUC_DBSELECTION_H
#define QUC_DBSELECTION_H

// includes from this library
// includes from other ESO libraries
#include <DfcDbSelection.h>
// includes from other libraries
// system includes
#include "DfsTValVector.h"

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

/// A selection calss
class QucDbSelection : public DfcDbSelection {
  /**
  {\footnotesize $Id: QucDbSelection.h,v 1.2 2004/10/26 15:58:06 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <QucDbSelection.h>
  
    QucDbSelection myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class allows selections on databases.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{DfcDbSelection}\\
  \Ref{DfcDb}\\
  \Ref{DfcDbData}\\
  \Ref{DfcDbManager}\\ */

public:

  // database the keys are defined for
  static const DfsCString Db;  // calibDB

  // the query keys
  static int KW_TELESCOP;      // telescope
  static int KW_INS_ID;        // instument id
  static int KW_INS_MODE;      // instument mode
  static int KW_INS_PATH;      // instument path
  static int KW_OPTI_ID;       // optical id
  static int KW_OPTI_NAME;     // optical name
  static int KW_OPTI_TYPE;     // optical type
  static int KW_OBS_TECH;      // observation technique
  static int KW_OBS_TYPE;      // observation type
  static int KW_DET_TYPE;      // detector type
  static int KW_DET_NAME;      // detector name
  static int KW_REDUCTION;     // reduction rules or recipes
  static int KW_DPR_PHYSICAL;  // physical data
  static int KW_DPR_CALIB;     // calibration frames
  static int KW_DPR_DICB;      // dicb
  static int KW_DATA;          // file name
  static int KW_PATH_LONG;     // long path
  static int KW_PATH_SHORT;    // short path


  /// 
  QucDbSelection(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    int aKeyword = DfcDbSelection::KW_ANY,
    /** keyword */
    ///
    const DfsCString& aValue = VAL_ANY,
    /** value */
    ///
    int anOperator = DfcDbSelection::OP_LIKE
    /** operator */
  );
    

  /// 
  QucDbSelection(
    /** {\large Purpose} \\
    The c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments} \\
    */
    ///
    const DfsCString& aSelection
    /** full selection string */
  );
    

  /// 
  QucDbSelection(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const QucDbSelection& aRef
    /** the reference instance */
  );
    

  /// 
  QucDbSelection& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const QucDbSelection& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~QucDbSelection();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  virtual void select(
    /** {\large Purpose} \\
    Create a selection on a keyword-value pair.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns something?

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int aKeyword = DfcDbSelection::KW_ANY,
    /** keyword */
    ///
    const DfsCString& aValue = VAL_ANY,
    /** value */
    ///
    int anOperator = DfcDbSelection::OP_LIKE
    /** operator */
  );


  ///
  virtual void addAndSelection(
    /** {\large Purpose} \\
    Add a selection, ANDing it to this.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfcDbSelection* aSelection
    /** Another value */
  );


  virtual int getNextKey();
  virtual const DfsCString& QucDbSelection::getKey(int anIndex);

protected:
private:
  friend void utest_QucDbSelection(CorUnitTest& aTest); // test function

  static DfsTValVector<DfsCString> setKeys();
  static DfsTValVector<DfsCString> mKeys;   // all keys

};

#endif // QUC_DBSELECTION_H
