//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: QucInstrumentKeywords.h,v $
//Revision 1.1  2004/09/27 14:28:03  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 15:01:28  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/08/25 14:11:49  cguirao
//First check-in
//
//Revision 1.3  2002/07/04 14:06:31  dataflow
//Upgrade to GNU G++ 3.0.4. CG
//
//Revision 1.2  1998/06/30 09:19:46  dataflow
//Removing $Name
//
//Revision 1.1  1996/12/20 11:41:12  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef QUC_INSTRUMENT_KEYWORDS_H
#define QUC_INSTRUMENT_KEYWORDS_H

// includes from this library
#include "DfcKeyword.h"

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

class QucInstrumentKeywords;

///
QucInstrumentKeywords operator+(const QucInstrumentKeywords &first, const QucInstrumentKeywords &second);



///
class QucInstrumentKeywords {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: QucInstrumentKeywords.h,v 1.1 2004/09/27 14:28:03 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
  \end{verbatim}

  {\large DESCRIPTION} \\
  Basically contains a list of DfcKeywordInterface objects (not the Structures though).

  {\large EXAMPLES} \\
  \begin{verbatim}
  \end{verbatim}

  {\large DESIGN DECISIONS} \\
  Too many levels of indirection here. If efficiency is a problem then it can easily be made better (maybe with the loss of some safety). This could easily be derived from one of the Keyword hierarchy classes although while I'm not sure of exactly what the place of this class is I have not derived it.
*/

public:
  ///
  QucInstrumentKeywords(void);

  ///
  QucInstrumentKeywords(
  ///
  const DFC_STRING &name
  );

  ///
  QucInstrumentKeywords(
  ///
  DfcKeyword *realKeyword
  );
  // throw();
  /**
  {\large Purpose}\\
  Takes a keyword structure and wraps it up so that we have a class which we can use independently of the keyword structures. 
   */

  ///
  QucInstrumentKeywords(const QucInstrumentKeywords &other);
  /**
  {\large Purpose}
  Copy constructor. Performs a deep copy of everything contained within this object.
   */

  ///
  DfcKeyword *operator()(
  ///
  int position
  /**
  The element's index. 
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct(1)(1)(3).report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeyword *operator()(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  DfcKeyword *getKeyword(
  ///
  int position
  /**
  The element's index. 
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct(1)(1)(3).report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeyword *getKeyword(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void addCopyOfKeyword(
  /// 
  DfcKeywordInterface &newKeyword
  /**
  A reference to the Keyword to be added into the current Keyword. This Keyword will be copied into the KeywordStructure so the responsibility for the deletion of this argument still lies with the method invoker. Likewise, further changes of this argument will not result in the newly added Keyword being changed. 
  */
  );
  // throw(DfcAlreadyExistsExc, DfcTypeExc);
  /** {\large Purpose} \\
  Add a new Keyword (Structure or Data) to the current Keyword.
 
  {\large Preconditions}\\
  The current Keword should really be a KeywordStructure if this method is to succeed.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - The name already exists in the current Keyword. Names should be unique in any given KeywordStructure.
  \item DfcTypeExc - the Keyword that we're attempting to add the Keyuword into does not contain Keywords !?!
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void deleteKeyword(
  ///
  const DFC_STRING &target
  /**
  The keyword to find and remove from the current Keyword.
   */
  );
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find and remove the specified Keyword from the current Keyword.
 
  {\large Preconditions}\\
  the current Keyword should be a KeywordStructure object.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExist - the requested Keyword for removal was not found in the current Keyword (it may be that this Keyword does not contain other Keywords).
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  int getNumKeywords(void) const;
  // throw();
  /** {\large Purpose} \\
  Returns the number of Keywords that this object contains. 

  {\large Returns}\\
  the number of keywords.
  */

  ///
  DFC_STRING report(void) const;


  ///
  DFC_STRING getName(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the name of this Keyword.
   */

  ///
  DFC_STRING getDescription(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the description field of this Keyword which is free form textual thing describing something about the Keyword.
   */

  ///
  void setDescription(
  ///
  const DFC_STRING &newDesc
  /**
  A new descriptive text string for the Keyword.
   */
  );
  // throw();
  /**
  {\large Purpose}\\
  Set a descriptive text field in the Keyword. This forms part of the report produced by the report() method.
  */

  ///
  void setName(
  ///
  const DFC_STRING &newName
  /**
  The new name for the Keyword.
   */
  );
  // throw(DfcAlreadyExistsExc);
  /** {\large Purpose} \\
  Set the name of the Keyword. Each Keyword in a KeywordStructure must have a unique name, this is enforced.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - the new name for the Keyword already exists within the containing Keyword which is not allowed. The attempt to set the name has failed and the name remains as it was.
  \end{itemize}

  {\large Arguments}\\
  */

  
  ///
  void setAlias(const DFC_STRING &nerwAlias);

  ///
  DFC_STRING getAlias(void) const;


  ///
  QucInstrumentKeywords &operator=(
    const QucInstrumentKeywords &other
  );
  /**
  {\large Purpose}
  Assignment operator. Performs a deep copy of everything contained within this object, that is all sub-objects etc.
   */

  ///
  QucInstrumentKeywords &operator+=(const QucInstrumentKeywords &other);
  /**
  {\large Purpose}\\
  Adds another list TO this one. Returns a reference to this object.
   */

  ///
  virtual ~QucInstrumentKeywords(void);  
protected:

private:
  DfcKeyword *keywords;
};

#endif // QUC_INSTRUMENT_KEYWORDS_H
