package org.eso.ohs.phase2.orang;

import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.phase2.timeline.TimelineTask;
import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCFInterval;
import org.eso.ohs.phase2.visibility.VisibilityCalcUtils;

/**
 * This class maintains information regarding an ObservationBlock's visibility 
 * and ranking/schedule information.
 * 
 * Created on Jul 12, 2005
 *  * @author pnunes
 *
 */
public class OBScheduleInfo implements TimelineTask {

	private static final double TIME_INTERVAL_THRESOLD = 0.5;

	/*
	 * ObservationBlock
	 */
	private ObDataContainer ob_;
	
	/*
	 * PCFs used for plotting with visiplots
	 */
	private PCF resultPCF_;		
	private PCF airmassPCF_;
	private PCF meetsAirmassConstraintPCF_;	
	private PCF sunDownPCF_;
	private PCF moonUpPCF_;	
	private PCF moonAnglePCF_;
	private PCF fliPCF_;	
	private PCF execTimePCF_;	
	private PCF timeIntervalPCF_;
	private PCF siderealTimeIntervalPCF_;	
    private PCF meetsZenithConstraintPCF_;
    
    
    private double [] result_;
    private double [] airmass_;
    private double [] meetsAirmassConstraint_;
    private double [] sunDown_;
    private double [] moonUp_;
    private double [] moonAngle_;
    private double [] fli_;
    private double [] execTime_;
    private double [] timeInterval_;
    private double [] siderealTimeInterval_;
    private double [] meetsZenithConstraint_;
    
    
	
	/** Start date of the period over which computation is performed */
	private Date periodStart_;
    
    /** End date of the period over which computation is performed */
	private Date periodEnd_;
    
    /** Length of the interval for which computation is performed, in minutes */
	private int computeInterval_;
	
    /** Scheduled start time for this observation */
	protected Date schedStart;
    
    /** Scheduled end time for this observation */
    protected Date schedEnd;
    
	/*
	 * Intervals in which is observable
	 */
	private LinkedList<PCFInterval> observIntervals_;
	
	
	
	
	/*
	 * Ranking score
	 */
	private String rankingScore_;

	/*
	 * Ranking description
	 */
	private String rankingDesc_;

  	
	
	/**
	 * Constructs de object
	 */
	public OBScheduleInfo() {
		observIntervals_ = new LinkedList<PCFInterval>();
	}	
	
	public OBScheduleInfo( ObservationBlock ob ) {
		this();
		ob_ = new ObDataContainer(ob);
	}
	
	/*
	 * Add an interval in which this OB is observable
	 * @param start
	 * @param end
	 */
	public void addObservableInterval(Date start, Date end) {
		PCFInterval interval = new PCFInterval(start,end,0.0);
		observIntervals_.add(interval);		
	}
	
	/*
	 * Returns an array of Intervals within which the OB is observable
	 * @return PCFInterval [] 
	 */
	public PCFInterval [] getObservableIntervals() {
		PCFInterval [] intervals = new PCFInterval[observIntervals_.size()];
		
		int i = 0;
		for (Iterator<PCFInterval> iter = observIntervals_.iterator(); iter.hasNext();) {
			PCFInterval interval = iter.next();
			intervals[i] = interval;
			i++;
		}		
		return intervals;
	}
	
	
	/*
	 * Setter and getter methods
	 */	
	public PCF getAirmassPCF() {
		if (airmassPCF_ == null) {
			airmassPCF_ = new PCF();
			airmassPCF_.buildFromArray(periodStart_,computeInterval_,airmass_);
		}		
		return airmassPCF_;
	}

    
    /** @see #periodEnd_ */
	public Date getEnd() {
		return periodEnd_;
	}
    
    /** @see #periodEnd_ */
	public void setEnd( Date end ) {
        this.periodEnd_ = end;
    }
    
    /** @see #computeInterval_ */
	public int getInterval() {
		return computeInterval_;
	}
    
    /** @see #computeInterval_ */
	public void setInterval( int interval ) {
        this.computeInterval_ = interval;
    }
    
	public PCF getMoonAnglePCF() {
		if (moonAnglePCF_ == null) {
			moonAnglePCF_ = new PCF();
			moonAnglePCF_.buildFromArray(periodStart_, computeInterval_,moonAngle_);
		}
		return moonAnglePCF_;
	}

	public ObDataContainer getOb() {
		return ob_;
	}
	public void setOb(ObDataContainer ob) {
		this.ob_ = ob;
	}
	public boolean isObservable() {
        boolean result = false;  
                if ( observIntervals_.size() > 0 ){
                    result = true;
                } else {
                    result = false;
                }
		return result;
	}
	
	public PCF getResultPCF() {
		if (resultPCF_ == null) {
			resultPCF_ = new PCF();
			resultPCF_.buildFromArray(periodStart_, computeInterval_,result_);
		}
		return resultPCF_;
	}

    
    /** @see #periodStart_ */
	public Date getStart() {
		return periodStart_;
	}
    
    /** @see #periodStart_ */
	public void setStart( Date start ) {
        this.periodStart_ = start;
    }

	public PCF getFliPCF() {
		if (fliPCF_ == null) {
			fliPCF_ = new PCF();
			fliPCF_.buildFromArray(periodStart_, computeInterval_,fli_);
		}
		return fliPCF_;
	}

	
	public PCF getExecTimePCF() {
		if (execTimePCF_ == null) {
			execTimePCF_ = new PCF();
			execTimePCF_.buildFromArray(periodStart_, computeInterval_,execTime_);
		}
		
		return execTimePCF_;
	}
	
		
	
	public PCF getSiderealTimeIntervalPCF() {
		if (siderealTimeIntervalPCF_ == null) {
			siderealTimeIntervalPCF_ = new PCF();
			siderealTimeIntervalPCF_.buildFromArray(periodStart_, computeInterval_,siderealTimeInterval_);
		}
		
		return siderealTimeIntervalPCF_;
	}

	public PCF getTimeIntervalPCF() {
		
		if (timeIntervalPCF_ == null) {
			timeIntervalPCF_ = new PCF();
			timeIntervalPCF_.buildFromArray(periodStart_, computeInterval_,timeInterval_);
		}
		
		return timeIntervalPCF_;
	}

	public PCF getMoonUpPCF() {
		if (moonUpPCF_ == null) {
			moonUpPCF_ = new PCF();
			moonUpPCF_.buildFromArray(periodStart_, computeInterval_,moonUp_);
		}
		return moonUpPCF_;
	}

	public PCF getSunDownPCF() {
		if (sunDownPCF_ == null) {
			sunDownPCF_ = new PCF();
			sunDownPCF_.buildFromArray(periodStart_, computeInterval_,sunDown_);
		}
		return sunDownPCF_;
	}


	public String getRankingScore() {
		return rankingScore_;
	}

	public void setRankingScore(String rankingScore) {
		this.rankingScore_ = rankingScore;
	}	
	
	/**
	 * Determine the part of the interval in which the date falls. 
	 * Returns 0 if it is the beginning and 1 if it is in the end of the interval 
	 * @return a double between 0 and 1 if the date falls within a time interval, otherwise returns -1
	 */
	public double whereInInterval(Date now) {
		
		double res = -1;
		
		TimeInterval [] intervals = ob_.getTimeIntervals();						
		for (int i = 0; i < intervals.length; i++) {
			TimeInterval ti = intervals[i];
			
			Date startTime =
                new Date(ti.getStartTime() * VisibilityCalcUtils.MILLISECS_PER_SEC);
            Date endTime =
                new Date(ti.getEndTime() * VisibilityCalcUtils.MILLISECS_PER_SEC);
            
            if ( startTime.getTime() <= now.getTime() )
            	if (now.getTime() <= endTime.getTime()) {
            		
            		double a =  now.getTime()-startTime.getTime();
            		double b = endTime.getTime()-startTime.getTime();
            		
            		res = a/b;            		
            	}
		}
		
		return(res);
		
	}
	
	public boolean isSecondHalfTimeInterval() {
		
		double critical = whereInInterval(periodStart_);
		
		if ( critical > TIME_INTERVAL_THRESOLD )
			return(true);
		else
			return(false);
	}
	
	
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("OB #");
		buf.append(getOb().getObInfo().getId());
		buf.append("\n");
		
		
		buf.append("-- FLI: \n");
		buf.append(getFliPCF().toStringCommas());
		buf.append("-- FLI: \n");
		
		buf.append("-- Moon Angle: \n");
		buf.append(getMoonAnglePCF().toStringCommas());
		buf.append("-- Moon Angle: \n");
		
		buf.append("-- Moon Down: \n");
		buf.append(getMoonUpPCF().toStringCommas());
		buf.append("-- Moon Down: \n");
		
		buf.append("-- Sun down: \n");
		buf.append(getSunDownPCF().toStringCommas());
		buf.append("-- Sun down: \n");
		
		buf.append("-- TI: \n");
		buf.append(getTimeIntervalPCF().toStringCommas());
		buf.append("-- TI: \n");
		
		buf.append("-- STI: \n");
		buf.append(getSiderealTimeIntervalPCF().toStringCommas());
		buf.append("-- STI: \n");
		
		buf.append("-- Airmass: \n");
		buf.append(getSunDownPCF().toStringCommas());
		buf.append("-- Airmass: \n");
		
		buf.append("-- Result Observable intervals: " );
		buf.append(observIntervals_.size());
		buf.append("\n");
		buf.append(getResultPCF().toStringCommas());
		buf.append("-- Result: \n");
		
		return(buf.toString());
	}
	
	
    

    /**
     * @return zenith constraint PCF
     */
    public PCF getMeetsZenithConstraintPCF() {
    	
    	if (meetsZenithConstraintPCF_ == null) {
    		meetsZenithConstraintPCF_ = new PCF();
    		meetsZenithConstraintPCF_.buildFromArray(periodStart_, computeInterval_,meetsZenithConstraint_);
		}
        return meetsZenithConstraintPCF_;
    }




	public void setAirmass(double[] airmass_) {
		this.airmass_ = airmass_;
	}



	public void setExecTime(double[] execTime_) {
		this.execTime_ = execTime_;
	}



	public void setFli(double[] fli_) {
		this.fli_ = fli_;
	}



	public void setMeetsAirmassConstraint(double[] meetsAirmassConstraint_) {
		this.meetsAirmassConstraint_ = meetsAirmassConstraint_;
	}



	public void setMeetsZenithConstraint(double[] meetsZenithConstraint_) {
		this.meetsZenithConstraint_ = meetsZenithConstraint_;
	}



	public void setMoonAngle(double[] moonAngle_) {
		this.moonAngle_ = moonAngle_;
	}



	public void setMoonUp(double[] moonUp_) {
		this.moonUp_ = moonUp_;
	}



	public void setResult(double[] result_) {
		this.result_ = result_;
	}



	public void setSiderealTimeInterval(double[] siderealTimeInterval_) {
		this.siderealTimeInterval_ = siderealTimeInterval_;
	}



	public void setSunDown(double[] sunDown_) {
		this.sunDown_ = sunDown_;
	}



	public void setTimeInterval(double[] timeInterval_) {
		this.timeInterval_ = timeInterval_;
	}

	/**
     * @see TimelineTask#getDuration()
     */
    // amchavan, 20-Sep-2005: added
    public long getDuration() {
        long t = getOb().getObInfo().getExecutionTime();
        return t;
    }

    /**
     * @see TimelineTask#getID()
     */
    // amchavan, 20-Sep-2005: added
    public long getID() {
        return getOb().getObInfo().getDbaseId();
    }

    /**
     * @see TimelineTask#getPreference()
     */
    // amchavan, 20-Sep-2005: added
    public PCF getPreference() {
        return getResultPCF();
    }

    /**
     * @see TimelineTask#getStartTime()
     */
    // amchavan, 20-Sep-2005: added
    public Date getStartTime() {
        return this.schedStart;
    }

    /**
     * @see TimelineTask#setStartTime(Date)
     */
    // amchavan, 20-Sep-2005: added
    public void setStartTime( Date start ) {
        this.schedStart = start;
        if( start != null )
            this.schedEnd = new Date( start.getTime() + getDuration() * 1000 );
    }

    /**
     * @see TimelineTask#getName()
     */
    // amchavan, 20-Sep-2005: added
    public String getName() {
        return this.ob_.getObInfo().getName();
    }

    /**
     * @see TimelineTask#compareTo(TimelineTask)
     */
    // amchavan, 20-Sep-2005: added
    public int compareTo( TimelineTask anotherTask ) {
        OBScheduleInfo other = (OBScheduleInfo) anotherTask;
        return getRankingScore().compareTo( other.getRankingScore() );
    }

    /**
     * @see TimelineTask#isDoable()
     */
    // amchavan, 20-Sep-2005: added
    public boolean isDoable() {
        return isObservable();
    }
    public boolean isExecutable() {
    boolean result = false;
       Date startDate = getStart();
       for (int i = 0; i < observIntervals_.size(); i++) {
           result = false;
           PCFInterval interval = observIntervals_.get(i);
       // if the starting of the execution is within the observability interval, 
         // the object is observable.
           if (!startDate.before(interval.getStart())
                   && !startDate.after(interval.getEnd())) {
               result = true;
               break;
           }
    } 
    return result;
    }
    /**
     * @see TimelineTask#getEndTime()()
     */
    // amchavan, 12-Oct-2005: added
    public Date getEndTime() {
        return this.schedEnd;
    }
    
    /**
     * The equals checks if an object has the same obId properties
     * 
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
    	boolean result = false;
    	if (obj!= null&& obj instanceof OBScheduleInfo) {
    		OBScheduleInfo currentOb = (OBScheduleInfo) obj;
    		result = (this.getID() == currentOb.getID());
    	}
    	return result;
    }

	public PCF getMeetsAirmassConstraintPCF() {
		
		if (meetsAirmassConstraintPCF_ == null) {
			meetsAirmassConstraintPCF_ = new PCF();
			meetsAirmassConstraintPCF_.buildFromArray(periodStart_, computeInterval_,meetsAirmassConstraint_);
		}
		
		return meetsAirmassConstraintPCF_;
	}



	public String getRankingDesc() {
		return rankingDesc_;
	}

	public void setRankingDesc(String rankingDesc) {
		this.rankingDesc_ = rankingDesc;
	}

    
   
}
