package org.eso.ohs.phase2.orang;

import java.util.Date;

import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.WeatherConstraint;
import org.eso.ohs.dfs.WindInfo;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.phase2.visibility.JulianDate;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;
import org.eso.ohs.phase2.visibility.VisicalcOptions;

/**
 * This class is a default implementation of the OrangModel
 * 
 * @author pnunes
 *
 */
public class OrangModelDefault implements OrangModel {
	
	// ob selection
	private int obSelectionType;
    private String selectedTelescope;
    private String selectedInstrument; 
    private String [] selectedQueues;

    private boolean obSelectionTypeChanged;
    private boolean selectedQueuesChanged;
    private boolean selectedInstrumentChanged;    
    private boolean selectedTelescopeChanged;
    
    private String[] currentQueueTel;
    private String[] currentQueueInst;  
    
    // weather 
    private boolean seeingEnabled;
    private boolean windEnabled;
    private boolean skyTranspEnabled;

    private int wind;
    private String skyTranspValue;                                  
    private String seeing;                                  
    private String maxSeeing;                                  

    private boolean seeingEnabledChanged;
      private boolean windEnabledChanged;
    private boolean skyTranspEnabledChanged;
    private boolean  seeingMinValueChanged;
    private boolean  seeingMaxValueChanged;
    private boolean  windValueChanged;
    private boolean  skyTranspChanged;

    
    // visibility
    private boolean fliConstrain;
    private boolean moonDistanceConstrain;  
    private boolean sunAngleConstrain;
    private boolean moonAngleConstrain;
    private boolean timeConstrain;
    private boolean sidTimeConstrain;
    private boolean zenithConstrain;
    private boolean executionConstrain;
    
    private boolean fliConstrainChanged; 
    private boolean moonDistanceConstrainChanged; 
    private boolean sunAngleConstrainChanged;
    private boolean moonAngleConstrainChanged;
    private boolean timeConstrainChanged;
    private boolean sidTimeConstrainChanged;
    private boolean zenithConstrainChanged;
    private boolean executionCheckConstrainChanged;

    
    private int fliRelaxPercentage;
    private int moonRelaxPercentage;   
    private int airmassValuePercentage; 
    private int sunAngleDegree;
    private int moonAngleDegree;
    
    private boolean fliRelaxPercentageChanged; 
    private boolean moonRelaxPercentageChanged; 
    private boolean airmassValuePercentageChanged;   
    private boolean sunAngleDegreeChanged;
    private boolean moonAngleDegreeChanged;
    
    private String user;
    private Date startDate;
    private Date endDate;
        
    private boolean startDateChanged;
    private boolean endDateChanged;
    
    private int interval;
    private boolean intervalChanged;

   
    private String duration;
    private boolean durationChanged;
    
    private double latitude_;
    private double longitude_; 
    
       
    
    //ranking
    private String rankingAlgorithm; 
    private boolean rankingAlgorithmChanged;
    
    
    // timeline
    private String timelineAlgorithm;
    private boolean timelineAlgorithmChanged;
    
    
    // widgets data
    private OrangWidgetsData data;
  
    
    public OrangModelDefault(OrangWidgetsData data) {
    	this.data = data;
        setDefaults();
        setModelChanged(false);
    }
    
    

    private void setDefaults() { 
    	obSelectionType = OrangModel.OB_SELECTION_WRONG;
    	selectedTelescope = "";
    	selectedInstrument = "";
    	selectedQueues = new String[0];
    	currentQueueTel = new String[0];
    	currentQueueInst = new String[0];
    	
    	seeingEnabled = true;
    	windEnabled = true;
    	skyTranspEnabled = true;
    	
        seeing = String.valueOf(Seeing.MIN_SEEING);
        maxSeeing = String.valueOf(Seeing.MAX_SEEING);
        	wind = WindInfo.DEFAULT_WIND_ANGLE;
    	skyTranspValue = SkyTransparency.PHOTOMETRIC;
    	
    	
    	fliConstrain = true; 
        moonDistanceConstrain = true;     
        sunAngleConstrain = true;
        moonAngleConstrain = true;  
        timeConstrain = false;
        sidTimeConstrain = false;
        zenithConstrain = true; 
        executionConstrain = true; 
    	
        fliRelaxPercentage = 0; 
        moonRelaxPercentage = 0;   
        airmassValuePercentage = 0;   
        sunAngleDegree = OrangModel.DEFAULT_SUN_ELEV;
        moonAngleDegree = OrangModel.DEFAULT_MOON_ELV;
        
        interval = OrangModel.DEFAULT_INTERVAL;
        duration = "4";
        rankingAlgorithm = "";  
        timelineAlgorithm = "";
        
        startDate = new Date();
        endDate = new Date();
        
        Config cfg = Config.getCfg();
        user = String.valueOf(cfg.getAccountId());
        
        this.latitude_ = AppConfig.getAppConfig().getLatitude();
        this.longitude_ = AppConfig.getAppConfig().getLongitude();
        
        
		
	}



	public String getSelectedTelescope() {
        return selectedTelescope;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setObSelectionTelescope(java.lang.String)
     */
    public void setSelectedTelescope(String obSelectionTelescope) {    	
    	setSelectedTelescopeChanged(!this.selectedTelescope.equals(obSelectionTelescope));
        this.selectedTelescope = obSelectionTelescope;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getObSelectionType()
     */
    public int getObSelectionType() {
        return obSelectionType;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setObSelectionType(int)
     */
    public void setObSelectionType(int obSelectionType) {        
    	setObSelectionTypeChanged( obSelectionType != this.obSelectionType);
    	this.obSelectionType = obSelectionType;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getSeeingValue()
     */
    public Seeing getSeeingValue() {
        Seeing seeingValue = new Seeing(Seeing.MIN_SEEING);
        if( seeing != null ) {
            if (!seeing.equals("")) {
                seeingValue = new Seeing(seeing);
            }
        }
        return seeingValue;
    } 
    
    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getSeeingValue()
     */
    public Seeing getMaxSeeingValue() {
        Seeing seeingValue = new Seeing(Seeing.MAX_SEEING);
        if( maxSeeing != null ) {
            if (! maxSeeing.equals("")) {
                seeingValue = new Seeing(maxSeeing);
            }
        }
        return seeingValue;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getWindValue()
     */
    public WindInfo getWindValue() {
        WindInfo windInfoValue = new WindInfo(wind);    
        return windInfoValue;
    }




    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getSkyTransp()
     */
    public SkyTransparency getSkyTranspValue() {
    	SkyTransparency skyTransp = new SkyTransparency(SkyTransparency.PHOTOMETRIC);
    	if( skyTranspValue != null) {
    		if (! skyTranspValue.equals(""))
    		skyTransp = new SkyTransparency(skyTranspValue);
    	}

        return skyTransp;
    	
    }




    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getAirmassValuePercentage()
     */
    public int getAirmassValuePercentage() {
        return airmassValuePercentage;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setAirmassValuePercentage(int)
     */
    public void setAirmassValuePercentage(int airmassValuePercentage) {
    	setAirmassValuePercentageChanged(airmassValuePercentage != this.airmassValuePercentage);
        this.airmassValuePercentage = airmassValuePercentage;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#isFliConstrain()
     */
    public boolean isFliConstrain() {
        return fliConstrain;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setFliConstrain(boolean)
     */
    public void setFliConstrain(boolean fliConstrain) {
    	setFliConstrainChanged(this.fliConstrain != fliConstrain);
        this.fliConstrain = fliConstrain;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getFliRelaxPercentage()
     */
    public int getFliRelaxPercentage() {
        return fliRelaxPercentage;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setFliRelaxPercentage(int)
     */
    public void setFliRelaxPercentage(int fliRelaxPercentage) {
    	setFliRelaxPercentageChanged(this.fliRelaxPercentage != fliRelaxPercentage);
        this.fliRelaxPercentage = fliRelaxPercentage;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#isMoonAngleConstrain()
     */
    public boolean isMoonAngleConstrain() {
        return moonAngleConstrain;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setMoonAngleConstrain(boolean)
     */
    public void setMoonAngleConstrain(boolean moonAngleConstrain) {
    	setMoonAngleConstrainChanged(this.moonAngleConstrain != moonAngleConstrain);
        this.moonAngleConstrain = moonAngleConstrain;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getMoonAngleDegree()
     */
    public int getMoonAngleDegree() {
        return moonAngleDegree;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setMoonAngleDegree(int)
     */
    public void setMoonAngleDegree(int moonAngleDegree) {
    	setMoonAngleDegreeChanged(this.moonAngleDegree != moonAngleDegree);
        this.moonAngleDegree = moonAngleDegree;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#isMoonDistanceConstrain()
     */
    public boolean isMoonDistanceConstrain() {
        return moonDistanceConstrain;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setMoonDistanceConstrain(boolean)
     */
    public void setMoonDistanceConstrain(boolean moonDistanceConstrain) {
    	setMoonDistanceConstrainChanged(this.moonDistanceConstrain != moonDistanceConstrain);
        this.moonDistanceConstrain = moonDistanceConstrain;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getMoonRelaxPercentage()
     */
    public int getMoonRelaxPercentage() {
        return moonRelaxPercentage;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setMoonRelaxPercentage(int)
     */
    public void setMoonRelaxPercentage(int moonRelaxPercentage) {
    	setMoonRelaxPercentageChanged(this.moonRelaxPercentage != moonRelaxPercentage);
        this.moonRelaxPercentage = moonRelaxPercentage;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#isSunAngleConstrain()
     */
    public boolean isSunAngleConstrain() {
        return sunAngleConstrain;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setSunAngleConstrain(boolean)
     */
    public void setSunAngleConstrain(boolean sunAngleConstrain) {
    	setSunAngleConstrainChanged(this.sunAngleConstrain != sunAngleConstrain);
        this.sunAngleConstrain = sunAngleConstrain;
        
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getSunAngleDegree()
     */
    public int getSunAngleDegree() {
        return sunAngleDegree;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setSunAngleDegree(int)
     */
    public void setSunAngleDegree(int sunAngleDegree) {
    	setSunAngleDegreeChanged(this.sunAngleDegree != sunAngleDegree);
        this.sunAngleDegree = sunAngleDegree;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#setRankingAlgorithm(java.lang.String)
     */
    public void setRankingAlgorithm(String rankingAlgorithm) {
    	setRankingAlgorithmChanged(!this.rankingAlgorithm.equals(rankingAlgorithm));
        this.rankingAlgorithm = rankingAlgorithm;
        
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getRankingAlgorithm()
     */
    public String getRankingAlgorithm() {
        return rankingAlgorithm;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModelInterface#getVisiCalcOptions()
     */
    public VisicalcOptions getVisiCalcOptions() {
    	VisicalcOptions options = new VisicalcOptions();
        
    	options.setCalcAirmass(true);    	
        options.setCalcFli(isFliConstrain()); 
        options.setCalcMoonDist(isMoonDistanceConstrain()); 

        options.setCalcSun(isSunAngleConstrain());
        options.setCalcMoon(isMoonAngleConstrain());
        options.setCalcTimeInterval(isTimeIntervalConstrain());
        options.setCalcSiderealTimeInterval(isSidTimeIntervalConstrain());
        
        options.setFliTolerance(getFliRelaxPercentage());         
        options.setMoonDistanceTolerance(getMoonRelaxPercentage());
        options.setAirmassTolerance(getAirmassValuePercentage());   
        options.setSunConstraint(getSunAngleDegree());
        options.setMoonConstraint(getMoonAngleDegree());
        
         return options;
    }


    public boolean isTimeIntervalConstrain() {
        return timeConstrain;
    }


    public boolean isSidTimeIntervalConstrain() {
        return sidTimeConstrain;
    }
    
    public void setTimeIntervalConstrain(boolean value) {
    	setTimeConstrainChanged(this.timeConstrain != value);
        this.timeConstrain = value;
    }


    public void setSidTimeIntervalConstrain(boolean value) {
    	setSidTimeConstrainChanged(this.sidTimeConstrain != value);
        this.sidTimeConstrain = value;
    }


    public String getUser() {     
        return user;
    }


    public boolean isSuperUser() {
        return false;
    }


    /**
     * @param user The user to set.
     */
    public void setUser(String user) {    	
        this.user = user;
    }


    public Date getStartDate() {     
        return startDate;
    }


    public Date getEndDate() {
        return endDate;
    }



    public void setStartDate(Date date) {  
    	setStartDateChanged(!this.startDate.equals(date));
    	startDate = date;
    }


    public void getEndDate(Date date) {
         endDate = date;
    }


    public void getStepInterval(int step) {
         interval = step;
    }


    public double getLatitude() {
     
        return latitude_;
    }


    public double getLongitude() {
        return longitude_;
    }

    public void setLatitude(double latitude) {    	
       latitude_ = latitude;
    }


    public void setLongitude(double longitude) {    	
        longitude_ = longitude;
    }


    public String getEndISODate() {
     
        return Convert.dateToIsoDate(getEndDate());
    }


    public String getStartISODate() {
       return Convert.dateToIsoDate(getStartDate());
    }


	public String[] getSelectedQueues() {
		return(selectedQueues);
	}


	public void setSelectedQueues(String[] selectedQueues) {
		boolean changed = arrayChanged(this.selectedQueues,selectedQueues);
		
		setSelectedQueuesChanged(changed);
		this.selectedQueues = selectedQueues;
	}


	private boolean arrayChanged(String [] oldArray , String[] newArray ) {
		boolean changed = false;
		
		if ( oldArray.length != newArray.length )
			changed = true;
		else {
			for (int i = 0; i < newArray.length; i++) {
				String str = newArray[i];
				
				// find the string, if its not there it changed
				if ( !find(str, oldArray) )
					changed = true;
			}
		}
		return changed;
	}


    private boolean find(String queue, String[] strings) {
		boolean result = false;
		for (int i = 0; i < strings.length; i++) {
			if ( queue.equals(strings[i]) )
				result = true;
		}
		
		return result;
	}


	
    public String getTimelineAlgorithm() {
        return timelineAlgorithm;
    }

    
    public void setTimelineAlgorithm( String timelineAlgorithm ) {
    	setTimelineAlgorithmChanged(!this.timelineAlgorithm.equals(timelineAlgorithm));
        this.timelineAlgorithm = timelineAlgorithm;
    }



    /**
     * @param currentQueueTel void
     */
    public void setQueuesTelescopes(String[] currentQueueTel) {    	
        this.currentQueueTel = currentQueueTel;        
    }


    /**
     * @param currentQueueInst void
     */
    public void setQueuesInstruments(String[]  currentQueueInst) {    	
        this.currentQueueInst = currentQueueInst;
        
    }


    /**
     * @return Returns the weatherConstraint.
     */
    public WeatherConstraint getWeatherConstraint() {
        Seeing seeing = getSeeingValue();
        Seeing maxSeeing = getMaxSeeingValue();
           WindInfo windInfo = getWindValue();
        	SkyTransparency skyTransp = getSkyTranspValue();    	
    	
        WeatherConstraint constraint = new WeatherConstraint(seeing,maxSeeing,skyTransp,windInfo,getSeeingEnabled(),getSkyTranspEnabled(),getWindEnabled());
        
        return(constraint);
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModel#getQueuesTelescopes()
     */
    public String[] getQueuesTelescopes() {
    	String tel = getSelectedTelescope();
        String[] queueTel = new String[0];
        if (tel != null) {
            queueTel = data.getQueueTel(tel);
        }
        return queueTel;
    }


    /* (non-Javadoc)
     * @see org.eso.ohs.phase2.apps.ot.wizard.OrangModel#getQueuesInstruments()
     */
    public String[] getQueuesInstruments() {
    	 String instrument = getSelectedInstrument();
         String[] queueInst = new String[0];
         if (instrument != null) {
             queueInst = data.getQueueInst(instrument);
         }
         return queueInst;
    }


    /**
     * 
     * @param verifyZenith defines if the zenith constraint must be verified.
     * 
     */
    public void setTelescopeZenithConstraint(boolean verifyZenith) {  
        setZenithConstrainChanged(this.zenithConstrain != verifyZenith);
        this.zenithConstrain = verifyZenith;
    }

    public boolean isTelescopeZenithConstraint() {      
        return zenithConstrain;
    }

    /**
     * 
     * @param verifyZenith defines if the zenith constraint must be verified.
     * 
     */
    public void setExecutionCheck(boolean verifyExecution) {  
        setExecutionCheckChanged(this.executionConstrain != verifyExecution);
        this.executionConstrain = verifyExecution;
    }
    public boolean isExecutionConstraint() {      
        return executionConstrain;
    }


    


	public boolean obSelectionChanged() {
		return (obSelectionTypeChanged || selectedQueuesChanged
				|| selectedInstrumentChanged || selectedTelescopeChanged);
	}


    public boolean weatherChanged() {
        boolean isChanged = false;
        if ((seeingMinValueChanged || seeingMaxValueChanged || seeingEnabledChanged)|| 
        (skyTranspChanged || skyTranspEnabledChanged)|| 
        (windValueChanged || windEnabledChanged || startDateChanged)) {
            isChanged = true;
        }
         return isChanged; 
         }


	public boolean visibilityConstraintChanged() {
		return(  				
				
				startDateChanged ||
				endDateChanged ||
				intervalChanged ||
				durationChanged ||
				
				airmassValuePercentageChanged || 				
				fliRelaxPercentageChanged ||
				moonRelaxPercentageChanged||
				sunAngleDegreeChanged ||
				moonAngleDegreeChanged ||
				
				fliConstrainChanged ||	    
				moonDistanceConstrainChanged ||								
				moonAngleConstrainChanged ||								
				sunAngleConstrainChanged ||				
				timeConstrainChanged ||
				sidTimeConstrainChanged ||                      
                zenithConstrainChanged  ||                      
                executionCheckConstrainChanged  							
				
		);
	}


	public boolean rankAlgorithmChanged() {		
		return rankingAlgorithmChanged;
	}
	
	
	
	public boolean isAirmassValuePercentageChanged() {
		return airmassValuePercentageChanged;
	}


	public void setAirmassValuePercentageChanged(
			boolean airmassValuePercentageChanged) {		
		this.airmassValuePercentageChanged = airmassValuePercentageChanged;
	}


	public boolean isFliConstrainChanged() {
		return fliConstrainChanged;
	}


	public void setFliConstrainChanged(boolean fliConstrainChanged) {
		this.fliConstrainChanged = fliConstrainChanged;
	}


	public boolean isFliRelaxPercentageChanged() {
		return fliRelaxPercentageChanged;
	}


	public void setFliRelaxPercentageChanged(boolean fliRelaxPercentageChanged) {
		this.fliRelaxPercentageChanged = fliRelaxPercentageChanged;
	}


	public boolean isIntervalChanged() {
		return intervalChanged;
	}


	public void setIntervalChanged(boolean intervalChanged) {
		this.intervalChanged = intervalChanged;
	}



	public boolean isMoonAngleConstrainChanged() {
		return moonAngleConstrainChanged;
	}


	public void setMoonAngleConstrainChanged(boolean moonAngleConstrainChanged) {
		this.moonAngleConstrainChanged = moonAngleConstrainChanged;
	}


	public boolean isMoonAngleDegreeChanged() {
		return moonAngleDegreeChanged;
	}


	public void setMoonAngleDegreeChanged(boolean moonAngleDegreeChanged) {
		this.moonAngleDegreeChanged = moonAngleDegreeChanged;
	}


	public boolean isMoonDistanceConstrainChanged() {
		return moonDistanceConstrainChanged;
	}


	public void setMoonDistanceConstrainChanged(boolean moonDistanceConstrainChanged) {
		this.moonDistanceConstrainChanged = moonDistanceConstrainChanged;
	}


	public boolean isMoonRelaxPercentageChanged() {
		return moonRelaxPercentageChanged;
	}


	public void setMoonRelaxPercentageChanged(boolean moonRelaxPercentageChanged) {
		this.moonRelaxPercentageChanged = moonRelaxPercentageChanged;
	}



	public boolean isObSelectionTypeChanged() {
		return obSelectionTypeChanged;
	}


	public void setObSelectionTypeChanged(boolean obSelectionTypeChanged) {
		this.obSelectionTypeChanged = obSelectionTypeChanged;
	}


	public boolean isRankingAlgorithmChanged() {
		return rankingAlgorithmChanged;
	}


	public void setRankingAlgorithmChanged(boolean rankingAlgorithmChanged) {
		this.rankingAlgorithmChanged = rankingAlgorithmChanged;
	}


    public boolean isSeeingMinValueChanged() {
        return seeingMinValueChanged;
    }


    public void setSeeingMinValueChanged(boolean seeingValueChanged) {
        this.seeingMinValueChanged = seeingValueChanged;
    }

    public void setSeeingMaxValueChanged(boolean seeingValueChanged) {
        this.seeingMaxValueChanged = seeingValueChanged;
    }


    public boolean isSeeingMaxValueChanged() {
        return seeingMaxValueChanged ;
    }


    public boolean isWindValueChanged() {
        return windValueChanged;
    }


    public void setWindValueChanged(boolean windValueChanged) {
        this.windValueChanged = windValueChanged;
    }


	public boolean isSelectedQueuesChanged() {
		return selectedQueuesChanged;
	}


	public void setSelectedQueuesChanged(boolean selectedQueuesChanged) {
		this.selectedQueuesChanged = selectedQueuesChanged;
	}


	public boolean isSidTimeConstrain() {
		return sidTimeConstrain;
	}


	public void setSidTimeConstrain(boolean sidTimeConstrain) {
		setSidTimeConstrainChanged(this.sidTimeConstrain != sidTimeConstrain);
		this.sidTimeConstrain = sidTimeConstrain;
	}


	public boolean isSidTimeConstrainChanged() {
		return sidTimeConstrainChanged;
	}


	public void setSidTimeConstrainChanged(boolean sidTimeConstrainChanged) {
		this.sidTimeConstrainChanged = sidTimeConstrainChanged;
	}


	public boolean isSkyTranspChanged() {
		return skyTranspChanged;
	}


	public void setSkyTranspChanged(boolean skyTranspChanged) {		
		this.skyTranspChanged = skyTranspChanged;
	}


	public boolean isStartDateChanged() {
		return startDateChanged;
	}


	public void setStartDateChanged(boolean startDateChanged) {
		this.startDateChanged = startDateChanged;
	}



	public boolean isSunAngleConstrainChanged() {
		return sunAngleConstrainChanged;
	}


	public void setSunAngleConstrainChanged(boolean sunAngleConstrainChanged) {
		this.sunAngleConstrainChanged = sunAngleConstrainChanged;
	}


	public boolean isSunAngleDegreeChanged() {
		return sunAngleDegreeChanged;
	}


	public void setSunAngleDegreeChanged(boolean sunAngleDegreeChanged) {
		this.sunAngleDegreeChanged = sunAngleDegreeChanged;
	}


	public boolean isTimeConstrainChanged() {
		return timeConstrainChanged;
	}


	public void setTimeConstrainChanged(boolean timeConstrainChanged) {
		this.timeConstrainChanged = timeConstrainChanged;
	}


	public boolean isTimelineAlgorithmChanged() {
		return timelineAlgorithmChanged;
	}


	public void setTimelineAlgorithmChanged(boolean timelineAlgorithmChanged) {
		this.timelineAlgorithmChanged = timelineAlgorithmChanged;
	}



    public boolean isZenithConstrainChanged() {
        return zenithConstrainChanged;
    }


    public void setZenithConstrainChanged(boolean zenithConstrainChanged) {
        this.zenithConstrainChanged = zenithConstrainChanged;
    }

    public boolean isExecutionCheckChanged() {
        return executionCheckConstrainChanged;
    }


    public void setExecutionCheckChanged(boolean executionCheckConstrainChanged) {
        this.executionCheckConstrainChanged = executionCheckConstrainChanged;
    }


	public void setModelChanged(boolean changed) {
		obSelectionTypeChanged = changed;
		selectedInstrumentChanged = changed;
		selectedQueuesChanged = changed;
		selectedTelescopeChanged = changed;
		
		seeingEnabledChanged = changed;
		windEnabledChanged = changed;
		skyTranspEnabledChanged = changed;
        seeingMinValueChanged = changed;
        seeingMaxValueChanged = changed;
		windValueChanged = changed;
		skyTranspChanged = changed;
		
		
		fliConstrainChanged = changed;
		moonDistanceConstrainChanged = changed;
		sunAngleConstrainChanged = changed;
		moonAngleConstrainChanged = changed;
		timeConstrainChanged = changed;
		sidTimeConstrainChanged = changed;		
		
		fliRelaxPercentageChanged = changed;
		moonRelaxPercentageChanged = changed;
		airmassValuePercentageChanged = changed;
		sunAngleDegreeChanged = changed;
		moonAngleDegreeChanged = changed;
		
		startDateChanged = changed;
		intervalChanged = changed;
		durationChanged = changed;
		rankingAlgorithmChanged = changed;
		timelineAlgorithmChanged = changed;
	}


	public boolean timelineAlgorithmChanged() {
		return(this.timelineAlgorithmChanged);
	}




	public String getSelectedInstrument() {
		return(selectedInstrument);
	}



	public void setSelectedInstrument(String selectedInstrument) {
		setSelectedInstrumentChanged( ! this.selectedInstrument.equals(selectedInstrument)  );
		this.selectedInstrument = selectedInstrument;
	}



	private void setSelectedInstrumentChanged(boolean b) {
		this.selectedInstrumentChanged = b;
		
	}

	public String getDuration() {
		return duration;
	}

	public void setDuration(String duration) {
		setDurationChanged(!this.duration.equals(duration));
		this.duration = duration;
	}

	public int getStepInterval() {
		return interval;
	}

	public void setStepInterval(int interval) {
		setIntervalChanged(this.interval != interval);
		this.interval = interval;
	}

    public boolean isSeeingEnabled() {
        return seeingEnabled;
    }

    public void setSeeingEnabled(boolean seeingEnabled) {
        setSeeingEnabledChanged(this.seeingEnabled != seeingEnabled);
        this.seeingEnabled = seeingEnabled;
    }

    public boolean isSeeingEnabledChanged() {
        return seeingEnabledChanged;
    }

    public void setSeeingEnabledChanged(boolean seeingEnabledChanged) {
        this.seeingEnabledChanged = seeingEnabledChanged;
    }

    public boolean isSelectedTelescopeChanged() {
        return selectedTelescopeChanged;
    }

    public boolean isWindEnabled() {
        return windEnabled;
    }

    public void setWindEnabled(boolean windEnabled) {
        setWindEnabledChanged(this.windEnabled != windEnabled);
        this.windEnabled = windEnabled;
    }

    public boolean isWindEnabledChanged() {
        return windEnabledChanged;
    }

    public void setWindEnabledChanged(boolean windEnabledChanged) {
        this.windEnabledChanged = windEnabledChanged;
    }


	public void setSelectedTelescopeChanged(boolean selectedTelescopeChanged) {
		this.selectedTelescopeChanged = selectedTelescopeChanged;
	}

	public boolean getSkyTranspEnabled() {
		return skyTranspEnabled;
	}

	public void setSkyTranspEnabled(boolean skyTranspEnabled) {
		setSkyTranspEnabledChanged(this.skyTranspEnabled != skyTranspEnabled);
		this.skyTranspEnabled = skyTranspEnabled;
	}

	public boolean isSkyTranspEnabledChanged() {
		return skyTranspEnabledChanged;
	}

	public void setSkyTranspEnabledChanged(boolean skyTranspEnabledChanged) {
		this.skyTranspEnabledChanged = skyTranspEnabledChanged;
	}


	public boolean isZenithConstrain() {
		return zenithConstrain;
	}

	public void setZenithConstrain(boolean zenithConstrain) {
		setZenithConstrainChanged(this.zenithConstrain != zenithConstrain);
		this.zenithConstrain = zenithConstrain;
	}

	public String[] getCurrentQueueInst() {
		return currentQueueInst;
	}

	public String[] getCurrentQueueTel() {
		return currentQueueTel;
	}

	public boolean isSelectedInstrumentChanged() {
		return selectedInstrumentChanged;
	}

	public void setEndDate(Date endDate) {
		setEndDateChanged(!this.endDate.equals(endDate));
		this.endDate = endDate;
	}

	public boolean isDurationChanged() {
		return durationChanged;
	}

	public void setDurationChanged(boolean durationChanged) {		
		this.durationChanged = durationChanged;
	}

	public String getSkyTransp() {
		return skyTranspValue;
	}

	public void setSkyTransp(String skyTranspValue) {
		setSkyTranspChanged(!this.skyTranspValue.equals(skyTranspValue));
		this.skyTranspValue = skyTranspValue;
	}

     public String getSeeing() {
            return seeing;
        } 
     public String getMaxSeeing() {
            return maxSeeing;
        }

     public void setSeeing(String seeing) {
         setSeeingMinValueChanged(!this.seeing.equals(seeing));
         this.seeing = seeing;
     }
     public void setMaxSeeing(String seeing) {
         setSeeingMaxValueChanged(!this.maxSeeing.equals(seeing));
         this.maxSeeing = seeing;
     }

    public boolean getSeeingEnabled() {
        return(seeingEnabled);
    
    }

    
    public int getWind() {
        return wind;
    }

    public void setWind(int wind) {        
       setWindValueChanged(getWind() != wind); 
       this.wind = wind;
    }

    public boolean getWindEnabled() {
        return(windEnabled);
    
    }



	public OrangWidgetsData getWidgetsData() { 
		return data;
	}



	public boolean optionsChanged() {
		return (obSelectionChanged() || weatherChanged()
				|| visibilityConstraintChanged()
				|| rankAlgorithmChanged() || timelineAlgorithmChanged());
	}



	public boolean isEndDateChanged() {
		return endDateChanged;
	}



	public void setEndDateChanged(boolean endDateChanged) {
		this.endDateChanged = endDateChanged;
	}


	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.orang.OrangModel#getLST()
	 */
	public double getLST() {

		Date start = this.getStartDate();
		double jdate = JulianDate.getJDC(start);
		double longitude = this.getLongitude();
		VisibilityJCalc visi = new VisibilityJCalc();
		
		double lst = visi.getLST(jdate,longitude);
		
		// convert from radians to hours
		lst = Math.toDegrees(lst)/15;
		return( lst );
	}



	public double getFLI() {
		Date start = this.getStartDate();
		double jdate = JulianDate.getJDC(start);
		double longitude = this.getLongitude();
		double latitude = this.getLatitude();
		VisibilityJCalc visi = new VisibilityJCalc();
		return( visi.getFLIValue(jdate,longitude,latitude) );
    }




}
