package org.eso.ohs.phase2.orang;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DBdataHelper;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;
import org.eso.ohs.phase2.visibility.VisibilityCalcUtils;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;

public class OrangWidgetsData {
	
	
    public static final String ALL_NIGHT = "All Night";

    public static final int STEP_INTERVAL_DEFAULT = 10;

    public static final int SUN_DAWN_DEFAULT = -18;

    /**
     * Field stdlog_
     */
    private static Logger stdlog_ = Logger.getLogger(OrangWidgetsData.class);

    private static final String[] allNightArray =    { ALL_NIGHT };

    private String[] telescopeList = new String[0];

    private String[] queueList =new String[0];

    private String[] instrumentList = new String[0];

    private HashMap<String, TreeSet<String>> instQueueAssociation;

    private HashMap<String, TreeSet<String>> telQueueAssociation;

    public OrangWidgetsData() {
        super();
        Config cfg = Config.getCfg();
        try {
            queueList = DBdataHelper.getInstance().readStringArray(
                    Phase2SelectStmt.selectQueues(String.valueOf(cfg
                            .getAccountId()), cfg.isSuperUser()));
        } catch (ObjectIOException e) {
            stdlog_.error("No queues found ", e);
        }
        
        if (instrumentList == null || instrumentList.length == 0) {
            OHSConfig config = AppConfig.getInstance();
            String regexp = config.getValue("ORANG.TODAY.INST.PATTERN");
            int instPatternIndex = config.getInteger("ORANG.TODAY.INST.INDEX",
                    1);
            Set<String> instrumentTodayQueues = new TreeSet<String>();
            Pattern pattern = Pattern.compile(regexp);

            for (int i = 0; i < queueList.length; i++) {
                String currentQueue = queueList[i];
                
                Matcher matcher = pattern.matcher(currentQueue);
                if (matcher.matches()
                        && matcher.groupCount() > instPatternIndex) {
                    String instrument = matcher.group(instPatternIndex);
                    instrumentTodayQueues.add(instrument);
                    stdlog_.debug("Instrument today queue: "+instrument);
                    addQueueInstAssociation(instrument,currentQueue);
                }

            }
            instrumentList = instrumentTodayQueues
                    .toArray(new String[instrumentTodayQueues.size()]);
        }
        
        if (telescopeList == null || telescopeList.length == 0) {
            String[] instrumentList = getInstruments();

            String nowDate = getCurrentDate();
             try {
                String[][] telInstList = DBdataHelper.getInstance().readDoubleStringArray(
                        Phase2SelectStmt.selectActiveTelInsAssociation(instrumentList,
                                nowDate));
                Set<String> telList = new TreeSet<String>();
                for ( int i =0 ; i < telInstList.length; i++){
                    String[] telInstAssociation = telInstList[i];
                    String currentTelescope = telInstAssociation[0];
                    String currentInstrument = telInstAssociation[1];
                    addQueueTelAssociation(currentTelescope,currentInstrument);
                telList.add(currentTelescope); 
               }
                telescopeList = telList.toArray(new String[telList.size()]);
            } catch (ObjectIOException e) {
                stdlog_.error("no tel list", e);
                MsgManager.errmsg("Telescope List not retrieved");
            }
        }
    }

    private void addQueueInstAssociation(String instrument, String currentQueue) {
        if (instQueueAssociation == null) {
            instQueueAssociation = new HashMap<String, TreeSet<String>> ();
        }
        TreeSet<String> queueSet =  instQueueAssociation.get(instrument);
        if (queueSet == null) {
            stdlog_.debug(" added instrument "+ instrument);
            queueSet = new TreeSet<String>();
            instQueueAssociation.put(instrument,queueSet);
        }
        queueSet.add(currentQueue);
    }
    
    private void addQueueTelAssociation(String tel, String inst) {
        if (telQueueAssociation == null) {
            telQueueAssociation = new HashMap<String, TreeSet<String>> ();
        }
        TreeSet<String> queueSet = telQueueAssociation.get(tel);
        if (queueSet == null) {
            queueSet = new TreeSet<String>();
            telQueueAssociation.put(tel,queueSet);
        }
        TreeSet<String> queueInstSet = instQueueAssociation.get(inst);
        if (queueInstSet != null) {
           queueSet.addAll(queueInstSet);
        }
    }

    /**
     * retrieve the telescopes having today queues
     * 
     * @return String[]
     */
    public String[] getTelescopes() {
        return telescopeList;
    }

    public String[] getQueues() { 
        return queueList;
    }
    
    public String[] getQueueInst() { 
        return queueList;
    }

    public String getStartISODate() {
        Calendar currentCal = Calendar.getInstance();
        String dateStart = Convert.longToISODateMilliSec(currentCal
                .getTimeInMillis());
        return dateStart;
    }
    
    public String getCurrentDate() {
        Calendar currentCal = Calendar.getInstance();
        String dateStart = Convert.longToDBDateMilliSec(currentCal
                .getTimeInMillis());
        return dateStart;
    }

    public String getStepInterval() {
        return String.valueOf(STEP_INTERVAL_DEFAULT);
    }

    public String[] getAlgorithmList() throws OrangScriptException {
        OBRankingEngine engine = OrangScriptingBox.getInstance();
        OHSConfig config = AppConfig.getInstance();
        String scriptDir = config.getValue(OrangScriptingBox.scriptDirectory);
        engine.setup(scriptDir);
        String[] algorithmList = engine.getAlgorithmNames();
        return algorithmList;
    }

    /**
     * retrieve the instruments having today queues. It uses
     * ORANG.QUEUE.TODAY.INST_PATTERN and ORANG.QUEUE.TODAY.INST_INDEX for
     * retrieving the instruments from the queues.
     * 
     * @return String[]
     */
    public String[] getInstruments() {
        return instrumentList;
    }

    public String[] getStepRangeList() {
        OHSConfig config = AppConfig.getInstance();
        String[] itemList = config.getValueStringArray("ORANG.STEP.RANGE", " ");
        return itemList;
    }

    public String[] getDurationRangeList() {
        OHSConfig config = AppConfig.getInstance();
        String[] itemList = config.getValueStringArray("ORANG.DURATION.RANGE",
                " ");
        itemList = TextUtils.concatenateStringArrays(itemList, allNightArray);
        return itemList;
    }

    public String[] getSkyTranspList() {
        return SkyTransparency.skyTransparencyDbRange;
    }

    public String[] getQueueTel(String tel) { 
        TreeSet<String> queueSet = telQueueAssociation.get(tel);
        String[] queueList =  queueSet.toArray(new String[queueSet.size()]);
        return queueList;
      } 
    public String[] getQueueInst(String inst) { 
          TreeSet<String> queueSet = instQueueAssociation.get(inst);
          stdlog_.debug(queueSet+" instrument selected: "+inst);
          String[] queueList =queueSet.toArray(new String[queueSet.size()]);
          return queueList;
        }

    public static Date convertGUIisoDate(String isoDate) {
		Date date = null;
	    
	    try {
	    	date = Convert.ISODateToDate(isoDate);
	    } catch (ParseException e) {
	        e.printStackTrace();
	    }
	    return date;
	}

	public static String getISOEndDate(String duration, Date startDate) {
		String isoStringDate = null;
	    Date endDate;               
	    Calendar cal = Calendar.getInstance();
	    if (OrangWidgetsData.ALL_NIGHT.equals(duration)) {
	        cal.setTime(startDate);
	        VisibilityJCalc calc = new VisibilityJCalc();
	        endDate = calc.getSunRiseTime(OrangWidgetsData.SUN_DAWN_DEFAULT,
	                OrangWidgetsData.STEP_INTERVAL_DEFAULT, cal.getTime(),
	                VisibilityCalcUtils.PLONG, VisibilityCalcUtils.PLAT);
	    } else {
	        
	        cal.setTime(startDate);
	        cal.add(Calendar.HOUR, Integer.parseInt(duration));
	        endDate = cal.getTime();
	    }
	    isoStringDate = Convert.longToISODateMilliSec(endDate.getTime());
	    return isoStringDate;
	}

}
