package org.eso.ohs.phase2.visibility;

import java.util.HashMap;

import org.apache.commons.lang.builder.CompareToBuilder;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.mhuss.AstroLib.AstroDate;

/**
 * Class VisibilityCache
 * 
 * Caches visibility information.
 * Assumes constant observatory coordinates. The cache keys don't take into
 * account change of latitude and longitude.
 * 
 * @version $Revision: 1.5 $
 * @author ddorigo, Jul 8, 2005
 */

// $Id: VisibilityCache.java,v 1.5 2006/05/08 12:41:33 pnunes Exp $
public class VisibilityCache {

    /*
     * Cache of values
     */
//    protected HashMap<ComparableAstroDate, Double> moonRA_;
//    protected HashMap<ComparableAstroDate, Double> moonDEC_;
//    protected HashMap<ComparableAstroDate, Double> sunElevations_;
//    protected HashMap<ComparableAstroDate, Double> moonElevations_;
//    protected HashMap<ComparableAstroDate, Double> localSTimes_;
//    protected HashMap <CoordsDate,Double> airmasses_;    
//    protected HashMap <CoordsDate,Double> moonDistances_;
    
    
    protected HashMap moonRA_;
    protected HashMap moonDEC_;
    protected HashMap sunElevations_;
    protected HashMap moonElevations_;
    protected HashMap localSTimes_;
    protected HashMap airmasses_;    
    protected HashMap moonDistances_;
    
    
    
    private int cacheHits_;
    private int noHits_;
    

    
    
    private static VisibilityCache visibilityCache_;

    /**
     * Initializes the class
     */
    protected VisibilityCache() {
//        moonRA_ = new HashMap<ComparableAstroDate, Double>();
//        moonDEC_ = new HashMap<ComparableAstroDate, Double>();
//        sunElevations_ = new HashMap<ComparableAstroDate, Double>();
//        moonElevations_ = new HashMap<ComparableAstroDate, Double>();
//        localSTimes_ = new HashMap<ComparableAstroDate, Double>();
//
//        airmasses_ = new  HashMap <CoordsDate,Double>();
//        moonDistances_ = new  HashMap <CoordsDate,Double>();
        
        
        moonRA_ = new HashMap();
        moonDEC_ = new HashMap();
        sunElevations_ = new HashMap();
        moonElevations_ = new HashMap();
        localSTimes_ = new HashMap();

        airmasses_ = new  HashMap();
        moonDistances_ = new  HashMap();
        
        cacheHits_ = 0;
        noHits_ = 0;
    }

    public static VisibilityCache getInstance() {
        if (visibilityCache_ == null)
            visibilityCache_ = new VisibilityCache();

        return (visibilityCache_);

    }

    /**
     * Clears the chache 
     */
    public void clear() {
    	airmasses_.clear();
    	moonDistances_.clear();
        moonRA_.clear();
        moonDEC_.clear();
        sunElevations_.clear();
        moonElevations_.clear();
        localSTimes_.clear();
    }

    /**
     * Returns LST
     * 
     * @param jdate
     * @return Double or null if not cached
     */
    public Double getLST(double jdate) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double sidTime = (Double) localSTimes_.get(jdate);
        
        if (sidTime != null) {
        	cacheHits_ ++;
        } else {
        	noHits_++;
        }
        
        return sidTime;
    }

    /**
     * Returns sun elevation
     * 
     * @param jdate
     * @return Double or null if not cached
     */
    public Double getSunElevation(double jdate) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double sunElev = (Double) sunElevations_.get(jdate);
        if (sunElev != null) {
        	cacheHits_ ++;
        } else {
        	noHits_++;
        }
        return sunElev;
    }

    /**
     * Returns moon elevation 
     * @param jdate
     * @return Double or null if not cached
     */
    public Double getMoonElevation(double jdate) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonElev = (Double) moonElevations_.get(jdate);
        
        if (moonElev != null) {
        	cacheHits_ ++;
        }else {
        	noHits_++;
        }
        return moonElev;
    }

    /**
     * Returns moon RA
     * 
     * @param jdate
     * @return Double  or null if not cached
     */
    public Double getMoonRa(double jdate) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonRa = (Double) moonRA_.get(jdate);
        
        if (moonRa != null) {
        	cacheHits_ ++;
        }else {
        	noHits_++;
        }
        return moonRa;
    }

    /**
     * Returns moon DEC
     * @param jdate
     * @return Double or null if not cached
     */
    public Double getMoonDec(double jdate) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonDec = (Double) moonDEC_.get(jdate);
        
        if (moonDec != null) {
        	cacheHits_ ++;
        }else {
        	noHits_++;
        }
        return moonDec;
    }
    
    
    public Double getAirmass(double jdate, double targetRA,double targetDEC) {
    	CoordsDate coords = new CoordsDate(jdate, targetRA, targetDEC);
    	
    	Double airmass = (Double) airmasses_.get(coords);
    	if (airmass != null) {
        	cacheHits_ ++;
        }else {
        	noHits_++;
        }
    	
    	return(airmass);
    }
    
    public void putAirmass(double jdate, double targetRA,double targetDEC , double airmass ) {
    	CoordsDate coords = new CoordsDate(jdate, targetRA, targetDEC);
    	airmasses_.put(coords,airmass);
    }
    
    
    public Double getMoonDistance(double jdate, double targetRA,double targetDEC) {
    	CoordsDate coords = new CoordsDate(jdate, targetRA, targetDEC);
    	
    	Double distance = (Double) moonDistances_.get(coords);
    	if (distance != null) {
        	cacheHits_ ++;
        }else {
        	noHits_++;
        }
    	
    	return(distance);
    }
    
    public void putMoonDistance(double jdate, double targetRA,double targetDEC , double distance ) {
    	CoordsDate coords = new CoordsDate(jdate, targetRA, targetDEC);
    	moonDistances_.put(coords,distance);
    }
    

    public void putLST(double jdate, double lst) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double lstObj = new Double(lst);
        localSTimes_.put(jdate, lst);
    }

    public void putSunElevation(double jdate, double sunEl) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double sunElObj = new Double(sunEl);
        sunElevations_.put(jdate, sunEl);
    }

    public void putMoonElevation(double jdate, double moonEl) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonElObj = new Double(moonEl);
        moonElevations_.put(jdate, moonEl);
    }

    public void putMoonRa(double jdate, double moonRa) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonRaObj = new Double(moonRa);
        moonRA_.put(jdate, moonRa);
    }

    public void putMoonDec(double jdate, double moonDec) {
        ComparableAstroDate date = new ComparableAstroDate(jdate);
        Double moonDecObj = new Double(moonDec);
        moonDEC_.put(jdate, moonDec);
    }

 

    /*
     * 
     * Class AstroDateExt
     * 
     * Represents AstroDateExt with equals
     * 
     * @version $Revision: 1.5 $ @author ddorigo, Jul 8, 2005
     */
    private class ComparableAstroDate extends AstroDate  implements Comparable{
        /**
         * @param jdate
         */
        public ComparableAstroDate(double jdate) {         
           super(jdate);
        }

        public boolean equals(Object obj) {
        	if (obj instanceof ComparableAstroDate == false) {
        		return false;
        	}
        	if (this == obj) {
        		return true;
        	}
        	ComparableAstroDate rhs = (ComparableAstroDate) obj;
        	return new EqualsBuilder()
        	.append(jd(), rhs.jd())        	                 
        	.isEquals();
        }
        	                        

        

        public int hashCode() {
        	return new HashCodeBuilder(11, 39).
        	append(jd()).        	       
        	toHashCode();
        }
    

		public int compareTo(Object o) {
			
			ComparableAstroDate other = (ComparableAstroDate)o;
    		return new CompareToBuilder()		       
    		.append(this.jd(), other.jd())
    		.toComparison();                        		
            
		}
    }

    
    public int getCacheHits() {
		return cacheHits_;
	}
	
	public int getCacheNoHits() {
		return noHits_;
	}

    
  
    
    private class CoordsDate  implements Comparable {
    	
    	private double ra;
    	private double dec;
    	private double jdate;
		
    	
    	public CoordsDate(double jdate, double dec, double ra) {		
			this.jdate = jdate;
			this.dec = dec;
			this.ra = ra;
		}

		
    	public boolean equals(Object obj) {
    		if (obj instanceof CoordsDate == false) {
    			return false;
    		}
    		if (this == obj) {
    			return true;
    		}
    		CoordsDate other = (CoordsDate) obj;
    		return new EqualsBuilder()    		
    		.append(this.ra, other.ra)
    		.append(this.dec, other.dec)
    		.append(this.jdate, other.jdate)
    		.isEquals();
    	}
			 
		
    	public int compareTo(Object o) {
    		CoordsDate other = (CoordsDate)o;
    		return new CompareToBuilder()		       
    		.append(this.ra, other.ra)
    		.append(this.dec, other.dec)
    		.append(this.jdate, other.jdate)
    		.toComparison();
    	}
    	
		
		public int hashCode() {
			return new HashCodeBuilder(17, 37).
		       append(ra).
		       append(dec).
		       append(jdate).
		       toHashCode();
		}

		public double getJDate() {
			return jdate;
		}

		public void setJDate(double jdate) {
			this.jdate = jdate;
		}

		public double getDec() {
			return dec;
		}

		public void setDec(double dec) {
			this.dec = dec;
		}

		public double getRa() {
			return ra;
		}

		public void setRa(double ra) {
			this.ra = ra;
		}


		
		
    	
    	
    	
    	
    }

	
}