package org.eso.ohs.phase2.visiplot;

import java.awt.Color;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.apache.log4j.Logger;
import org.eso.ohs.phase2.actions.AreaColorEditorAction;
import org.eso.ohs.phase2.actions.LabelsColorEditorAction;
import org.eso.ohs.phase2.actions.PerimeterColorEditorAction;
/**
 * Onjects that contains the information about the colors used by a chart
 * @author gdonino
 *
 */
public class ChartColorItem {
	
	
	private JLabel name;
	
	//We don't really need a JButton. Probably we could manage it using Color class
	private JButton perimeter;
	private JButton area;
	private JButton labels;
	private boolean hasLine = false;
	private boolean hasLabels = false;
	private JComponent[] components;
	
	private static Logger stdlog_ = Logger.getLogger(ChartColorItem.class);

	
	/**
	 * Constructor.
	 * @param name The name of the chart
	 * @param hasLine True if the chart uses perimeter and area colors. False if the chart uses only area color.
	 * @param hasLabels True if the chart shows the labels.
	 */
	public ChartColorItem(String name,  boolean hasLine, boolean hasLabels){
		this.hasLine = hasLine;
		this.hasLabels = hasLabels;
		
	
		
		components = new JComponent[4];
		
		this.name = new JLabel();
		perimeter = new JButton();
		area = new JButton();
		labels = new JButton();
		this.name.setText(name);
		perimeter.setEnabled(true);
		area.setEnabled(true);
		labels.setEnabled(true);
		
		String title = "Chart: " + this.name.getText() + " - Choose color for: ";
		
		
		area.addActionListener( new AreaColorEditorAction(this, new JFrame(), title + " plotting line"));
		area.setToolTipText(getToolTipText("area", name));
		
		if (!hasLine){
			perimeter.setEnabled(false);
			perimeter.setVisible(false);
		}else{
			perimeter.addActionListener(new PerimeterColorEditorAction(this, new JFrame(), title + " plotting area"));
			perimeter.setToolTipText(getToolTipText("line", name));
		}
		if (!hasLabels){
			labels.setEnabled(false);
			labels.setVisible(false);
		}else{
			labels.addActionListener(new LabelsColorEditorAction(this, new JFrame(), title + " chart labels"));
			labels.setToolTipText(getToolTipText("labels", name));
		}
		
		components[0] = this.name;
		components[1] = perimeter;
		components[2] = area;
		components[3] = labels;
		
		setChartColorItem(this,true,true,null,null,null);
		
		
	}

	public Color getAreaColor() {
		return area.getBackground();
	}

	public void setAreaColor(Color color) {
		this.area.setBackground(color);
	//	this.area.setForeground(color);
		this.area.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
		stdlog_.debug("Set ("+ this.getName() +") area color to: " + color.toString());
		
	}

	public Color getLabelsColor() {
		return labels.getBackground();
	}

	public void setLabelsColor(Color color) {
		this.labels.setBackground(color);
		//this.labels.setForeground(color);
		this.labels.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
		stdlog_.debug("Set ("+ this.getName() +") labels color to: " + color.toString());
	}

	public String getName() {
		return name.getText();
	}

	public void setName(String name) {
		this.name.setText(name);
	}

	public Color getPerimeterColor() {
		return perimeter.getBackground();
	}

	public void setPerimeterColor(Color color) {
		this.perimeter.setBackground(color);
	//	this.perimeter.setForeground(color);
		this.perimeter.setText(color.getRed() + ", " + color.getGreen() + ", " + color.getBlue());
		stdlog_.debug("Set ("+ this.getName() +") perimeter color to: " + color.toString());
	}

	public boolean hasPerimeter() {
		return hasLine;
	}

	public boolean hasLabels() {
		return hasLabels;
	}

	public JComponent[] getComponents() {
		return components;
	}
	
	private String getToolTipText (String object, String label){
		return "Set the color for the " + object + " of the " + label + " chart";
	}
	
	public Object clone(){
		ChartColorItem result  = new ChartColorItem(name + "Clone", true, true);
		Color areaCol  = this.getAreaColor();
		Color perimeterCol = this.getPerimeterColor();
		Color labelsCol = this.getLabelsColor();
		
		Color newAreaCol = new Color(areaCol.getRed(), areaCol.getGreen(), areaCol.getBlue());
		Color newPerimeterCol = new Color(perimeterCol.getRed(), perimeterCol.getGreen(), perimeterCol.getBlue());
		Color newLabelsColor = new Color(labelsCol.getRed(), labelsCol.getGreen(), labelsCol.getBlue());
		
		result.setAreaColor(newAreaCol);
		result.setPerimeterColor(newPerimeterCol);
		result.setLabelsColor(newLabelsColor);
		
		return result;
	}

	public static int[] getRGBColors(String colorString){
		int[] result = new int[3];
		
		StringTokenizer st = new StringTokenizer(colorString, ",");
		int i = 0;
		while (st.hasMoreElements()){
			result[i++] = Integer.parseInt(st.nextToken());
		}
		
		return result;
	}

	public static ChartColorItem setChartColorItem(ChartColorItem item, boolean hasArea, boolean hasLabels, String lineColor, String areaColor, String labelsColor) {
		ChartColorItem result = item;
		int[] lineRGB;
		int[] areaRGB;
		int[] labelsRGB;
		stdlog_.debug("");
		stdlog_.debug("VPlotOptionsDialog.java: setting ChartColorItem " + item.getName());
		stdlog_.debug("currentPerimeter: " + item.getPerimeterColor().toString() + " lineCol: " + lineColor);
		stdlog_.debug("currentArea: " + item.getAreaColor().toString() + " areaCol: " + areaColor);
		stdlog_.debug("currentLabels: " + item.getLabelsColor().toString() + " labelsCol: " + labelsColor);
		
		if (lineColor != null)
			lineRGB = getRGBColors(lineColor);
		else
			lineRGB = new int[]{80, 80, 80};
		
		if (areaColor != null)
			areaRGB = getRGBColors(areaColor);
		else
			areaRGB = new int[]{0, 0, 0};
		
		if (labelsColor != null)
			labelsRGB = getRGBColors(labelsColor);
		else
			labelsRGB = new int[]{0, 0, 50};
		
		Color lineCol = new Color (lineRGB[0], lineRGB[1], lineRGB[2]);
		Color areaCol = new Color (areaRGB[0], areaRGB[1], areaRGB[2]);
		Color labelsCol = new Color (labelsRGB[0], labelsRGB[1], labelsRGB[2]);
		
		result.setPerimeterColor(lineCol);
		result.setAreaColor(areaCol);
		result.setLabelsColor(labelsCol);
		stdlog_.debug("currentPerimeter: " + item.getPerimeterColor().toString() + " lineCol: " + lineColor);
		stdlog_.debug("currentArea: " + item.getAreaColor().toString() + " areaCol: " + areaColor);
		stdlog_.debug("currentLabels: " + item.getLabelsColor().toString() + " labelsCol: " + labelsColor);
		
		
		return result;
	}

	public static String getColorLineValue(ChartColorItem item) {
		String result;
		int red = item.getPerimeterColor().getRed();
		int green = item.getPerimeterColor().getGreen();
		int blue = item.getPerimeterColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}

	public static  String getColorAreaValue(ChartColorItem item) {
		String result;
		int red = item.getAreaColor().getRed();
		int green = item.getAreaColor().getGreen();
		int blue = item.getAreaColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}

	public static  String getColorLabelsValue(ChartColorItem item) {
		String result;
		int red = item.getLabelsColor().getRed();
		int green = item.getLabelsColor().getGreen();
		int blue = item.getLabelsColor().getBlue();
		result = red + "," + green + "," + blue;	
		return result;
	}
	
	
	
	
	
	
	
	
	
	public void repaint() {
		perimeter.repaint();
		area.repaint();
		labels.repaint();
	}
	
}
