package org.eso.ohs.phase2.visiplot;

/**
 * Dialog for loading, editing and saving visibility plot calculation and
 * display parameters.
 * 
 * @version $Revision: 1.28 $
 * @author Nick Kornweibel for ESO. Created on July 31, 2001, 10:14 AM
 * <br>ChangeLog: cmavros 26/09/01 Add Time Interval menu option 
 */

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.Convert;

@SuppressWarnings("serial")
public class VPlotOptionsDialog extends JDialog {
	
	private JPanel buttonPanel_;
	private JButton okButton_;
	private JButton cancelButton_;
	private JButton loadButton_;
	private JButton saveButton_;
	private JTabbedPane optionsEditorTabPane_;
	private PlotOptionsPanel plotOptionsPanel_;
	private HeaderOptionsPanel headerOptionsPanel_;
	private MiscOptionsPanel miscOptionsPanel_;
	private ColorOptionsPanel colorOptionsPanel_;
	private static JFileChooser fileChooser_;
	private boolean settingsAccepted_ = false;
	private boolean disablePlotOptions_;
	
	/**
	 * Creates new form VPlotOptionsDialog
	 */
	
	public VPlotOptionsDialog(Frame parent, boolean modal, boolean disablePlotOptions) {
		super(parent, modal);
		this.disablePlotOptions_ = disablePlotOptions;
		initComponents(disablePlotOptions);
		setTitle("Visibility Plot Options");
		pack();
	}
			
	/**
	 * Shows the dialog and initializes it with the VisiplotOptions
	 * @param visiplotOptions
	 * @return
	 */
	public void showDialog(VisiPlotOptions visiplotOptions) {
						
		updateGUI(visiplotOptions);
		setVisible(true);
        if (getSettingsAccepted()) {
            saveSettingsFromGUI(visiplotOptions);
        }        
        
        
	}
	
	/**
	 * This method is called from within the constructor to
	 * initialize the form.
	 */	
	private void initComponents(boolean disablePlotOptions) {
		buttonPanel_ = new JPanel();
		okButton_ = new JButton();
		cancelButton_ = new JButton();
		loadButton_ = new JButton();
		saveButton_ = new JButton();
		optionsEditorTabPane_ = new JTabbedPane();
		
		buttonPanel_.setLayout(new FlowLayout());
		okButton_.setText("OK");
		okButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String entries = checkEntries();
				if (entries.length() > 0) {
					showMessage(entries,
					"Please correct settings before proceeding");
				} else {
					closeDialog(true);
				}
			}
		});
		buttonPanel_.add(okButton_);
		cancelButton_.setText("cancel");
		cancelButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeDialog(false);
			}
		});
		buttonPanel_.add(cancelButton_);
		loadButton_.setText("load settings");
		loadButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadSettings();
			}
		});
		buttonPanel_.add(loadButton_);
		saveButton_.setText("save settings");
		saveButton_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveSettings();
			}
		});
		buttonPanel_.add(saveButton_);
		
		getContentPane().add(buttonPanel_, BorderLayout.SOUTH);
		getContentPane().add(optionsEditorTabPane_, BorderLayout.CENTER);
		
		// Create the panels
		plotOptionsPanel_ = new PlotOptionsPanel(disablePlotOptions);
		plotOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		headerOptionsPanel_ = new HeaderOptionsPanel();
		headerOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		miscOptionsPanel_ = new MiscOptionsPanel();
		miscOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		colorOptionsPanel_ = new ColorOptionsPanel(new VisiPlotOptions());
		colorOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));
		
		buttonPanel_.setBorder(
				new EmptyBorder(new Insets(10, 10, 10, 10)));
		

		optionsEditorTabPane_.addTab("plot", plotOptionsPanel_);
		optionsEditorTabPane_.addTab("header", headerOptionsPanel_);
		optionsEditorTabPane_.addTab("advanced", miscOptionsPanel_);
		optionsEditorTabPane_.addTab("colors", colorOptionsPanel_);
	}
	
	/**
	 * Closes the dialog
	 */
	
	private void closeDialog(WindowEvent evt) {
		closeDialog(false);
	}
	
	public ChartColorItem[] getChartColorItems(){
		return	colorOptionsPanel_.getChartColorItems();
	}
	
	private void closeDialog(boolean acceptSettings) {
		settingsAccepted_ = acceptSettings;
		setVisible(false);
		dispose();
	}
	
	private void showMessage(String msg, String title) {
		JOptionPane.showMessageDialog(this,
				new Object[] { msg }, title, JOptionPane.ERROR_MESSAGE);
	}
	
	/**
	 * Inquire whether or not the OK button was pressed.
	 * @return true if ok was pressed.
	 */
	
	public boolean getSettingsAccepted() {
		return settingsAccepted_;
	}
	
	/**
	 * Prompt the user for a settings file and initialize the panel
	 * to the settings in that file.
	 */
	
	private void loadSettings() {
		JFileChooser fc = getFileChooser();
		fc.setDialogTitle("Select settings file to load");
		
		int returnVal = fc.showOpenDialog(this);
		File target = fc.getSelectedFile();
		if (target != null && (returnVal == JFileChooser.APPROVE_OPTION)) {
			try {
				FileReader fr = new FileReader(target);
				
				// keep the old start and end dates
				String startDate = plotOptionsPanel_.getStartTime();
				String endDate = plotOptionsPanel_.getEndTime();
				
				VisiPlotOptions opts = VisiplotOptionsHandler.readFrom(fr);					
				
				
				
				//	put back
				opts.getVisicalcOptions().setStartTime(Convert.ISODateToDate(startDate));
				opts.getVisicalcOptions().setEndTime(Convert.ISODateToDate(endDate));
				
				fr.close();
				
				updateGUI(opts);
				
				
			} catch (IOException e) {
				// show the error to the user
				JOptionPane.showMessageDialog(this,
						new Object[] { "Error reading settings: "+e.getMessage() },
						"Error reading settings",
						JOptionPane.ERROR_MESSAGE
				);
			} catch (ParseException e) {
				// It shouldn't happen 
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Prompt the user to select a file and save the settings to the
	 * selected file.
	 */
	
	private void saveSettings() {
		String entries = checkEntries();
		if (entries.length() > 0) {
			// report to user...fix errors before saving.
			JOptionPane.showMessageDialog(this,
					new Object[] { entries },
					"Please correct settings before saving",
					JOptionPane.ERROR_MESSAGE
			);
		} else {
			JFileChooser fc = getFileChooser();
			fc.setDialogTitle("Select file to save settings");
			fc.showSaveDialog(this);
			File target = fc.getSelectedFile();
			if (target != null) {
				boolean saved = false;
				try {
					FileWriter fw = new FileWriter(target);
					VisiPlotOptions opts = new VisiPlotOptions(); 
						saveSettingsFromGUI(opts);
					VisiplotOptionsHandler.writeTo(fw,opts);
					fw.flush();
					fw.close();
					saved = true;
				} catch (IOException e) {
					// Show the error to the user.
					JOptionPane.showMessageDialog(this,
							new Object[] { "Error saving settings: "
							+e.getMessage() },
							"Error saving settings",
							JOptionPane.ERROR_MESSAGE
					);
				}
				if (saved) {
					// inform user that the settings are saved
					JOptionPane.showMessageDialog(this,
							new Object[] {"Options saved to file." },
							"Saving settings",
							JOptionPane.INFORMATION_MESSAGE
					);
				}
			}
		}
	}
	
	/**
	 * Update the settings object passed, with the values from the dialog
	 * @param the settings object to be updated.
	 */
	
	public void saveSettingsFromGUI(VisiPlotOptions settings) {
				
		
		
		try {
            settings.getVisicalcOptions().setStartTime(
                    Convert.ISODateToDate(plotOptionsPanel_.getStartTime()));
            settings.getVisicalcOptions().setExecutableCheck(
                    plotOptionsPanel_.getExecutableCheck());

			settings.getVisicalcOptions().setEndTime(
					Convert.ISODateToDate(plotOptionsPanel_.getEndTime()));
		
		} catch (ParseException e) {
			// this shouldn't happen, since a validation was made in the form
			e.printStackTrace();
		}
		settings.getVisicalcOptions().setCalcAirmass(
				plotOptionsPanel_.airMassConstraintSelected());		
		settings.setAirmassValue(plotOptionsPanel_.getAirMassConstraint());

		settings.getVisicalcOptions().setCalcExecTime(
				plotOptionsPanel_.execTimeConstraintSelected());
		settings.getVisicalcOptions().setCalcTimeInterval(
				plotOptionsPanel_.timeIntervalConstraintSelected());
		
		settings.getVisicalcOptions().setCalcSiderealTimeInterval(
				plotOptionsPanel_.siderealTimneIntervalConstraintSelected());

		settings.getVisicalcOptions().setCalcFli(
				plotOptionsPanel_.fliConstraintSelected());         
		
		settings.setFliValue(plotOptionsPanel_.getFLIConstraint());
		
		settings.setGroupOBs(plotOptionsPanel_.groupIdentOBsConstraintSelected());
		
		settings.getVisicalcOptions().setCalcMoonDist(
				plotOptionsPanel_.moonDistConstraintSelected());  
		
		settings.setMoonDistanceValue(plotOptionsPanel_.getMoonDistConstraint());
		
		settings.getVisicalcOptions().setCalcSun(plotOptionsPanel_.sunConstraintSelected());
		
		settings.setSunConstraint(plotOptionsPanel_.getSunConstraint());
		

        settings.getVisicalcOptions().setCalcZenithInvisibility(
                plotOptionsPanel_.zenithConstraintSelected());      
       
		
		
		// update the headerOptionsPanel
		
		settings.setReportShowDec(headerOptionsPanel_.getDec());						
		settings.setReportShowExecTime(headerOptionsPanel_.getExecTime());
		settings.setReportShowStartTime(headerOptionsPanel_.getStartTime());		
		settings.setReportShowFLI(headerOptionsPanel_.getFLI());
		settings.setReportShowAirMass(headerOptionsPanel_.getAirMass());
		settings.setReportShowMoonDist(headerOptionsPanel_.getMoonDistance());
		settings.setReportShowOBId(headerOptionsPanel_.getOBId());
		settings.setReportShowProgId(headerOptionsPanel_.getProgId());
		settings.setReportShowRA(headerOptionsPanel_.getRA());
		settings.setReportShowRank(headerOptionsPanel_.getRankClass());
		settings.setReportShowSeeing(headerOptionsPanel_.getSeeing());
		settings.setReportShowSkyTrans(headerOptionsPanel_.getSkyTrans());

		//sort		
		settings.setSortPlot(headerOptionsPanel_.getSortOrder());
	
		
		// update the misc options panel		
		settings.getVisicalcOptions().setInterval(miscOptionsPanel_.getPrecision());
		settings.setReportPrinter(miscOptionsPanel_.getPrinterName());
		
			
		
		 // chart colors                                   
        ChartColorItem item  = colorOptionsPanel_.getAirMass();
        settings.setAirMassAreaColor(item.getAreaColor());
        settings.setAirMassLineColor(item.getPerimeterColor());
        settings.setAirMassLabelColor(item.getLabelsColor());
                        
        item  = colorOptionsPanel_.getFli();
        settings.setFliAreaColor(item.getAreaColor());
        settings.setFliLineColor(item.getPerimeterColor());
        settings.setFliLabelColor(item.getLabelsColor());
               
        item  = colorOptionsPanel_.getMoonDist();
        settings.setMoonDistAreaColor(item.getAreaColor());
        settings.setMoonDistLineColor(item.getPerimeterColor());
        settings.setMoonDistLabelColor(item.getLabelsColor());
       
        item  = colorOptionsPanel_.getExecTime();
        settings.setObexecAreaColor(item.getAreaColor());
        settings.setObexecLineColor(item.getPerimeterColor());
        settings.setObexecLabelColor(item.getLabelsColor());
       
        item  = colorOptionsPanel_.getSun();
        settings.setSunAreaColor(item.getAreaColor());
        settings.setSunLineColor(item.getPerimeterColor());
        settings.setSunLabelColor(item.getLabelsColor());
              
        item  = colorOptionsPanel_.getTimeInterval();
        settings.setObTimeIntervalAreaColor(item.getAreaColor());
        settings.setObTimeIntervalLineColor(item.getPerimeterColor());
        settings.setObTimeIntervalLabelColor(item.getLabelsColor());                
        
                
        item  = colorOptionsPanel_.getSidereal();        
        settings.setObSiderealTimeIntervalAreaColor(item.getAreaColor());
        settings.setObSiderealTimeIntervalLineColor(item.getPerimeterColor());
        settings.setObSiderealTimeIntervalLabelColor(item.getLabelsColor()); 
		
		
		
		
	}
	

	
	/**
	 * Load the settings from the options and update the display widgets
	 * @param the settings/options object.
	 */
	
	public void updateGUI(VisiPlotOptions settings) {
		
		
		// update the plotOptionsPanel						
		plotOptionsPanel_.setStartTime(
				settings.getStartTimeISO());
		plotOptionsPanel_.setEndTime(
				settings.getEndTimeISO());
		plotOptionsPanel_.setAirMassConstraint(
				settings.getVisicalcOptions().calcAirmass()  ,
				settings.getAirmassValue());
		
		
		plotOptionsPanel_.setExecTimeConstraint(
				settings.getVisicalcOptions().calcExecTime());
		
		
		plotOptionsPanel_.setTimeIntervalConstraint(
				settings.getVisicalcOptions().calcTimeInterval());
		
		plotOptionsPanel_.setSiderealTimeIntervalConstraint(
				settings.getVisicalcOptions().calcSiderealTimeInterval());
		
		plotOptionsPanel_.setFLIConstraint(
				settings.getVisicalcOptions().calcFli(),
				settings.getFliValue());
		
		plotOptionsPanel_.setGroupIdentOBsConstraint(
				settings.isGroupOBs());
		
		plotOptionsPanel_.setMoonDistConstraint(
				settings.getVisicalcOptions().calcMoonDist(),
				settings.getMoonDistanceValue());
		
		int sunConstraint = new Double(settings.getVisicalcOptions().getSunConstraint()).intValue();
		
		plotOptionsPanel_.setSunConstraint(
				settings.getVisicalcOptions().calcSun(),
				sunConstraint+"");
		
		plotOptionsPanel_.setZenithConstraint(
				settings.getVisicalcOptions().calcZenithInvisibility()			
		);
			
			
		
		// update the headerOptionsPanel
		headerOptionsPanel_.setDec(settings.reportShowDec());
		headerOptionsPanel_.setExecTime(settings.reportShowExecTime());
		headerOptionsPanel_.setStartTime(settings.reportShowStartTime());
		headerOptionsPanel_.setFLI(settings.reportShowFLI());
		headerOptionsPanel_.setAirMass(settings.reportShowAirMass());
		headerOptionsPanel_.setMoonDistance(settings.reportShowMoonDist());
		headerOptionsPanel_.setOBId(settings.reportShowOBId());
		headerOptionsPanel_.setProgId(settings.reportShowProgId());
		headerOptionsPanel_.setRA(settings.reportShowRA());
		headerOptionsPanel_.setRankClass(settings.reportShowRank());
		headerOptionsPanel_.setSeeing(settings.reportShowSeeing());
		headerOptionsPanel_.setSkyTrans(settings.reportShowSkyTrans());
		headerOptionsPanel_.setSortOrder(settings.getSortPlot());
		
		//Plotting colors.
		colorOptionsPanel_.updateSettings(settings);
		colorOptionsPanel_.setBorder(
				new EmptyBorder(new Insets(5, 5, 5, 5)));		
		
		
		// update the misc options panel
		int prec = VisiplotOptionsHandler.defaultPrecision;
		try {
			prec = settings.getInterval();
		} catch (NumberFormatException e) {
			prec = VisiplotOptionsHandler.defaultPrecision;
		}
		miscOptionsPanel_.setPrecision(prec);
		miscOptionsPanel_.setPrinterName(settings.getReportPrinter());

	}
	

	
	private static JFileChooser getFileChooser() {
		if (fileChooser_ == null) {
			fileChooser_ = new SwingFileChooser();
		}
		return fileChooser_;
	}
	
	/**
	 * Check all entries in the sub panels
	 * @return a string error code, empty string indicates no errors.
	 */
	
	public String checkEntries() {
		StringBuffer rc = new StringBuffer();
		String msg = plotOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		msg = headerOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		msg = miscOptionsPanel_.checkEntries();
		if (msg.length() > 0) {
			rc.append(msg+"\n");
		}
		return rc.toString();
	}
	
	
	
	
	
}
